@@ -52,9 +52,11 @@ static PyObject *RetInt(int ok,int result) /* status/integer return */
   return(Py_BuildValue("(ii)",!ok,result));
 }
 
+typedef void(*CObjectDestruct)(void*) ;
+
 static PyObject *_new(PyObject *self,      PyObject *args)
 {
-  return(PyCObject_FromVoidPtr((void*)ChampNew(),(void*)(void*)ChampFree));
+  return(PyCObject_FromVoidPtr((void*)ChampNew(),(CObjectDestruct)ChampFree));
 }
 
 static PyObject *_memory_dump(PyObject *self,      PyObject *args)
@@ -119,6 +121,21 @@ static PyObject *pattern_orient_bonds(PyObject *self,      PyObject *args)
   return(RetStatus(ok));
 }
 
+static PyObject *pattern_detect_chirality(PyObject *self,      PyObject *args)
+{
+  int ok=true;
+  int int1;
+  PyObject *O;
+  CChamp *I;
+  ok = PyArg_ParseTuple(args,"Oi",&O,&int1);
+  ok = PyCObject_Check(O);
+  if(ok) {
+    I = PyCObject_AsVoidPtr(O);
+    ChampDetectChirality(I,int1);
+  }
+  return(RetStatus(ok));
+}
+
 
 static PyObject *pattern_generalize(PyObject *self,      PyObject *args)
 {
@@ -381,6 +398,96 @@ static PyObject *pattern_get_tags(PyObject *self,      PyObject *args)
   return(RetObj(ok,result));
 }
 
+static PyObject *pattern_get_ext_indices_with_tags(PyObject *self,      PyObject *args)
+{
+  int ok=true;
+  int int1,ai,bi;
+  PyObject *result = NULL;
+  PyObject *O,*l1,*l2,*l3,*l4;
+  CChamp *I;
+  ListPat *pat;
+  ListAtom *at;
+  ListBond *bd;
+  int a,b,c;
+  int n_atom,n_bond;
+  int bit_mask;
+  int bit_no;
+  int n_bits;
+
+  ok = PyArg_ParseTuple(args,"Oi",&O,&int1);
+  ok = PyCObject_Check(O);
+  if(ok) {
+    I = PyCObject_AsVoidPtr(O);
+    pat = I->Pat+int1;
+    n_atom = ListLen(I->Atom,pat->atom);
+    ai = pat->atom;
+    l1 = PyList_New(n_atom);
+    for(a=0;a<n_atom;a++) {
+      at = I->Atom + ai;
+
+      n_bits = 0;
+      bit_mask = at->tag;
+      while(bit_mask) { /* count bits */
+        if(bit_mask&0x1) 
+          n_bits++;
+        bit_mask=((bit_mask>>1)&0x7FFFFFFF);
+      }
+      l4 = PyList_New(2);
+      l3 = PyList_New(n_bits);
+      bit_no = 0;
+      bit_mask = at->tag;
+      c=0;
+      for(bit_no=0;bit_no<32;bit_no++) {
+        if(bit_mask&0x1) {
+          PyList_SetItem(l3,c,PyInt_FromLong(bit_no));
+          c++;
+        }
+        bit_mask=((bit_mask>>1)&0x7FFFFFFF);
+      }
+      PyList_SetItem(l1,a,l4);
+      PyList_SetItem(l4,0,PyInt_FromLong(at->ext_index));
+      PyList_SetItem(l4,1,l3);
+      ai = at->link;
+    }
+
+    n_bond = ListLen(I->Bond,pat->bond);
+    l2 = PyList_New(n_bond);
+    bi = pat->bond;
+    for(b=0;b<n_bond;b++) {
+      bd = I->Bond + bi;
+      n_bits = 0;
+      bit_mask = bd->tag;
+      while(bit_mask) { /* count bits */
+        if(bit_mask&0x1) 
+          n_bits++;
+        bit_mask=((bit_mask>>1)&0x7FFFFFFF);
+      }
+      l4 = PyList_New(2);
+      l3 = PyList_New(n_bits);
+      bit_no = 0;
+      bit_mask = bd->tag;
+      c=0;
+      for(bit_no=0;bit_no<32;bit_no++) {
+        if(bit_mask&0x1) {
+          PyList_SetItem(l3,c,PyInt_FromLong(bit_no));
+          c++;
+        }
+        bit_mask=((bit_mask>>1)&0x7FFFFFFF);
+      }
+      PyList_SetItem(l2,b,l4);
+      PyList_SetItem(l4,0,PyInt_FromLong(bd->ext_index));
+      PyList_SetItem(l4,1,l3);
+
+      bi = bd->link;
+    }
+
+    result = PyList_New(2);
+    PyList_SetItem(result,0,l1);
+    PyList_SetItem(result,1,l2);
+  }
+  return(RetObj(ok,result));
+}
+
 static PyObject *pattern_get_tag_masks(PyObject *self,      PyObject *args)
 {
   int ok=true;
@@ -530,20 +637,21 @@ static PyObject *list_get_pattern_indices(PyObject *self,      PyObject *args)
   return(RetObj(ok,result));
 }
 
-static PyObject *get_pattern_string(PyObject *self,      PyObject *args)
+static PyObject *pattern_get_string(PyObject *self,      PyObject *args)
 {
   int ok=true;
   PyObject *result = NULL;
   PyObject *O;
   int pat_index;
+  int mode;
   CChamp *I;
   char *smi;
 
-  ok = PyArg_ParseTuple(args,"Oi",&O,&pat_index);
+  ok = PyArg_ParseTuple(args,"Oii",&O,&pat_index,&mode);
   ok = PyCObject_Check(O);
   if(ok) {
     I = PyCObject_AsVoidPtr(O);
-    smi = ChampPatToSmiVLA(I,pat_index,NULL);
+    smi = ChampPatToSmiVLA(I,pat_index,NULL,mode);
     result = PyString_FromString(smi);
     vla_free(smi);
   }
@@ -598,6 +706,35 @@ static PyObject *pattern_get_atom_symbols(PyObject *self,      PyObject *args)
   return(RetObj(ok,result));
 }
 
+static PyObject *pattern_get_atom_names(PyObject *self,      PyObject *args)
+{
+  int ok=true;
+  int int1,ai;
+  PyObject *result = NULL;
+  PyObject *O;
+  CChamp *I;
+  ListPat *pat;
+  ListAtom *at;
+  int a;
+  int n_atom;
+
+  ok = PyArg_ParseTuple(args,"Oi",&O,&int1);
+  ok = PyCObject_Check(O);
+  if(ok) {
+    I = PyCObject_AsVoidPtr(O);
+    pat = I->Pat+int1;
+    n_atom = ListLen(I->Atom,pat->atom);
+    ai = pat->atom;
+    result = PyList_New(n_atom);
+    for(a=0;a<n_atom;a++) {
+      at = I->Atom + ai;
+      PyList_SetItem(result,a,PyString_FromString(at->name));
+      ai = at->link;
+    }
+  }
+  return(RetObj(ok,result));
+}
+
 
 static PyObject *list_get_pattern_strings(PyObject *self,      PyObject *args)
 {
@@ -623,7 +760,7 @@ static PyObject *list_get_pattern_strings(PyObject *self,      PyObject *args)
     i = list_index;
     c = 0;
     while(i) {
-      smi = ChampPatToSmiVLA(I,I->Int[i].value,smi);
+      smi = ChampPatToSmiVLA(I,I->Int[i].value,smi,0);
       str = PyString_FromString(smi);
       PyList_SetItem(result,c,str);
       i = I->Int[i].link;
@@ -761,7 +898,10 @@ static PyObject *match_1v1_map(PyObject *self,      PyObject *args)
       targ = PyList_New(pair_cnt);
       j = mat->atom;
       for(b=0;b<pair_cnt;b++) {
-        /*        printf("%2d %2d\n",I->Int2[j].value[0],I->Int2[j].value[1]);*/
+        /*
+                printf("%2d %2d %4d %4d\n",I->Int2[j].value[0],I->Int2[j].value[1],
+                       I->Atom[I->Int2[j].value[0]].index,
+                       I->Atom[I->Int2[j].value[1]].index);*/
         PyList_SetItem(tmpl,b,PyInt_FromLong(I->Atom[I->Int2[j].value[0]].index));
         PyList_SetItem(targ,b,PyInt_FromLong(I->Atom[I->Int2[j].value[1]].index));
         j = I->Int2[j].link;
@@ -873,7 +1013,6 @@ static PyObject *insert_model(PyObject *self,      PyObject *args)
 static PyMethodDef champ_methods[] = {
   {"new",	                     _new,                   METH_VARARGS },
   {"memory_dump",               _memory_dump,           METH_VARARGS },
-  {"get_pattern_string",        get_pattern_string,       METH_VARARGS },
   {"insert_pattern_string",     insert_pattern_string,          METH_VARARGS },
   {"insert_model",              insert_model,           METH_VARARGS },
   {"pattern_free",              pattern_free,          METH_VARARGS },
@@ -881,11 +1020,15 @@ static PyMethodDef champ_methods[] = {
   {"pattern_get_cycle",   pattern_get_cycle,       METH_VARARGS },
   {"pattern_get_class",   pattern_get_class,       METH_VARARGS },
   {"pattern_get_codes",   pattern_get_codes,       METH_VARARGS },
+  {"pattern_get_string",        pattern_get_string,       METH_VARARGS },
   {"pattern_get_tags",    pattern_get_tags,       METH_VARARGS },
   {"pattern_get_tag_masks",    pattern_get_tag_masks,       METH_VARARGS },
   {"pattern_get_atom_symbols",  pattern_get_atom_symbols,     METH_VARARGS },
+  {"pattern_get_atom_names",  pattern_get_atom_names,     METH_VARARGS },
+  {"pattern_get_ext_indices_with_tags",  pattern_get_ext_indices_with_tags, METH_VARARGS },
   {"pattern_dump",  pattern_dump,     METH_VARARGS },
   {"pattern_orient_bonds",   pattern_orient_bonds,       METH_VARARGS },
+  {"pattern_detect_chirality",   pattern_detect_chirality,       METH_VARARGS },
   {"pattern_generalize",   pattern_generalize,     METH_VARARGS },
   {"list_prepend_pattern_strings",  list_prepend_pattern_strings, METH_VARARGS },
   {"list_prepend_pattern_index",  list_prepend_pattern_index, METH_VARARGS },
@@ -898,6 +1041,7 @@ static PyMethodDef champ_methods[] = {
   {"match_1v1_n",               match_1v1_n,             METH_VARARGS},
   {"match_1vN_n",               match_1vN_n,              METH_VARARGS },
   {"match_Nv1_n",               match_Nv1_n,              METH_VARARGS },
+  /*  {"map_1v1_to_indexed_strings",  map_1v1_to_indexed_strings, METH_VARARGS },*/
   {"exact_1vN_n",               exact_1vN_n,              METH_VARARGS },
 
   {NULL,		                    NULL}     /* sentinel */        
