@@ -63,6 +63,7 @@ typedef struct {
   EdgeRec *edge;
   int nEdge;
   MapType *map;
+  MapCache map_cache;
   LinkType *link;
   int nLink;
   int N;
@@ -180,7 +181,7 @@ static int *TriangleMakeStripVLA(float *v,float *vn,int n)
 				  tFlag[t0]=true;
 				  c++;
 				  done=false;
-              if(c==1) { /* make sure vertices follow standard convention */
+              if((c==1)||(c==2)) { /* make sure vertices follow standard convention */
                 
                 /* sum normal */
                 tn0 = vn+(*(s-3))*3;
@@ -197,16 +198,26 @@ static int *TriangleMakeStripVLA(float *v,float *vn,int n)
                 subtract3f(v0,v1,vt1);
                 subtract3f(v0,v2,vt2);
                 cross_product3f(vt1,vt2,xtn);
-                
-                /* reorder triangle if necessary */
 
-                if(dot_product3f(xtn,tn)<0.0) {
-                  tmp_int=*(s-3);
-                  *(s-3)=*(s-2);
-                  *(s-2)=tmp_int;
+                if(c&0x1) {
+                  /* reorder triangle if necessary */
+
+                  if(dot_product3f(xtn,tn)<0.0) {
+                    tmp_int=*(s-3);
+                    *(s-3)=*(s-2);
+                    *(s-2)=tmp_int;
+                  }
+                } else {
+                  if(dot_product3f(xtn,tn)>0.0) { /* if we're blowing the right hand rule
+                                                     on the second triangle, then 
+                                                     terminate this strip and try again */
+                    tFlag[t0]=false;
+                    c--;
+                    s--;
+                    break;
+                  }
                 }
               }
-
 				}
 				if(!c)
 				  s=sc;
@@ -275,19 +286,21 @@ static void TriangleAdjustNormals(float *v,float *vn,int n)
 {
   TriangleSurfaceRec *I=&TriangleSurface;
   /* points all normals to the average of the intersecting triangles in order to maximum surface smoothness */
-  float *tNorm = NULL;
+  float *tNorm = NULL,*tWght;
   int *vFlag=NULL;
-  float *v0,*v1,*v2,*tn,vt1[3],vt2[3],*vn0,*tn0,*tn1,*tn2;
+  float *v0,*v1,*v2,*tn,vt1[3],vt2[3],*vn0,*tn0,*tn1,*tn2,*tw;
   int a,*t,i0,i1,i2;
-
+  float tmp[3];
   tNorm=Alloc(float,3*I->nTri);
+  tWght=Alloc(float,I->nTri);
   vFlag=Alloc(int,n);
   for(a=0;a<n;a++) {
 	 vFlag[a]=0;
   }
-  /* first, calculate and store all triangle normals */
+  /* first, calculate and store all triangle normals & weights */
   t=I->tri;
   tn=tNorm;
+  tw=tWght;
   for(a=0;a<I->nTri;a++) {
 	 vFlag[t[0]]=1;
 	 vFlag[t[1]]=1;
@@ -298,6 +311,7 @@ static void TriangleAdjustNormals(float *v,float *vn,int n)
 	 subtract3f(v1,v0,vt1);
 	 subtract3f(v2,v0,vt2);
 	 cross_product3f(vt1,vt2,tn);
+    *(tw++) = length3f(tn); /* store weight */
 	 normalize3f(tn);
 	 tn+=3;
   }
@@ -313,18 +327,21 @@ static void TriangleAdjustNormals(float *v,float *vn,int n)
   
   /* sum */
   tn=tNorm;
+  tw=tWght;
   t=I->tri;
   for(a=0;a<I->nTri;a++) {
 	 i0 = *(t++);
 	 i1 = *(t++);
 	 i2 = *(t++);
+    scale3f(tn,*tw,tmp);
 	 tn0 = vn+i0*3;
 	 tn1 = vn+i1*3;
 	 tn2 = vn+i2*3;
-	 add3f(tn,tn0,tn0);
-	 add3f(tn,tn1,tn1);
-	 add3f(tn,tn2,tn2);
+	 add3f(tmp,tn0,tn0);
+	 add3f(tmp,tn1,tn1);
+	 add3f(tmp,tn2,tn2);
 	 tn+=3;
+    tw++;
   }
   /* normalize */
   vn0=vn;
@@ -334,6 +351,7 @@ static void TriangleAdjustNormals(float *v,float *vn,int n)
 	 vn0+=3;
   }
   FreeP(vFlag);
+  FreeP(tWght);
   FreeP(tNorm);
 }
 
@@ -446,6 +464,9 @@ static void TriangleAdd(int i0,int i1,int i2,float *tNorm,float *v,float *vn)
   int s01,s12,s02,it;
   float *v0,*v1,*v2,*n0,*n1,*n2,*ft;
   float vt[3],vt1[3],vt2[3],vt3[3];
+  int e1,e2,e3,h,j,k,l;
+  MapType *map = I->map;
+  MapCache *cache = &I->map_cache;
 
   v0 = v+3*i0;
   v1 = v+3*i1;
@@ -454,6 +475,32 @@ static void TriangleAdd(int i0,int i1,int i2,float *tNorm,float *v,float *vn)
   n1 = vn+3*i1;
   n2 = vn+3*i2;
 
+  /* mark this quadrant as visited so that further activity in this
+     quadrant won't prolong the rendering process indefinitely */
+  
+  
+  MapLocus(map,v0,&h,&k,&l);
+  e1=*(MapEStart(map,h,k,l));
+  
+  if(e1) {
+    j=map->EList[e1];
+    MapCache(cache,j);
+  }
+  
+  MapLocus(map,v1,&h,&k,&l);
+  e2=*(MapEStart(map,h,k,l));
+  if(e2&&(e1!=e2)) {
+    j=map->EList[e2];
+    MapCache(cache,j);
+  }
+  
+  MapLocus(map,v2,&h,&k,&l);
+  e3=*(MapEStart(map,h,k,l));
+  if(e3&&(e3!=e2)&&(e3!=e1)) {
+    j=map->EList[e3];
+    MapCache(cache,j);
+  }
+
   /* make sure the triangle obeys the right hand rule */
 
   subtract3f(v1,v0,vt1);
@@ -567,7 +614,9 @@ static void TriangleBuildObvious(int i1,int i2,float *v,float *vn,int n)
   float dif,minDist,d1,d2,dp;
   int flag;
   int used = -1;
-
+  float maxDot,dot,dot1,dot2;
+  const float _plus = R_SMALL4, _0=0.0F;
+  const float _5 = 0.5F;
   /*  PRINTFD(FB_Triangle)
       " TriangleBuildObvious-Debug: entered: i1=%d i2=%d n=%d\n",i1,i2,n
       ENDFD;*/
@@ -583,8 +632,10 @@ static void TriangleBuildObvious(int i1,int i2,float *v,float *vn,int n)
   if(s12>0) used = I->edge[s12].vert3;
   if(s12>=0) {
     minDist = MAXFLOAT;
+    maxDot = _plus;
     i0=-1;
     v1=v+i1*3; v2=v+i2*3;
+    n1 = vn+3*i1; n2 = vn+3*i2;							 
     MapLocus(map,v1,&h,&k,&l);
 	 i=*(MapEStart(map,h,k,l));
     if(i) {
@@ -598,8 +649,25 @@ static void TriangleBuildObvious(int i1,int i2,float *v,float *vn,int n)
             dif= ( d2 > d1 ? d2 : d1 );
 				if(dif<minDist)
 				  {
-					 minDist = dif;
-					 i0=j; 
+                n0 = vn + 3*j;
+                dot1 = dot_product3f(n0,n1);
+                dot2 = dot_product3f(n0,n2);
+                dot = dot1 + dot2;
+                if((dif/minDist)<_5) {
+                  minDist = dif;
+                  maxDot = dot;
+                  i0=j; 
+                } else if((dot>_0)&&(dot1>_0)&&(dot2>_0)) {
+                  if((i0<0)||(dot>maxDot)) {
+                    minDist = dif;
+                    maxDot = dot;
+                    i0=j; 
+                  } else if( _5*(dif/minDist) < (dot/maxDot)) {
+                    minDist = dif;
+                    maxDot = dot;
+                    i0=j; 
+                  }
+                }
 				  }
 			 }
 		  j=map->EList[i++];
@@ -735,14 +803,19 @@ static void TriangleBuildSecondPass(int i1,int i2,float *v,float *vn,int n)
   float dif,minDist,d1,d2,dp;
   int flag;
   int used = -1;
+  float dot,dot1,dot2,maxDot;
+  const float _plus = R_SMALL4, _0=0.0F;
+  const float _5 = 0.5F;
 
   map=I->map;
   s12 = TriangleEdgeStatus(i1,i2);
   if(s12>0) used = I->edge[s12].vert3;
   if(s12>=0) {
     minDist = MAXFLOAT;
+    maxDot = _plus;
     i0=-1;
     v1=v+i1*3; v2=v+i2*3;
+    n1 = vn+3*i1; n2 = vn+3*i2;							 
     MapLocus(map,v1,&h,&k,&l);
 	 i=*(MapEStart(map,h,k,l));
     if(i) {
@@ -752,13 +825,163 @@ static void TriangleBuildSecondPass(int i1,int i2,float *v,float *vn,int n)
 			 /* eliminate closed vertices from consideration - where vertactive is 0 */
           {
             v0 = v+3*j;
+            n0 = vn+3*j;
             d1 = (float)diff3f(v0,v1);
             d2 = (float)diff3f(v0,v2);
             dif= ( d2 > d1 ? d2 : d1 );
 				if(dif<minDist)
 				  {
-					 minDist = dif;
-					 i0=j; 
+                dot1 = dot_product3f(n0,n1);
+                dot2 = dot_product3f(n0,n2);
+                dot = dot1 + dot2;
+                if((dif/minDist)<_5) {
+                  minDist = dif;
+                  maxDot = dot;
+                  i0=j; 
+                } else if((dot>_0)&&(dot1>_0)&&(dot2>_0)) {
+                  if((i0<0)||(dot>maxDot)) {
+                    minDist = dif;
+                    maxDot = dot;
+                    i0=j; 
+                  } else if( _5*(dif/minDist) < (dot/maxDot)) {
+                    maxDot = dot;
+                    minDist = dif;
+                    i0=j; 
+                  }
+                }
+				  }
+			 }
+		  j=map->EList[i++];
+		}
+		if(i0>=0) {
+		  s01 = TriangleEdgeStatus(i0,i1); s02 = TriangleEdgeStatus(i0,i2);
+		  if(I->vertActive[i0]>0) {
+			 if(!((s01>0)||(s02>0)))
+				i0=-1; /* don't allow non-adjacent joins to active vertices */
+		  }
+		}
+		if(i0>=0) {
+		  v0 = v+3*i0;
+		  flag=false;
+		  if(I->vertActive[i0]) {
+			 if((s01>=0)&&(s02>=0)) flag=true;
+			 if(flag) { /* are all normals pointing in generally the same direction? */
+				n0 = vn+3*i0; n1 = vn+3*i1; n2 = vn+3*i2;							 
+				add3f(n0,n1,vt1);
+				add3f(n2,vt1,vt2);
+				normalize3f(vt2);
+				if(((dot_product3f(n0,vt2))<0.1)||
+               ((dot_product3f(n1,vt2))<0.1)||
+               ((dot_product3f(n2,vt2))<0.1)) flag=false;
+            /* modified 010916 to effect workaround of apparent bug in GCC's optimizer */
+			 } /*printf("pass normal sums %i\n",flag);*/
+			 if(flag) { /* does the sum of the normals point in the same direction as the triangle? */
+				subtract3f(v1,v0,vt3);
+				subtract3f(v2,v0,vt4);
+				cross_product3f(vt3,vt4,tNorm); 
+				normalize3f(tNorm); 							 
+				dp = dot_product3f(vt2,tNorm);
+				if(dp<0) scale3f(tNorm,-1.0F,tNorm);
+				if(fabs(dp)<0.1) flag = false;
+			 } /*printf("pass tNorm  %i\n",flag);*/
+
+			 if(flag) {
+				if(s12>0) if(dot_product3f(I->vNormal+s12*3,tNorm)<0.1) flag=false; 
+				if(s01>0) if(dot_product3f(I->vNormal+s01*3,tNorm)<0.1) flag=false; 
+				if(s02>0) if(dot_product3f(I->vNormal+s02*3,tNorm)<0.1) flag=false; 
+			 } /*printf("pass compare tNorm %i\n",flag);*/
+			 if(flag) { /* are all the Blocking vectors pointing outward, and are the triangle normals consistent? */
+				if(s12>0) {
+				  i4 = I->edge[s12].vert3;
+				  v4=v+i4*3;
+				  subtract3f(v0,v1,vt1);
+				  subtract3f(v4,v1,vt2);
+				  subtract3f(v1,v2,vt);
+				  normalize3f(vt);
+				  remove_component3f(vt1,vt,vt3);
+				  remove_component3f(vt2,vt,vt4);
+				  normalize3f(vt3);
+				  normalize3f(vt4);
+				  if(dot_product3f(vt3,vt4)>0.0) flag=false;
+				}			 
+				if(s01>0) {
+				  i4 = I->edge[s01].vert3;
+				  v4=v+i4*3;
+				  subtract3f(v2,v0,vt1);
+				  subtract3f(v4,v0,vt2);
+				  subtract3f(v0,v1,vt);
+				  normalize3f(vt);
+				  remove_component3f(vt1,vt,vt3);
+				  remove_component3f(vt2,vt,vt4);
+				  normalize3f(vt3);
+				  normalize3f(vt4);
+				  if(dot_product3f(vt3,vt4)>0.0) flag=false;
+				}
+				if(s02>0) {
+				  i4 = I->edge[s02].vert3;
+				  v4=v+i4*3;
+				  subtract3f(v1,v0,vt1);
+				  subtract3f(v4,v0,vt2);
+				  subtract3f(v0,v2,vt);
+				  normalize3f(vt);
+				  remove_component3f(vt1,vt,vt3);
+				  remove_component3f(vt2,vt,vt4);
+				  normalize3f(vt3);
+				  normalize3f(vt4);
+				  if(dot_product3f(vt3,vt4)>0.0) flag=false;
+				}
+			 } /*printf("pass blocking %i\n",flag);*/
+		  }
+		  if(flag) TriangleAdd(i0,i1,i2,tNorm,v,vn);
+		}
+	 }
+  }
+}
+
+static void TriangleBuildSecondSecondPass(int i1,int i2,float *v,float *vn,int n,float cutoff)
+{
+
+  /* in this version, the closest active point is tried.  Closed points
+	  are skipped. */
+
+  TriangleSurfaceRec *I=&TriangleSurface;
+  MapType *map;
+  float *v1,*v2,*v0,*v4,vt[3],vt1[3],vt2[3],vt3[3],vt4[3],*n0,*n1,*n2,tNorm[3];
+  int i0,s01=0,s02=0,s12,i,j,h,k,l,i4;
+  float dif,minDist,d1,d2,dp;
+  int flag;
+  int used = -1;
+  float dot;
+  const float _5 = 0.5;
+
+  map=I->map;
+  s12 = TriangleEdgeStatus(i1,i2);
+  if(s12>0) used = I->edge[s12].vert3;
+  if(s12>=0) {
+    minDist = MAXFLOAT;
+    i0=-1;
+    v1=v+i1*3; v2=v+i2*3;
+    n1 = vn+3*i1; n2 = vn+3*i2;							 
+    MapLocus(map,v1,&h,&k,&l);
+	 i=*(MapEStart(map,h,k,l));
+    if(i) {
+      j=map->EList[i++];
+      while(j>=0) {
+        if((j!=i1)&&(j!=i2)&&(j!=used)&&(I->vertActive[j])) 
+			 /* eliminate closed vertices from consideration - where vertactive is 0 */
+          {
+            v0 = v+3*j;
+            n0 = vn+3*j;
+            d1 = (float)diff3f(v0,v1);
+            d2 = (float)diff3f(v0,v2);
+            dif= ( d2 > d1 ? d2 : d1 );
+				if(dif<minDist)
+				  {
+                dot = dot_product3f(n0,n1) + dot_product3f(n0,n2);
+                if((dot>cutoff)||((dif/minDist)<_5)) {
+                  minDist = dif;
+                  i0=j; 
+                }
 				  }
 			 }
 		  j=map->EList[i++];
@@ -847,6 +1070,8 @@ static void TriangleBuildSecondPass(int i1,int i2,float *v,float *vn,int n)
   }
 }
 
+
+
 static void TriangleBuildSingle(int i1,int i2,float *v,float *vn,int n)
 {
 
@@ -965,9 +1190,7 @@ static void TriangleBuildSingle(int i1,int i2,float *v,float *vn,int n)
 
 static void TriangleBuildThirdPass(int i1,int i2,float *v,float *vn,int n)
 {
-  /* This routine fills in triangles surrounded by three active edges
-
-	*/
+  /* This routine fills in triangles surrounded by three active edges */
 
   TriangleSurfaceRec *I=&TriangleSurface;
   MapType *map;
@@ -1090,18 +1313,15 @@ static void FollowActives(float *v,float *vn,int n,int mode)
 {
   TriangleSurfaceRec *I=&TriangleSurface;
   int i1,i2;
-  int cnt;
   
   PRINTFD(FB_Triangle)
     " TriangleFollowActives-Debug: entered: n=%6d     mode=%d\n TriangleFollowActives-Debug:       nTri=%6d nActive=%6d\n",
     n,mode,I->nTri,I->nActive
     ENDFD;
 
-  cnt = (int)SettingGet(cSetting_max_triangles);
-  
   OrthoBusyFast((I->N*3)+I->nTri,I->N*5); /* 3/5 to 4/5 */
 
-  while(I->nActive&&(I->nTri<cnt)) {
+  while(I->nActive) {
     I->nActive--;
     i1 = I->activeEdge[I->nActive*2];
     i2 = I->activeEdge[I->nActive*2+1];
@@ -1113,9 +1333,12 @@ static void FollowActives(float *v,float *vn,int n,int mode)
 		TriangleBuildSecondPass(i1,i2,v,vn,n);
 		break;
 	 case 2:
+		TriangleBuildSecondSecondPass(i1,i2,v,vn,n,0.0F);
+		break;
+	 case 4:
 		TriangleBuildThirdPass(i1,i2,v,vn,n);
 		break;
-	 case 3:
+	 case 5:
 		TriangleBuildLast(i1,i2,v,vn,n);
 		break;
 	 }
@@ -1128,24 +1351,32 @@ static void FollowActives(float *v,float *vn,int n,int mode)
 
 }
 
-static void TriangleFill(float *v,float *vn,int n)
+static void TriangleFill(float *v,float *vn,int n,int first_time)
 {
   TriangleSurfaceRec *I=&TriangleSurface;
   int lastTri,lastTri2,lastTri3;
   int a,i,j,h,k,l;
-  float dif,minDist,*v0;
+  float dif,minDist,*v0,*n0,*n1;
   int i1,i2=0;
+  int n_pass =0;
+  int first_vert=0,first_vert_used=0;
+
   MapType *map;
+  MapCache *cache;
 
   PRINTFD(FB_Triangle)
     " TriangleFill-Debug: entered: n=%d\n",n
     ENDFD;
   
   map=I->map;
+  cache = &I->map_cache;
 
   lastTri3=-1;
   while(lastTri3!=I->nTri) {
 	 lastTri3=I->nTri;
+    n_pass++;
+    if(n_pass>(int)SettingGet(cSetting_triangle_max_passes))
+      break;
 
 	 I->nActive=0;
     while((!I->nActive)&&(I->nTri==lastTri3))
@@ -1157,10 +1388,13 @@ static void TriangleFill(float *v,float *vn,int n)
           if(!I->edgeStatus[a])
             {
               v0=v+a*3;
+              n0 = vn+3*a;
+
               MapLocus(map,v0,&h,&k,&l);
               i=*(MapEStart(map,h,k,l));
               if(i) {
                 j=map->EList[i++];
+                first_vert = j;
                 while(j>=0) {
                   if(j!=a) 
                     {
@@ -1169,17 +1403,34 @@ static void TriangleFill(float *v,float *vn,int n)
                         if(I->vertActive[a]==-1)
                           if(TriangleEdgeStatus(a,j)>=0) /* can we put a triangle here? */
                             {
-                              minDist = dif;
-                              i1=a;
-                              i2=j;
+                              n1 = vn + 3*j;
+                              if(dot_product3f(n0,n1)>0.5) { /* start with vertices pointing the same way */
+                                minDist = dif;
+                                i1=a;
+                                i2=j;
+                                first_vert_used = first_vert;
+                              }
                             }
                     }
                   j=map->EList[i++];
                 }
               }
             }
-	 
+        
         if(i1>=0) {
+
+          if(!MapCached(cache,first_vert_used)) {
+            MapCache(cache,first_vert_used);
+            if(first_time) {
+              n_pass=n_pass/2; 
+              /* if we've entered a new map quadrant then half the effective number of passes */
+
+              /* this is a very non-obvious way of making sure that we
+                 don't prematurely terminate when surfacing
+                 discontinuous surfaces that will always require many passes */
+            }
+          }
+
           if(I->vertActive[i1]<0) I->vertActive[i1]--;
           VLACheck(I->activeEdge,int,I->nActive*2+1);
           I->activeEdge[I->nActive*2] = i1;
@@ -1195,7 +1446,9 @@ static void TriangleFill(float *v,float *vn,int n)
             FollowActives(v,vn,n,0);
           }
         } else break;
-	 }
+      }
+
+
     PRINTFD(FB_Triangle)
       " TriangleFill-Debug: Follow actives 1 nTri=%d\n",I->nTri
       ENDFD;
@@ -1207,37 +1460,32 @@ static void TriangleFill(float *v,float *vn,int n)
 			 TriangleActivateEdges(a);
 		FollowActives(v,vn,n,1);
 	 }	 
-	 lastTri2=I->nTri-1;
-	 while(lastTri2!=I->nTri) {
-		lastTri2=I->nTri;
-		for(a=0;a<n;a++) 
-		  if(I->vertActive[a])
-			 {
-				TriangleActivateEdges(a);
-				if(I->nActive) {
+
+    lastTri2=I->nTri-1;
+    while(lastTri2!=I->nTri) {
+      lastTri2=I->nTri;
+      for(a=0;a<n;a++) 
+        if(I->vertActive[a])
+          {
+            TriangleActivateEdges(a);
+            if(I->nActive) {
               PRINTFD(FB_Triangle)
                 " TriangleFill-Debug: build single:     nTri=%d nActive=%d\n",I->nTri,I->nActive
                 ENDFD;
-				  I->nActive--;
-				  i1 = I->activeEdge[I->nActive*2];
-				  i2 = I->activeEdge[I->nActive*2+1];
-				  TriangleBuildSingle(i1,i2,v,vn,n);
+              I->nActive--;
+              i1 = I->activeEdge[I->nActive*2];
+              i2 = I->activeEdge[I->nActive*2+1];
+              TriangleBuildSingle(i1,i2,v,vn,n);
               PRINTFD(FB_Triangle)
                 " TriangleFill-Debug: follow actives 1: nTri=%d nActive=%d\n",I->nTri,I->nActive
                 ENDFD;
-				  FollowActives(v,vn,n,1);
-				}
-			 }
-	 }
+              FollowActives(v,vn,n,1);
+            }
+          }
+    }
+
     PRINTFD(FB_Triangle)
-                " TriangleFill-Debug: follow actives 2: nTri=%d nActive=%d\n",I->nTri,I->nActive
-      ENDFD;
-	 for(a=0;a<n;a++) 
-		if(I->vertActive[a])
-		  TriangleActivateEdges(a);
-	 FollowActives(v,vn,n,2);
-    PRINTFD(FB_Triangle)
-      " TriangleFill-Debug: follow actives 3: nTri=%d nActive=%d\n",I->nTri,I->nActive
+      " TriangleFill-Debug: Follow actives 1 nTri=%d\n",I->nTri
       ENDFD;
 	 lastTri=I->nTri-1;
 	 while(lastTri!=I->nTri) {
@@ -1245,7 +1493,52 @@ static void TriangleFill(float *v,float *vn,int n)
 		for(a=0;a<n;a++) 
 		  if(I->vertActive[a])
 			 TriangleActivateEdges(a);
-		FollowActives(v,vn,n,3); /* this is a sloppy, forcing tesselation */
+		FollowActives(v,vn,n,2);
+	 }	 
+
+    lastTri2=I->nTri-1;
+    while(lastTri2!=I->nTri) {
+      lastTri2=I->nTri;
+      for(a=0;a<n;a++) 
+        if(I->vertActive[a])
+          {
+            TriangleActivateEdges(a);
+            if(I->nActive) {
+              PRINTFD(FB_Triangle)
+                " TriangleFill-Debug: build single:     nTri=%d nActive=%d\n",I->nTri,I->nActive
+                ENDFD;
+              I->nActive--;
+              i1 = I->activeEdge[I->nActive*2];
+              i2 = I->activeEdge[I->nActive*2+1];
+              TriangleBuildSingle(i1,i2,v,vn,n);
+              PRINTFD(FB_Triangle)
+                " TriangleFill-Debug: follow actives 2: nTri=%d nActive=%d\n",I->nTri,I->nActive
+                ENDFD;
+              FollowActives(v,vn,n,2);
+            }
+          }
+    }
+    
+    PRINTFD(FB_Triangle)
+                " TriangleFill-Debug: follow actives 4: nTri=%d nActive=%d\n",I->nTri,I->nActive
+      ENDFD;
+
+	 for(a=0;a<n;a++) 
+		if(I->vertActive[a])
+		  TriangleActivateEdges(a);
+	 FollowActives(v,vn,n,4);
+
+    PRINTFD(FB_Triangle)
+      " TriangleFill-Debug: follow actives 5: nTri=%d nActive=%d\n",I->nTri,I->nActive
+      ENDFD;
+
+	 lastTri=I->nTri-1;
+	 while(lastTri!=I->nTri) {
+		lastTri=I->nTri;
+		for(a=0;a<n;a++) 
+		  if(I->vertActive[a])
+			 TriangleActivateEdges(a);
+		FollowActives(v,vn,n,5); /* this is a sloppy, forcing tesselation */
 	 }
   }
     PRINTFD(FB_Triangle)
@@ -1363,7 +1656,7 @@ static void TriangleFixProblems(float *v,float *vn,int n)
 		}
 		
 		TriangleAdjustNormals(v,vn,n);
-		TriangleFill(v,vn,n);
+		TriangleFill(v,vn,n,false);
 
 	 }
   FreeP(vFlag);
@@ -1488,7 +1781,6 @@ int *TrianglePointsToSurface(float *v,float *vn,int n,float cutoff,int *nTriPtr,
   TriangleSurfaceRec *I=&TriangleSurface;
   MapType *map;
   int a;
-  int cnt;
 
   I->N=n;
   I->nActive = 0;
@@ -1508,7 +1800,8 @@ int *TrianglePointsToSurface(float *v,float *vn,int n,float cutoff,int *nTriPtr,
   I->map=MapNew(cutoff,v,n,extent);
   MapSetupExpress(I->map);
   map=I->map;
-  
+  MapCacheInit(&I->map_cache,map,0,0);
+
   I->edgeStatus = Alloc(int,n);
   for(a=0;a<n;a++) {
 	 I->edgeStatus[a]=0;
@@ -1524,7 +1817,7 @@ int *TrianglePointsToSurface(float *v,float *vn,int n,float cutoff,int *nTriPtr,
 	 I->vertWeight[a]=2;
   }
 
-  TriangleFill(v,vn,n);
+  TriangleFill(v,vn,n,true);
 
   if(Feedback(FB_Triangle,FB_Debugging)) {
     for(a=0;a<n;a++) 
@@ -1571,10 +1864,6 @@ NTestLine=0; for(a=0;a<n;a++)
 
   *(stripPtr) = TriangleMakeStripVLA(v,vn,n);
 
-  cnt = (int)SettingGet(cSetting_max_triangles);
-  if(I->nTri>cnt)
-	 I->nTri=cnt;
-
   (*nTriPtr)=I->nTri;
   VLAFreeP(I->activeEdge);
   VLAFreeP(I->link);
@@ -1583,6 +1872,7 @@ NTestLine=0; for(a=0;a<n;a++)
   FreeP(I->edgeStatus);
   FreeP(I->vertActive);
   FreeP(I->vertWeight);
+  MapCacheFree(&I->map_cache,0,0);
   MapFree(map);
   return(I->tri);
 }
