@@ -1,6 +1,7 @@
 #include"os_predef.h"
 #include"os_gl.h"
 
+#include<stdio.h>
 
 #ifdef _PYMOL_PRETEND_GLUT_FONT
 void plutBitmapCharacter(int c);
@@ -140,12 +141,14 @@ void     p_glutMainLoop(void) {
 
 void     p_glutGameModeString(char *str) {}
 void     p_glutEnterGameMode(void) {}
+void     p_glutLeaveGameMode(void) {}
 
 void     p_glutBitmapCharacter(void *font, int character){}
 void     p_glutSwapBuffers(void){}
 
 void     p_glutPopWindow(void){}
 void     p_glutShowWindow(void){}
+void     p_glutDestroyWindow(int theWindow) {}
 
 void     p_glutReshapeWindow(int width, int height){ WinX=width;WinY=height;}
 
@@ -166,6 +169,7 @@ void     p_glutReshapeFunc(void (*func)(int width, int height)) {}
 void     p_glutKeyboardFunc(void (*func)(unsigned char key, int x, int y)) {}
 void     p_glutMouseFunc(void (*func)(int button, int state, int x, int y)) {}
 void     p_glutMotionFunc(void (*func)(int x, int y)) {}
+void     p_glutPassiveMotionFunc(void (*func)(int x, int y)) {}
 void     p_glutSpecialFunc(void (*func)(int key, int x, int y)) {}
 void     p_glutIdleFunc(void (*func)(void)) { idleFunc = func; }
 
@@ -197,6 +201,7 @@ static void (*mouseFunc)(int button, int state,int x,int y) = NULL;
 static void (*keyboardFunc)(unsigned char key,int x,int y) = NULL;
 static void (*specialFunc)(int key,int x,int y) = NULL;
 static void (*motionFunc)(int x,int y) = NULL;
+static void (*passiveMotionFunc)(int x,int y) = NULL;
 static int WinX = 640,WinY=480;
 
 static int redisplay=true;
@@ -211,6 +216,7 @@ int p_glutGetRedisplay(void)
 
 void     p_glutGameModeString(char *str) {}
 void     p_glutEnterGameMode(void) {}
+void     p_glutLeaveGameMode(void) {}
 
 void p_glutHandleEvent(p_glut_event *ev) {
   /* NOTE: this function should be called with an unblocked interpreter,
@@ -233,6 +239,9 @@ void p_glutHandleEvent(p_glut_event *ev) {
   case P_GLUT_MOTION_EVENT:
     if(motionFunc) motionFunc(ev->x,ev->y);
     break;
+  case P_GLUT_PASSIVE_MOTION_EVENT:
+    if(passiveMotionFunc) passiveMotionFunc(ev->x,ev->y);
+    break;
   case P_GLUT_CHAR_EVENT:
     if(keyboardFunc) keyboardFunc((unsigned char)ev->input,ev->x,ev->y);
     break;
@@ -261,6 +270,7 @@ void     p_glutSwapBuffers(void){PBlock();PyRun_SimpleString("pymol._swap_buffer
 
 void     p_glutPopWindow(void){}
 void     p_glutShowWindow(void){}
+void     p_glutDestroyWindow(int theWindow) {}
 
 void     p_glutReshapeWindow(int width, int height){ WinX=width;WinY=height;}
 
@@ -281,6 +291,7 @@ void     p_glutReshapeFunc(void (*func)(int width, int height)) { reshapeFunc =
 void     p_glutKeyboardFunc(void (*func)(unsigned char key, int x, int y)) { keyboardFunc = func;}
 void     p_glutMouseFunc(void (*func)(int button, int state, int x, int y)) { mouseFunc = func;}
 void     p_glutMotionFunc(void (*func)(int x, int y)) { motionFunc=func;}
+void     p_glutPassiveMotionFunc(void (*func)(int x, int y)) { passiveMotionFunc=func;}
 void     p_glutSpecialFunc(void (*func)(int key, int x, int y)) { specialFunc=func;}
 void     p_glutIdleFunc(void (*func)(void)) { idleFunc = func; }
 
