@@ -156,6 +156,11 @@ void ButModeInit(void)
   strcpy(I->Code[cButModePickAtom1], "Pk1  ");
   strcpy(I->Code[cButModeMoveAtom], "MovA ");
   strcpy(I->Code[cButModeMenu], "Menu ");
+  strcpy(I->Code[cButModeSeleSet], "Sele ");
+  strcpy(I->Code[cButModeSeleToggle], "+/-  ");
+  strcpy(I->Code[cButModeSeleAdd], "+Box ");
+  strcpy(I->Code[cButModeSeleSub], "-Box ");  
+
 
   I->Block = OrthoNewBlock(NULL);
   I->Block->fClick = ButModeClick;
@@ -263,6 +268,7 @@ int ButModeTranslate(int button, int mod)
 /*========================================================================*/
 int ButModeClick(Block *block,int button,int x,int y,int mod)
 {
+
   switch(mod) {
   case cOrthoSHIFT:
     PLog("cmd.mouse('backward')",cPLog_pym);
@@ -285,8 +291,10 @@ void ButModeDraw(Block *block)
   int nf;
 
   if(PMGUI) {
-    glColor3fv(I->Block->BackColor);
-    BlockFill(I->Block);
+    if(SettingGetGlobal_b(cSetting_internal_gui_mode)==0) {
+      glColor3fv(I->Block->BackColor);
+      BlockFill(I->Block);
+    }
 
     x = I->Block->rect.left+cButModeLeftMargin;
     y = (I->Block->rect.top-cButModeLineHeight)-cButModeTopMargin;
@@ -299,6 +307,8 @@ void ButModeDraw(Block *block)
     /*    GrapDrawStr("2-Bttn Selecting",x+88,y);*/
     y-=cButModeLineHeight;
 
+
+
     glColor3fv(I->Block->TextColor);
     GrapDrawStr("Buttons ",x+1,y);
     glColor3fv(I->TextColor1);
@@ -394,7 +404,8 @@ void ButModeDraw(Block *block)
 
 
     glColor3fv(I->Block->TextColor);
-    GrapDrawStr("Dbl-Clk ",x,y);
+    glColor3fv(I->TextColor1);
+    GrapDrawStr(" DblClk",x,y);
     glColor3fv(I->TextColor2);
     glRasterPos4d(x+64,y,0,1);
     for(a=16;a<19;a++) {
@@ -407,11 +418,38 @@ void ButModeDraw(Block *block)
     glColor3fv(I->Block->TextColor);
     y-=cButModeLineHeight;
 
-
-
-
-    glColor3fv(I->TextColor3);
+    /*
     if(I->Caption[0]) GrapDrawStr(I->Caption,x,y);
+    */
+
+    {
+      glColor3fv(I->Block->TextColor);
+      GrapDrawStr("Selecting ",x,y);
+      glColor3fv(I->TextColor3);
+      switch(SettingGetGlobal_i(cSetting_mouse_selection_mode)) {
+      case 0:
+        GrapDrawStr("Atoms",x+80,y);        
+        break;
+      case 1:
+        GrapDrawStr("Residues",x+80,y);        
+        break;
+      case 2:
+        GrapDrawStr("Chains",x+80,y);        
+        break;
+      case 3:
+        GrapDrawStr("Segments",x+80,y);        
+        break;
+      case 4:
+        GrapDrawStr("Objects",x+80,y);        
+        break;
+      case 5:
+        GrapDrawStr("Molecules",x+80,y);        
+        break;
+      case 6:
+        GrapDrawStr("C-alphas",x+80,y);        
+        break;
+      }
+    }
 
     glColor3fv(I->Block->TextColor);
     y-=cButModeLineHeight;
@@ -429,6 +467,7 @@ void ButModeDraw(Block *block)
             nf,(int)(rate+0.5F));
     GrapDrawStr(rateStr,x+48,y);
 
+
   }
 }
 
