@@ -30,6 +30,8 @@ Z* -------------------------------------------------------------------
 #include"Setting.h"
 #include"main.h"
 #include"PConv.h"
+#include"Util.h"
+#include"Parse.h"
 
 CMovie Movie;
 
@@ -181,11 +183,12 @@ static int MovieCmdFromPyList(PyObject *list,int *warning)
   int a;
   int warn=false;
 
+  if(ok) ok=(list!=NULL);
   if(ok) ok=PyList_Check(list);
   
   for(a=0;a<I->NFrame;a++) {
     if(ok) ok=PConvPyStrToStr(PyList_GetItem(list,a),I->Cmd[a],OrthoLineLength);
-    if(ok) warn=warn||I->Cmd[a][0];
+    if(ok) warn= (warn||I->Cmd[a][0]);
   }
   *warning=warn;
   return(ok);
@@ -208,14 +211,22 @@ int MovieFromPyList(PyObject *list,int *warning)
     ok=PConvPyListToFloatArrayInPlace(PyList_GetItem(list,2),I->Matrix,cSceneViewSize);
   if(ok) ok=PConvPyIntToInt(PyList_GetItem(list,3),&I->Playing);
   if(ok&&I->NFrame) {
-    I->Sequence=Alloc(int,I->NFrame+1);
-    I->Cmd=Alloc(MovieCmdType,I->NFrame+1);
+    I->Sequence=VLACalloc(int,I->NFrame);
+    I->Cmd=VLACalloc(MovieCmdType,I->NFrame);
     if(ok) ok=PConvPyListToIntArrayInPlace(PyList_GetItem(list,4),I->Sequence,I->NFrame);
     if(ok) ok=MovieCmdFromPyList(PyList_GetItem(list,5),warning);
-    if(warning&&Security) {
+    if((*warning)&&Security) {
       MovieSetLock(true);
     }
   }
+  if(ok&&(ll>6)) {
+    PyObject *tmp;
+    VLAFreeP(I->ViewElem);
+    I->ViewElem = NULL;
+    tmp = PyList_GetItem(list,6);
+    if(tmp && !(tmp == Py_None))
+      ok = ViewElemVLAFromPyList(tmp,&I->ViewElem,I->NFrame);
+  }
   if(!ok) {
     MovieReset();
   }
@@ -229,9 +240,10 @@ static PyObject *MovieCmdAsPyList(void)
   int a;
 
   result = PyList_New(I->NFrame);
-  for(a=0;a<I->NFrame;a++) {
-    PyList_SetItem(result,a,PyString_FromString(I->Cmd[a]));
-  }
+  if(result) 
+    for(a=0;a<I->NFrame;a++) {
+      PyList_SetItem(result,a,PyString_FromString(I->Cmd[a]));
+    }
   return(PConvAutoNone(result));
 }
 
@@ -241,7 +253,7 @@ PyObject *MovieAsPyList(void)
   CMovie *I=&Movie;
   PyObject *result = NULL;
 
-  result = PyList_New(6);
+  result = PyList_New(7);
   PyList_SetItem(result,0,PyInt_FromLong(I->NFrame));
   PyList_SetItem(result,1,PyInt_FromLong(I->MatrixFlag));
   PyList_SetItem(result,2,PConvFloatArrayToPyList(I->Matrix,cSceneViewSize));
@@ -256,6 +268,12 @@ PyObject *MovieAsPyList(void)
   } else {
     PyList_SetItem(result,5,PConvAutoNone(NULL));
   }
+  if(I->ViewElem) {
+    PyList_SetItem(result,6,ViewElemVLAAsPyList(I->ViewElem,I->NFrame));
+  } else {
+    PyList_SetItem(result,6,PConvAutoNone(NULL));
+  }
+
 /*   ImageType *Image;
   int *Sequence;
   MovieCmdType *Cmd;
@@ -416,13 +434,17 @@ int MoviePNG(char *prefix,int save,int start,int stop)
   return(true);
 }
 /*========================================================================*/
-void MovieSequence(char *str)
+void MovieAppendSequence(char *str,int start_from)
 {
   CMovie *I=&Movie;
   int c=0;
   int i;
-  char *s;
-  c=0;
+  char *s,number[20];
+ 
+  if(start_from<0)
+    start_from = I->NFrame;
+
+  c=start_from;
 
   PRINTFB(FB_Movie,FB_Debugging)
     " MovieSequence: entered. str:%s\n",str
@@ -430,39 +452,54 @@ void MovieSequence(char *str)
 
   s=str;
   while(*s) {
-	 if(sscanf(s,"%i",&i)) {
+    s=ParseWord(number,s,20);
+	 if(sscanf(number,"%i",&i)) { /* slow */
 		c++;
 	 }
-	 s++;
-	 while(*s) {
-		if(*s==' ') break;
-		s++;
-	 }
   }
-  FreeP(I->Sequence);
-  FreeP(I->Cmd);
-  I->NFrame=0;
-  if(str[0]) { /* not just a reset */
-    I->Sequence=Alloc(int,c+1);
-    I->Cmd=Alloc(MovieCmdType,c+1);
-    
-    for(i=0;i<c;i++)
+
+  if(!c) {
+    VLAFreeP(I->Sequence); 
+    VLAFreeP(I->Cmd);
+    VLAFreeP(I->ViewElem);
+    I->NFrame=0;
+  } else {
+    if(!I->Sequence) {
+      I->Sequence = VLACalloc(int,c);
+    } else {
+      VLASize(I->Sequence, int, start_from); /* to clear */
+      VLASize(I->Sequence, int, c);
+    }
+    if(!I->Cmd) {
+      I->Cmd = VLACalloc(MovieCmdType, c);
+    } else {
+      VLASize(I->Cmd, MovieCmdType, start_from);      
+      VLASize(I->Cmd, MovieCmdType, c);
+    }
+    if(!I->ViewElem) {
+      I->ViewElem = VLACalloc(CViewElem, c);    
+    } else {
+      VLASize(I->ViewElem, CViewElem, start_from);
+      VLASize(I->ViewElem, CViewElem, c);
+    }
+  }
+
+  if(c&&str[0]) { /* not just a reset */
+    for(i=start_from;i<c;i++)
       I->Cmd[i][0]=0;
-    c=0;
+    c=start_from;
     s=str;
     while(*s) {
-      if(sscanf(s,"%i",&I->Sequence[c])) {
+      s=ParseWord(number,s,20);
+      if(sscanf(number,"%i",&I->Sequence[c])) {
         c++;
       }
-      s++;
-      while(*s) {
-        if(*s==' ') break;
-        s++;
-      }
     }
-    I->Sequence[c]=-1;
     I->NFrame=c;
+  } else if(!str[0]) {
+    I->NFrame=start_from;
   }
+           
   VLACheck(I->Image,ImageType,I->NFrame);
   PRINTFB(FB_Movie,FB_Debugging)
     " MovieSequence: leaving... I->NFrame%d\n",I->NFrame
@@ -520,6 +557,8 @@ void MovieDoFrameCommand(int frame)
       {
         if(I->Cmd[frame][0]) 
           PParse(I->Cmd[frame]);
+        if(I->ViewElem) 
+          SceneFromViewElem(I->ViewElem+frame);
       }
   }
 }
@@ -543,6 +582,257 @@ void MovieSetCommand(int frame,char *command)
       ENDFB;
   }
 }
+
+static int interpolate_view(CViewElem *first,CViewElem *last,float power,float bias)
+{
+  float first3x3[9];
+  float last3x3[9];
+  float inverse3x3[9];
+  float inter3x3[9];
+  float rot_axis[3],trans[3]={0.0F,0.0F,0.0F};
+  float angle;
+  CViewElem *current;
+  int n = (last-first)-1;
+  Matrix53f rot,imat;
+  int a;
+  float tVector[3],tCenter[3],tDir[3];
+  float tLen;
+  float pLen;
+  float v1[3],v2[3],sProj[3];
+  float tAngle=0.0F;
+  float tLinear = true; /* always do linear for now... */
+  float pivot[3];
+  const float _1 = 1.0F, _p5 = 0.5F;
+  int parabolic = true;
+
+  if(power<0.0F) {
+    parabolic = false;
+    power = -power;
+  }
+  
+  /* I have no clue whether we're column or row major at this
+     point...but it hardly matters!  */
+
+  copy44d33f(first->matrix,first3x3);
+  copy44d33f(last->matrix,last3x3);
+  transpose33f33f(first3x3,inverse3x3);
+
+  copy3f(first->pre,&rot[3][0]);
+  copy3f(last->pre,&rot[4][0]);
+  multiply33f33f(inverse3x3,last3x3,&rot[0][0]);
+
+  matrix_to_rotation(rot,rot_axis,&angle);
+
+  if(!tLinear) {
+    subtract3f(last->pre,first->pre,tVector);
+    normalize23f(tVector,tDir);
+    tLen = length3f(tVector); 
+    
+    average3f(last->pre,first->pre,tCenter); /* center of translation */
+    
+    pLen = dot_product3f(tDir,rot_axis); /* project translation vector onto rotation axis */
+    
+    trans[0] -= tDir[0] * pLen; /* subtract component of translation from rotation axis*/
+    trans[1] -= tDir[1] * pLen; /* why??? */
+    trans[2] -= tDir[2] * pLen;
+    normalize3f(trans);
+    
+    /* rotation axis must be perpendicular to the direction of translation */
+    
+    cross_product3f(tDir,trans,v1);
+    
+    normalize3f(v1);
+    
+    transform33f3f(&rot[0][0],v1,v2);
+
+    project3f(v2,trans,sProj);  /* project vector onto plane _|_ to axis */
+    subtract3f(v2,sProj,v2); 
+    normalize3f(v2);
+  
+    tAngle = acos(dot_product3f(v1,v2));
+    
+    if(fabs(tAngle)>fabs(angle))
+      tAngle = fabs(angle)*(tAngle/fabs(angle));
+    
+    /* if translation angle > rotation angle then sets translation angle 
+     * to same as rotation angle, with proper sign of course */
+    
+    if (tAngle>0.0001) 
+      {
+        pLen = tan(tAngle/2); 
+        if(fabs(pLen)>0.0000001)
+          pLen = (tLen/2)/pLen;
+        else
+          {
+            pLen = 1000.0;
+            tLinear = true;
+          }
+      }
+    else
+      {
+        pLen = 1000.0;
+        tLinear = true;
+      }
+  
+    pivot[0] = tCenter[0] - pLen * v1[0];
+    pivot[1] = tCenter[1] - pLen * v1[1];
+    pivot[2] = tCenter[2] - pLen * v1[2];
+  }
+
+  current = first+1;
+  
+  for(a=0;a<n;a++) {
+    float fxn = ((float)a+1)/(n+1);
+    float fxn_1;
+
+    /*    printf("Fxn: %8.3f ",fxn);*/
+    if(bias!=1.0F) {
+      fxn = 1-pow(1-pow(fxn,bias),_1/bias);
+    }
+    /*    printf("%8.3f bias %8.3f\n",fxn,bias);*/
+
+    if(power!=1.0F) {
+      if(fxn<0.5F) {
+        if(parabolic) 
+          fxn = (float)pow(fxn*2.0F,power)*_p5; /* parabolic */
+        else
+          fxn = (_1-pow(_1-pow((fxn*2.0F),power),_1/power))*_p5; /* circular */
+      } else if(fxn>0.5F) {
+        fxn = _1 - fxn;
+        if(parabolic) 
+          fxn = (float)pow(fxn*2.0F,power)*_p5; /* parabolic */
+        else
+          fxn = (_1-pow(_1-pow((fxn*2.0F),power),_1/power))*_p5; /* circular */
+        fxn = _1 - fxn;
+      }
+    }
+
+    fxn_1 = 1.0F - fxn;
+
+    *current = *first;
+    matrix_interpolate(imat,rot,pivot,rot_axis,angle,tAngle,false,tLinear,fxn);
+    current->matrix_flag = true;
+    multiply33f33f(first3x3,&imat[0][0],inter3x3);
+
+    copy33f44d(inter3x3,current->matrix);
+
+    if(first->pre_flag && last->pre_flag) {
+      copy3f(&imat[4][0],current->pre);
+      current->pre_flag=true;
+    } else {
+      current->pre_flag=false;
+    }
+
+    if(first->clip_flag && last->clip_flag) {
+      current->front = first->front * fxn_1 + last->front * fxn;
+      current->back = first->back * fxn_1 + last->back * fxn;
+      current->clip_flag = true;
+    } else {
+      current->clip_flag = false;
+    }
+
+    if(first->post_flag && last->post_flag) {
+      mix3d(first->post,last->post,(double)fxn,current->post);
+      current->post_flag=true;
+    } else {
+      current->post_flag=false;
+    }
+    current->specification_level = 1;
+    current++;
+  }
+
+  return 1;
+}
+/*========================================================================*/
+int MovieView(int action,int first,int last,float power,float bias)
+{
+  CMovie *I=&Movie;
+  int frame;
+  switch(action) {
+  case 0: /* set */
+    if(I->ViewElem) {
+      if(first<0)
+        first = SceneGetFrame();
+      if(last<0)
+        last = first;
+      for(frame=first;frame<=last;frame++) {
+        if((frame>=0)&&(frame<I->NFrame)) {
+          VLACheck(I->ViewElem,CViewElem,frame);
+          PRINTFB(FB_Movie,FB_Details)
+            " MovieView: Setting frame %d.\n",frame+1
+            ENDFB;
+          SceneToViewElem(I->ViewElem+frame);          
+          I->ViewElem[frame].specification_level = 2;
+        }
+      }
+    }
+    break;
+  case 1: /* clear */
+    if(I->ViewElem) {
+      if(first<0)
+        first = SceneGetFrame();
+      if(last<0)
+        last = first;
+      for(frame=first;frame<=last;frame++) {
+        if((frame>=0)&&(frame<I->NFrame)) {
+          VLACheck(I->ViewElem,CViewElem,frame);
+          UtilZeroMem((void*)(I->ViewElem+frame),sizeof(CViewElem));
+        }
+      }
+    }
+    break;
+  case 2: /* interpolate & reinterpolate */
+  case 3:
+    {
+      CViewElem *first_view=NULL,*last_view=NULL;
+      if(first<0)
+        first = 0;
+      if(last<0)
+        last = SceneGetNFrame()-1;
+
+      VLACheck(I->ViewElem,CViewElem,last);
+      if(action==2) {
+        PRINTFB(FB_Movie,FB_Details)
+          " MovieView: interpolating unspecified frames %d to %d.\n",first+1,last+1
+          ENDFB;
+      } else {
+        PRINTFB(FB_Movie,FB_Details)
+          " MovieView: reinterpolating all frames %d to %d.\n",first+1,last+1
+          ENDFB;
+      }
+      for(frame=first;frame<=last;frame++) {
+        if((frame>=0)&&(frame<I->NFrame)) {
+          if(!first_view) {
+            if(I->ViewElem[frame].specification_level==2) { /* specified */
+              first_view = I->ViewElem + frame;
+            }
+          } else {
+            CViewElem *view;
+            int interpolate_flag = false;
+            if(I->ViewElem[frame].specification_level==2) { /* specified */
+              last_view = I->ViewElem + frame;
+              if(action==2) {/* interpolate */
+                for(view=first_view+1;view<last_view;view++) {
+                  if(!view->specification_level)
+                    interpolate_flag = true;
+                }
+              } else {
+                interpolate_flag=true;
+              }
+              if(interpolate_flag) {
+                interpolate_view(first_view,last_view,power,bias);
+              }
+              first_view = last_view;
+              last_view = NULL;
+            }
+          }
+        }
+      }
+    }
+    break;
+  }
+  return 1;
+}
 /*========================================================================*/
 void MovieAppendCommand(int frame,char *command)
 {
@@ -574,6 +864,12 @@ ImageType MovieGetImage(int index)
 	 return(NULL);
 }
 /*========================================================================*/
+int MovieDefined(void) 
+{
+  CMovie *I=&Movie;
+  return(I->NFrame>0);
+}
+/*========================================================================*/
 int MovieGetLength(void)
 {
   CMovie *I=&Movie;
@@ -610,8 +906,11 @@ void MovieClearImages(void)
 void MovieReset(void) {
   CMovie *I=&Movie;
   MovieClearImages();
-  FreeP(I->Cmd);
-  FreeP(I->Sequence);
+
+  VLAFreeP(I->Cmd);  
+  VLAFreeP(I->Sequence);  
+  VLAFreeP(I->ViewElem);  
+
   I->NFrame=0;
   I->MatrixFlag=false;
   I->Locked=false;
@@ -623,8 +922,9 @@ void MovieFree(void)
   CMovie *I=&Movie;
   MovieClearImages();
   VLAFree(I->Image);
-  FreeP(I->Cmd);
-  FreeP(I->Sequence);
+  VLAFreeP(I->ViewElem);
+  VLAFreeP(I->Cmd);
+  VLAFreeP(I->Sequence);
 }
 /*========================================================================*/
 void MovieInit(void)
@@ -635,6 +935,7 @@ void MovieInit(void)
   I->Image=VLAMalloc(10,sizeof(ImageType),5,true); /* auto-zero */
   I->Sequence=NULL;
   I->Cmd=NULL;
+  I->ViewElem = NULL;
   I->NImage=0;
   I->NFrame=0;
   for(a=0;a<16;a++)
