@@ -88,6 +88,7 @@ typedef struct {
   int SplashFlag;
   int LoopFlag;
   int LoopMod;
+  int HaveSeqViewer;
   BlockRect LoopRect;
   CQueue *cmds;
   CQueue *feedback;
@@ -692,8 +693,9 @@ void OrthoKey(unsigned char k,int x,int y,int mod)
       }
       break;
     case 22: /* CTRL V -- paste */
-      PParse("cmd.paste()");
-      PFlush();
+      PBlockAndUnlockAPI();
+      PRunString("cmd.paste()");
+      PLockAPIAndUnblock();
       /* PBlockAndUnlockAPI();
         PRunString("cmd.paste()");
         PLockAPIAndUnblock(); */
@@ -904,8 +906,14 @@ void OrthoDoDraw()
   int skip_prompt = 0;
   int render = false;
 
-  if(!SettingGet(cSetting_test1))
+  if(SettingGetGlobal_b(cSetting_seq_view)) {
     SeqUpdate(); 
+    I->HaveSeqViewer = true;
+  } else if(I->HaveSeqViewer) {
+    SeqUpdate();
+    I->HaveSeqViewer = false;
+  }
+
   if(SettingGet_i(NULL,NULL,cSetting_internal_prompt))
     skip_prompt = 0;
   else
@@ -928,7 +936,19 @@ void OrthoDoDraw()
     if(Feedback(FB_OpenGL,FB_Debugging))
       PyMOLCheckOpenGLErr("OrthoDoDraw checkpoint 0");
 
-    rightSceneMargin=(int)SettingGet(cSetting_internal_gui_width);
+    if(SettingGetGlobal_b(cSetting_internal_gui)) {
+      switch(SettingGetGlobal_i(cSetting_internal_gui_mode)) {
+      case 0:
+        rightSceneMargin=(int)SettingGet(cSetting_internal_gui_width);
+        break;
+      default:
+        rightSceneMargin = 0;
+        break;
+      }
+    } else {
+      rightSceneMargin=0;
+    }
+
     internal_feedback=(int)SettingGet(cSetting_internal_feedback);
 
     v=SettingGetfv(cSetting_bg_rgb);
@@ -1034,7 +1054,9 @@ void OrthoDoDraw()
         x = cOrthoLeftMargin;
         y = cOrthoBottomMargin;
         
-        
+#ifdef _PYMOL_SHARP3D
+        y+=(7*cOrthoLineHeight)/10;
+#endif
         if((int)SettingGet(cSetting_text)||I->SplashFlag)
           showLines=I->ShowLines;
         else {
@@ -1106,8 +1128,9 @@ static void OrthoDrawWizardPrompt(void)
   int nChar,c,ll;
   int maxLen;
   BlockRect rect;
+  int prompt_mode = SettingGetGlobal_i(cSetting_wizard_prompt_mode);
 
-  if(I->WizardPromptVLA) {
+  if(I->WizardPromptVLA && prompt_mode) {
     vla = I->WizardPromptVLA;
     
     nLine = UtilCountStringVLA(vla);
@@ -1127,7 +1150,7 @@ static void OrthoDrawWizardPrompt(void)
           ll=0;
           p++;
           c--;
-        } else if(((*p)=='`')&& /* color encoded */
+        } else if(((*p)=='\\')&& /* color encoded */
                   (p[1]>='0')&&(p[1]<='9')&&
                   (p[2]>='0')&&(p[2]<='9')&&
                   (p[3]>='0')&&(p[3]<='9')) /* relying upon short-circuit logic to avoid overrun */ {
@@ -1140,20 +1163,34 @@ static void OrthoDrawWizardPrompt(void)
         }
       }
       
-      rect.top = I->Height-cWizardTopMargin;
+      rect.top = I->Height;
+      if(I->HaveSeqViewer)
+        if(!SettingGetGlobal_b(cSetting_seq_view_location)) {
+          rect.top -= SeqGetHeight();
+        }
+
+      if(prompt_mode!=3) {
+        rect.top -= cWizardTopMargin;
+        rect.left = cWizardLeftMargin;
+      } else {
+        rect.top -= 1;
+        rect.left = 1;
+      }
+
       rect.bottom = rect.top-(nLine*cOrthoLineHeight+2*cWizardBorder)-2;
-      rect.left = cWizardLeftMargin;
       rect.right = rect.left + cOrthoCharWidth*maxLen + 2*cWizardBorder+1;
       
-      glColor3fv(I->WizardBackColor);
+      if(prompt_mode==1) {
+        glColor3fv(I->WizardBackColor);
+        
+        glBegin(GL_POLYGON);
+        glVertex2i(rect.right,rect.top);
+        glVertex2i(rect.right,rect.bottom);
+        glVertex2i(rect.left,rect.bottom);
+        glVertex2i(rect.left,rect.top);
+        glEnd();
+      }
 
-      glBegin(GL_POLYGON);
-      glVertex2i(rect.right,rect.top);
-      glVertex2i(rect.right,rect.bottom);
-      glVertex2i(rect.left,rect.bottom);
-      glVertex2i(rect.left,rect.top);
-      glEnd();
-      
       glColor3fv(I->WizardTextColor);
       
       x = rect.left+cWizardBorder;
@@ -1170,7 +1207,7 @@ static void OrthoDrawWizardPrompt(void)
       c=nChar;
       while(c>0) {
         if(*p) {
-          if((*p=='`')&&(*(p+1))&&(*(p+2))&&(*(p+3))) {
+          if((*p=='\\')&&(*(p+1))&&(*(p+2))&&(*(p+3))) {
             if(*(p+1)=='-') {
               glColor3fv(I->WizardTextColor);
               p+=4;
@@ -1200,20 +1237,21 @@ static void OrthoDrawWizardPrompt(void)
 }
 
 /*========================================================================*/
-void OrthoReshape(int width, int height)
+void OrthoReshape(int width, int height,int force)
 {
   OrthoObject *I=&Ortho;
 
   Block *block = NULL;
-  int sceneBottom,sceneRight;
+  int sceneBottom,sceneRight = 0;
   int internal_gui_width;
   int internal_feedback;
+  int sceneTop = 0;
 
   PRINTFD(FB_Ortho)
     " OrthoReshape-Debug: %d %d\n",width,height
     ENDFD;
 
-  if((width!=I->Width)||(height!=I->Height)) {
+  if((width!=I->Width)||(height!=I->Height)||force) {
   if(width<0) width=I->Width;
   if(height<0) height=I->Height;
 
@@ -1228,26 +1266,51 @@ void OrthoReshape(int width, int height)
     sceneBottom = 0;
     
   internal_gui_width = (int)SettingGet(cSetting_internal_gui_width);
-  if(SettingGet(cSetting_internal_gui)>0.0) {
-    sceneRight = internal_gui_width;
-  } else {
+  if(!SettingGetGlobal_b(cSetting_internal_gui)) {
+    internal_gui_width = 0;
     sceneRight = 0;
-    internal_gui_width=0;
+  } else {
+    switch(SettingGetGlobal_i(cSetting_internal_gui_mode)) {
+    case 1:
+      sceneRight = 0;
+      break;
+    default:
+      sceneRight = internal_gui_width;
+      break;
+    }
   }
 
-  if(SettingGet(cSetting_internal_gui)) {
+
+  {
+    int seqHeight;
+    block=SeqGetBlock();
+    block->active=true;
     
-    {
-      int seqHeight;
-      block=SeqGetBlock();
-      block->active=true;
+    if(SettingGetGlobal_b(cSetting_seq_view_location)) {
+      
       BlockSetMargin(block,height-sceneBottom-10,0,sceneBottom,sceneRight);
       if(block->fReshape)
         block->fReshape(block,width,height);			
       seqHeight = SeqGetHeight();
       BlockSetMargin(block,height-sceneBottom-seqHeight,0,sceneBottom,sceneRight);
-      sceneBottom +=seqHeight;
+      if(!SettingGetGlobal_b(cSetting_seq_view_overlay)) {
+        sceneBottom +=seqHeight;
+      }
+      
+    } else {
+      
+      BlockSetMargin(block,0,0,height-10,sceneRight);
+      if(block->fReshape)
+        block->fReshape(block,width,height);			
+      seqHeight = SeqGetHeight();
+      BlockSetMargin(block,0,0,height-seqHeight,sceneRight);
+      if(!SettingGetGlobal_b(cSetting_seq_view_overlay)) {
+        sceneTop = seqHeight;
+      }
     }
+  }
+
+  if(SettingGet(cSetting_internal_gui)) {
 
     block=ExecutiveGetBlock();
     block->active=true;
@@ -1277,8 +1340,9 @@ void OrthoReshape(int width, int height)
   }
 
   block=SceneGetBlock();
-  BlockSetMargin(block,0,0,sceneBottom,sceneRight);
-  BlockSetMargin(&I->LoopBlock,0,0,sceneBottom,sceneRight);
+
+  BlockSetMargin(block,sceneTop,0,sceneBottom,sceneRight);
+  BlockSetMargin(&I->LoopBlock,sceneTop,0,sceneBottom,sceneRight);
 
   if(PMGUI) 
     glGetIntegerv(GL_VIEWPORT,I->ViewPort);
@@ -1286,8 +1350,9 @@ void OrthoReshape(int width, int height)
   OrthoPushMatrix();
   block=NULL;
   while(ListIterate(I->Blocks,block,next))
-	 if(block->fReshape)
+	 if(block->fReshape) {
 		block->fReshape(block,width,height);			
+    }
   OrthoPopMatrix();
 
   WizardRefresh(); /* safe to call even if no wizard exists */
@@ -1444,9 +1509,9 @@ void OrthoSplash(void)
     PRINTF " Copyright (C) 2004 by DeLano Scientific LLC.\n All Rights Reserved.\n \n" ENDF;    
     PRINTF " Legal usage of this product beyond your free 90 evaluation period requires\n" ENDF;
     PRINTF " a current PyMOL subscription from DeLano Scientific LLC unless you meet the\n" ENDF;
-    PRINTF " MacPyMOL incentive exemption criteria. \n \n" ENDF;
+    PRINTF " exemption criteria for this Incentive Product. \n \n" ENDF;
     PRINTF " Please visit http://www.pymol.org/funding.html for subscription information.\n \n"ENDF;
-    PRINTF " MacPyMOL is based on the PyMOL(TM) Molecular Graphics System, Version " ENDF;
+    PRINTF " Based on the PyMOL(TM) Molecular Graphics System, Version " ENDF;
     PRINTF _PyMOL_VERSION ENDF;
     PRINTF ".\n " ENDF;
   } else {
@@ -1501,6 +1566,7 @@ void OrthoInit(int showSplash)
   I->GrabbedBy = NULL;
   I->ClickedIn = NULL;
   I->DrawText=1;
+  I->HaveSeqViewer = false;
   I->TextColor[0]=0.7F;
   I->TextColor[1]=0.7F;
   I->TextColor[2]=1.0;
@@ -1540,9 +1606,6 @@ void OrthoInit(int showSplash)
   /*printf("orthoNewLine: CC: %d CL:%d PC: %d IF:L %d\n",I->CurChar,I->CurLine,
 	 I->PromptChar,I->InputFlag);*/
   
-  ButModeInit();
-  ControlInit();
-  SeqInit();
   PopInit();
   for(a=0;a<=OrthoHistoryLines;a++)
     I->History[a][0]=0;
@@ -1554,9 +1617,6 @@ void OrthoFree(void)
 
   VLAFreeP(I->WizardPromptVLA);
   PopFree();
-  SeqFree();
-  ButModeFree();
-  ControlFree();
   QueueFree(I->cmds);
   I->cmds=NULL;
   QueueFree(I->feedback);
@@ -1621,7 +1681,7 @@ void OrthoPasteIn(char *buffer)
   int execFlag=false;
   OrthoLineType buf2;
 
-  if(I->CurChar) {
+  if(I->InputFlag) {
     if(I->CursorChar>=0) {
       strcpy(buf2,I->Line[curLine]+I->CursorChar);
       strcpy(I->Line[curLine]+I->CursorChar,buffer);
@@ -1652,8 +1712,7 @@ void OrthoPasteIn(char *buffer)
         }
     }
   } else {
-    strcpy(I->Line[curLine]+I->PromptChar,buffer);
-    I->CurChar=strlen(I->Line[curLine]);
+    OrthoRestorePrompt();
     
     while((I->Line[curLine][I->CurChar-1]==10)||(I->Line[curLine][I->CurChar-1]==13)) 
       {
@@ -1664,8 +1723,11 @@ void OrthoPasteIn(char *buffer)
           break;
       }
   }
-  if(execFlag)
+  if(execFlag) {
+    printf("[%s]\n",I->Line[curLine]);
     OrthoParseCurrentLine();
+    } else
+    I->InputFlag=true;
 }
 
 
