@@ -62,6 +62,8 @@ PyObject *P_unlock = NULL;
 PyObject *P_lock_c = NULL; /* C locks */
 PyObject *P_unlock_c = NULL;
 
+PyObject *P_do = NULL;
+
 PyObject *P_time = NULL;
 PyObject *P_sleep = NULL;
 PyObject *P_main = NULL;
@@ -128,7 +130,6 @@ int PComplete(char *str,int buf_size)
   char *st2;
   PBlockAndUnlockAPI();
   if(P_complete) {
-    fflush(stdout);
     result = PyObject_CallFunction(P_complete,"s",str);
     if(result) {
       if(PyString_Check(result)) {
@@ -143,6 +144,20 @@ int PComplete(char *str,int buf_size)
   return(ret);
 }
 
+int PTruthCallStr0(PyObject *object,char *method)
+{
+  int result = false;
+  PyObject *tmp;
+  tmp = PyObject_CallMethod(object,method,"");
+  if(tmp) {
+    if(PyObject_IsTrue(tmp))
+      result = 1;
+    Py_DECREF(tmp);
+  }
+  return(result);
+}
+
+
 int PTruthCallStr(PyObject *object,char *method,char *argument)
 {
   int result = false;
@@ -155,6 +170,31 @@ int PTruthCallStr(PyObject *object,char *method,char *argument)
   }
   return(result);
 }
+
+int PTruthCallStr1i(PyObject *object,char *method,int argument)
+{
+  int result = false;
+  PyObject *tmp;
+  tmp = PyObject_CallMethod(object,method,"i",argument);
+  if(tmp) {
+    if(PyObject_IsTrue(tmp))
+      result = 1;
+    Py_DECREF(tmp);
+  }
+  return(result);
+}
+int PTruthCallStr4i(PyObject *object,char *method,int a1,int a2,int a3,int a4)
+{
+  int result = false;
+  PyObject *tmp;
+  tmp = PyObject_CallMethod(object,method,"iiii",a1,a2,a3,a4);
+  if(tmp) {
+    if(PyObject_IsTrue(tmp))
+      result = 1;
+    Py_DECREF(tmp);
+  }
+  return(result);
+}
                                        
 void PXDecRef(PyObject *obj)
 {
@@ -231,6 +271,7 @@ int PAlterAtomState(float *v,char *expr,int read_only,AtomInfoType *at)
     PConvFloatToPyDictItem(dict,"q",at->q);
     PConvFloatToPyDictItem(dict,"b",at->b);
     PConvFloatToPyDictItem(dict,"vdw",at->vdw);
+    PConvFloatToPyDictItem(dict,"bohr",at->bohr_radius);
     PConvFloatToPyDictItem(dict,"partial_charge",at->partialCharge);
     PConvIntToPyDictItem(dict,"formal_charge",at->formalCharge);
     PConvIntToPyDictItem(dict,"cartoon",at->cartoon);
@@ -298,6 +339,8 @@ int PAlterAtom(AtomInfoType *at,char *expr,int read_only,char *model,int index)
   Chain alt;
   PyObject *alt_id1,*alt_id2=NULL;
   SegIdent segi;
+  PyObject *flags_id1,*flags_id2=NULL;
+  int flags;
   PyObject *segi_id1,*segi_id2=NULL;
   TextType textType;
   PyObject *text_type_id1,*text_type_id2=NULL;
@@ -305,11 +348,12 @@ int PAlterAtom(AtomInfoType *at,char *expr,int read_only,char *model,int index)
   PyObject *ss_id1,*ss_id2=NULL;
   char atype[7];
   PyObject *type_id1,*type_id2=NULL;
-  float b,q,partialCharge,vdw;
+  float b,q,partialCharge,vdw,bohr;
   PyObject *b_id1,*b_id2=NULL;
   PyObject *q_id1,*q_id2=NULL;
   PyObject *partial_charge_id1,*partial_charge_id2=NULL;
   PyObject *vdw_id1,*vdw_id2=NULL;
+  PyObject *bohr_id1,*bohr_id2=NULL;
   int formalCharge,numericType;
   PyObject *formal_charge_id1,*formal_charge_id2=NULL;
   PyObject *numeric_type_id1,*numeric_type_id2=NULL;
@@ -321,6 +365,8 @@ int PAlterAtom(AtomInfoType *at,char *expr,int read_only,char *model,int index)
   LabelType label;
   int id;
   PyObject *ID_id1,*ID_id2=NULL;
+  PyObject *state_id1,*state_id2;
+  int state;
   PyObject *dict;
   int result=true;
   
@@ -343,6 +389,7 @@ int PAlterAtom(AtomInfoType *at,char *expr,int read_only,char *model,int index)
   type_id1 = PConvStringToPyDictItem(dict,"type",atype);
   name_id1 = PConvStringToPyDictItem(dict,"name",at->name);
   resn_id1 = PConvStringToPyDictItem(dict,"resn",at->resn);
+  flags_id1 = PConvIntToPyDictItem(dict,"flags",at->flags);
   resi_id1 = PConvStringToPyDictItem(dict,"resi",at->resi);
   resv_id1 = PConvIntToPyDictItem(dict,"resv",at->resv); /* subordinate to resi */
   chain_id1 = PConvStringToPyDictItem(dict,"chain",at->chain);
@@ -355,12 +402,14 @@ int PAlterAtom(AtomInfoType *at,char *expr,int read_only,char *model,int index)
   q_id1 = PConvFloatToPyDictItem(dict,"q",at->q);
   b_id1 = PConvFloatToPyDictItem(dict,"b",at->b);
   vdw_id1 = PConvFloatToPyDictItem(dict,"vdw",at->vdw);
+  bohr_id1 = PConvFloatToPyDictItem(dict,"bohr",at->bohr_radius);
   partial_charge_id1 = PConvFloatToPyDictItem(dict,"partial_charge",at->partialCharge);
   formal_charge_id1 = PConvIntToPyDictItem(dict,"formal_charge",at->formalCharge);
   cartoon_id1 = PConvIntToPyDictItem(dict,"cartoon",at->cartoon);
   label_id1 = PConvStringToPyDictItem(dict,"label",at->label);
   color_id1 = PConvIntToPyDictItem(dict,"color",at->color);
   ID_id1 = PConvIntToPyDictItem(dict,"ID",at->id);
+  state_id1 = PConvIntToPyDictItem(dict,"state",at->discrete_state);
 
   PyRun_String(expr,Py_single_input,P_globals,dict);
   if(PyErr_Occurred()) {
@@ -385,6 +434,8 @@ int PAlterAtom(AtomInfoType *at,char *expr,int read_only,char *model,int index)
         result=false;
       else if(!(resn_id2 = PyDict_GetItemString(dict,"resn")))
         result=false;
+      else if(!(flags_id2 = PyDict_GetItemString(dict,"flags")))
+        result=false;
       else if(!(resi_id2 = PyDict_GetItemString(dict,"resi")))
         result=false;
       else if(!(resv_id2 = PyDict_GetItemString(dict,"resv")))
@@ -405,6 +456,8 @@ int PAlterAtom(AtomInfoType *at,char *expr,int read_only,char *model,int index)
         result=false;
       else if(!(vdw_id2=PyDict_GetItemString(dict,"vdw")))
         result=false;
+      else if(!(bohr_id2=PyDict_GetItemString(dict,"bohr")))
+        result=false;
       else if(!(partial_charge_id2 = PyDict_GetItemString(dict,"partial_charge")))
         result=false;
       else if(!(formal_charge_id2 = PyDict_GetItemString(dict,"formal_charge")))
@@ -419,6 +472,9 @@ int PAlterAtom(AtomInfoType *at,char *expr,int read_only,char *model,int index)
         result=false;
       if(!(ID_id2 = PyDict_GetItemString(dict,"ID")))
         result=false;
+      if(!(state_id2 = PyDict_GetItemString(dict,"state")))
+        result=false;
+
       if(PyErr_Occurred()) {
         PyErr_Print();
         result=false;
@@ -456,8 +512,7 @@ int PAlterAtom(AtomInfoType *at,char *expr,int read_only,char *model,int index)
           result=false;
         else {
           if(strcmp(at->resi,resi)!=0)
-            if(!sscanf(resi,"%i",&at->resv))
-              at->resv=1;
+            at->resv=AtomResvFromResi(resi);
           strcpy(at->resi,resi);
         }
       } else if (resv_id1!=resv_id2) {
@@ -522,6 +577,12 @@ int PAlterAtom(AtomInfoType *at,char *expr,int read_only,char *model,int index)
           at->vdw=vdw;
 
       }
+      if(bohr_id1!=bohr_id2) {
+        if(!PConvPyObjectToFloat(bohr_id2,&bohr))
+          result=false;
+        else
+          at->bohr_radius=bohr;
+      }
       if(partial_charge_id1!=partial_charge_id2) {
         if(!PConvPyObjectToFloat(partial_charge_id2,&partialCharge))
           result=false;
@@ -532,8 +593,10 @@ int PAlterAtom(AtomInfoType *at,char *expr,int read_only,char *model,int index)
       if(formal_charge_id1!=formal_charge_id2) {
         if(!PConvPyObjectToInt(formal_charge_id2,&formalCharge))
           result=false;
-        else
+        else {
           at->formalCharge=formalCharge;
+          at->chemFlag = false; /* invalidate chemistry info for this atom */
+        }
 
       }
       if(cartoon_id1!=cartoon_id2) {
@@ -548,6 +611,12 @@ int PAlterAtom(AtomInfoType *at,char *expr,int read_only,char *model,int index)
         else
           at->color=color;
       }
+      if(state_id1!=state_id2) {
+        if(!PConvPyObjectToInt(state_id2,&state))
+          result=false;
+        else
+          at->discrete_state=state;
+      }
       if(label_id1!=label_id2) {
         if(!PConvPyObjectToStrMaxLen(label_id2,label,sizeof(LabelType)-1))
           result=false;
@@ -555,6 +624,13 @@ int PAlterAtom(AtomInfoType *at,char *expr,int read_only,char *model,int index)
           strcpy(at->label,label);
         }
       }
+      if(flags_id1!=flags_id2) {
+        if(!PConvPyObjectToInt(flags_id2,&flags))
+          result=false;
+        else
+          at->flags = flags;
+      }
+
       if(numeric_type_id1!=numeric_type_id2) {
         if(!PConvPyObjectToInt(numeric_type_id2,&numericType))
           result=false;
@@ -602,11 +678,13 @@ int PLabelAtom(AtomInfoType *at,char *expr,int index)
   PConvStringToPyDictItem(dict,"name",at->name);
   PConvStringToPyDictItem(dict,"resn",at->resn);
   PConvStringToPyDictItem(dict,"resi",at->resi);
+  PConvIntToPyDictItem(dict,"resv",at->resv); 
   PConvStringToPyDictItem(dict,"chain",at->chain);
   PConvStringToPyDictItem(dict,"alt",at->alt);
   PConvStringToPyDictItem(dict,"segi",at->segi);
   PConvStringToPyDictItem(dict,"ss",at->ssType);
   PConvFloatToPyDictItem(dict,"vdw",at->vdw);
+  PConvFloatToPyDictItem(dict,"bohr",at->bohr_radius);
   PConvStringToPyDictItem(dict,"text_type",at->textType);
   PConvStringToPyDictItem(dict,"elem",at->elem);
   PConvIntToPyDictItem(dict,"geom",at->geom);
@@ -674,7 +752,7 @@ void PLockAPIAsGlut(void) /* must call with an unblocked interpreter */
   PXDecRef(PyObject_CallFunction(P_lock,NULL));
   while(P_glut_thread_keep_out) {
     /* IMPORTANT: keeps the glut thread out of an API operation... */
-    /* NOTE: the keep_out variable can only be changed by the thread
+    /* NOTE: the keep_out variable can only be changed or read by the thread
        holding the API lock, therefore it is safe even through increment
        isn't atomic. */
     PRINTFD(FB_Threads)
@@ -712,6 +790,7 @@ void PLockAPIAsGlut(void) /* must call with an unblocked interpreter */
 #ifndef _EPYMOL
 void	initExtensionClass(void);
 void	initsglite(void);
+void  init_champ(void);
 void    init_opengl(void);
 void    init_opengl_num(void);
 void    init_glu(void);
@@ -732,6 +811,7 @@ void	initarrayfns();
 void	initlapack_lite();
 void	initumath();
 void	initranlib();
+void  init_champ();
 #endif
 #endif
 #endif
@@ -742,6 +822,7 @@ void	initranlib();
 #ifndef _EPYMOL
 void	initExtensionClass(void);
 void	initsglite(void);
+void  init_champ(void);
 void    init_opengl(void);
 void    init_opengl_num(void);
 void    init_glu(void);
@@ -880,6 +961,7 @@ putenv(line3);
 #ifndef _EPYMOL
 	initExtensionClass();
 	initsglite();
+   init_champ();
 #ifdef WIN32
 	/* initialize numeric python */
 	init_numpy();
@@ -888,6 +970,8 @@ putenv(line3);
 	initlapack_lite();
 	initumath();
 	initranlib();
+/* initialize champ */
+   init_champ();
 #endif
     init_opengl();
     init_opengl_num();
@@ -1010,6 +1094,7 @@ void PGetOptions(PyMOLOptionRec *rec)
   rec->incentive_product = PyInt_AsLong(PyObject_GetAttrString(options,"incentive_product"));
   rec->multisample = PyInt_AsLong(PyObject_GetAttrString(options,"multisample"));
   load_str = PyString_AsString(PyObject_GetAttrString(options,"after_load_script"));
+
   if(load_str) {
     if(load_str[0]) {
       UtilNCopy(rec->after_load_script,load_str,PYMOL_MAX_OPT_STR);
@@ -1042,6 +1127,7 @@ void PInit(void)
 	/* sglite */
 	initExtensionClass();
 	initsglite();
+    init_champ();
 	/* initialize numeric python */
 	init_numpy();
 	initmultiarray();
@@ -1094,16 +1180,19 @@ void PInit(void)
   if(!P_cmd) ErrFatal("PyMOL","can't find 'cmd'");
 
   P_lock = PyObject_GetAttrString(P_cmd,"lock");
-  if(!P_lock) ErrFatal("PyMOL","can't find 'pm.lock()'");
+  if(!P_lock) ErrFatal("PyMOL","can't find 'cmd.lock()'");
 
   P_unlock = PyObject_GetAttrString(P_cmd,"unlock");
-  if(!P_unlock) ErrFatal("PyMOL","can't find 'pm.unlock()'");
+  if(!P_unlock) ErrFatal("PyMOL","can't find 'cmd.unlock()'");
 
   P_lock_c = PyObject_GetAttrString(P_cmd,"lock_c");
-  if(!P_lock_c) ErrFatal("PyMOL","can't find 'pm.lock_c()'");
+  if(!P_lock_c) ErrFatal("PyMOL","can't find 'cmd.lock_c()'");
 
   P_unlock_c = PyObject_GetAttrString(P_cmd,"unlock_c");
-  if(!P_unlock_c) ErrFatal("PyMOL","can't find 'pm.unlock_c()'");
+  if(!P_unlock_c) ErrFatal("PyMOL","can't find 'cmd.unlock_c()'");
+
+  P_do = PyObject_GetAttrString(P_cmd,"do");
+  if(!P_do) ErrFatal("PyMOL","can't find 'cmd.do()'");
 
   PRunString("import menu\n");  
   P_menu = PyDict_GetItemString(P_globals,"menu");
@@ -1221,6 +1310,13 @@ void PExit(int code)
   ExecutiveDelete("all");
   PBlock();
   MainFree();
+
+  /* we're having trouble with threading errors after calling Py_Exit,
+     so for the time being, let's just take the process down at this
+     point, instead of allowing PyExit to be called. */
+
+  exit(EXIT_SUCCESS);                        
+
   Py_Exit(code);
 }
 
@@ -1229,6 +1325,14 @@ void PParse(char *str)
   OrthoCommandIn(str);
 }
 
+void PDo(char *str) /* assumes we already hold the re-entrant API lock */
+{
+  int blocked;
+  blocked = PAutoBlock();
+  Py_XDECREF(PyObject_CallFunction(P_do,"s",str));
+  PAutoUnblock(blocked);
+}
+
 void PLog(char *str,int format) 
      /* general log routine can write PML 
         or PYM commands to appropriate log file */
@@ -1261,6 +1365,8 @@ void PLog(char *str,int format)
             }
             break;
           case cPLog_pym: /* .pym file */
+            if((str[0]=='_')&&(str[1])==' ')
+              str+=2;
             switch(format) {
             case cPLog_pml_lf:
               a =strlen(str);
@@ -1305,37 +1411,12 @@ void PLogFlush(void)
     }
 }
 
-static void PDeleteAll(void *p)
-{ /* assumes blocked and unlocked API */
-  if(!PyMOLTerminating)
-    PyObject_CallMethod(P_cmd,"delete","s","all");
-}
-
-static void PMaintainObjectAll(void) 
-/* legacy exception for "del all", which is broken by version 0.86
-   "del all" now means what it should in Python: delete an object
-   referenced by "all".  In order to support old scripts, we create an
-   "all" object and then catch its destruction event...(yes, I know
-   this is weak, but its is the most compatible solution...).  */
-{
-  PyObject *all;
-
-  all = PyDict_GetItemString(P_globals,"all");
-  if(all==NULL) {
-    all = PyCObject_FromVoidPtr(NULL,PDeleteAll);
-    PyDict_SetItemString(P_globals,"all",all);
-    Py_DECREF(all);
-  }
-}
-
-
 void PFlush(void) {  
   /* NOTE: ASSUMES unblocked Python threads and a locked API */
   PyObject *err;
   char buffer[OrthoLineLength+1];
   while(OrthoCommandOut(buffer)) {
     PBlockAndUnlockAPI();
-    PMaintainObjectAll();
     PXDecRef(PyObject_CallFunction(P_parse,"s",buffer));
     err = PyErr_Occurred();
     if(err) {
@@ -1353,7 +1434,6 @@ void PFlushFast(void) {
   PyObject *err;
   char buffer[OrthoLineLength+1];
   while(OrthoCommandOut(buffer)) {
-    PMaintainObjectAll();
     PRINTFD(FB_Threads)
       " PFlushFast-DEBUG: executing '%s' as thread 0x%x\n",buffer,
       PyThread_get_thread_ident()
@@ -1529,6 +1609,7 @@ void PDefineFloat(char *name,float value) {
   PUnblock();
 }
 
+
 /* This function is called by the interpreter to get its own name */
 char *getprogramname(void)
 {
