@@ -518,6 +518,31 @@ int PConvPyListToIntVLA(PyObject *obj,int **f)
   return(ok);
 }
 
+int PConvPyListToDoubleArrayInPlace(PyObject *obj,double *ff,int ll)
+{
+  int ok = true;
+  int a,l;
+  if(!obj) { 
+    ok=false;
+  } else if(!PyList_Check(obj)) {
+    ok=false;
+  } else {
+    l=PyList_Size(obj);
+    if (l!=ll) 
+      ok=false;
+    else {
+      if(!l)
+        ok=-1;
+      else
+        ok=l;
+      for(a=0;a<l;a++)
+        *(ff++) = PyFloat_AsDouble(PyList_GetItem(obj,a));
+    }
+    /* NOTE ASSUMPTION! */
+  }
+  return(ok);
+}
+
 int PConvPyListToFloatArrayInPlace(PyObject *obj,float *ff,int ll)
 {
   int ok = true;
@@ -650,6 +675,16 @@ PyObject *PConvFloatArrayToPyList(float *f,int l)
   return(result);
 }
 
+PyObject *PConvDoubleArrayToPyList(double *f,int l)
+{
+  int a;
+  PyObject *result = Py_None;
+  result=PyList_New(l);
+  for(a=0;a<l;a++) 
+    PyList_SetItem(result,a,PyFloat_FromDouble(*(f++)));
+  return(result);
+}
+
 PyObject *PConvFloatVLAToPyList(float *f)
 {
   int a,l;
@@ -779,27 +814,36 @@ int PConvPyListToStringVLA(PyObject *obj,char **vla_ptr)
 void PConv44PyListTo44f(PyObject *src,float *dest) /* note lost of precision */
 {
   PyObject *row;
-
-  row = PyList_GetItem(src,0);
-  dest[ 0]=(float)PyFloat_AsDouble(PyList_GetItem(row,0));
-  dest[ 1]=(float)PyFloat_AsDouble(PyList_GetItem(row,1));
-  dest[ 2]=(float)PyFloat_AsDouble(PyList_GetItem(row,2));
-  dest[ 3]=(float)PyFloat_AsDouble(PyList_GetItem(row,3));
-  row = PyList_GetItem(src,1);
-  dest[ 4]=(float)PyFloat_AsDouble(PyList_GetItem(row,0));
-  dest[ 5]=(float)PyFloat_AsDouble(PyList_GetItem(row,1));
-  dest[ 6]=(float)PyFloat_AsDouble(PyList_GetItem(row,2));
-  dest[ 7]=(float)PyFloat_AsDouble(PyList_GetItem(row,3));
-  row = PyList_GetItem(src,2);
-  dest[ 8]=(float)PyFloat_AsDouble(PyList_GetItem(row,0));
-  dest[ 9]=(float)PyFloat_AsDouble(PyList_GetItem(row,1));
-  dest[10]=(float)PyFloat_AsDouble(PyList_GetItem(row,2));
-  dest[11]=(float)PyFloat_AsDouble(PyList_GetItem(row,3));
-  row = PyList_GetItem(src,3);    
-  dest[12]=(float)PyFloat_AsDouble(PyList_GetItem(row,0));
-  dest[13]=(float)PyFloat_AsDouble(PyList_GetItem(row,1));
-  dest[14]=(float)PyFloat_AsDouble(PyList_GetItem(row,2));
-  dest[15]=(float)PyFloat_AsDouble(PyList_GetItem(row,3));
+  if(src&&dest&&PyList_Check(src)) {
+      row = PyList_GetItem(src,0);
+      if(row&&PyList_Check(row)) {
+        dest[ 0]=(float)PyFloat_AsDouble(PyList_GetItem(row,0));
+        dest[ 1]=(float)PyFloat_AsDouble(PyList_GetItem(row,1));
+        dest[ 2]=(float)PyFloat_AsDouble(PyList_GetItem(row,2));
+        dest[ 3]=(float)PyFloat_AsDouble(PyList_GetItem(row,3));
+      }
+      row = PyList_GetItem(src,1); 
+      if(row&&PyList_Check(row)) {
+        dest[ 4]=(float)PyFloat_AsDouble(PyList_GetItem(row,0));
+        dest[ 5]=(float)PyFloat_AsDouble(PyList_GetItem(row,1));
+        dest[ 6]=(float)PyFloat_AsDouble(PyList_GetItem(row,2));
+        dest[ 7]=(float)PyFloat_AsDouble(PyList_GetItem(row,3));
+      }
+      row = PyList_GetItem(src,2);
+      if(row&&PyList_Check(row)) {
+        dest[ 8]=(float)PyFloat_AsDouble(PyList_GetItem(row,0));
+        dest[ 9]=(float)PyFloat_AsDouble(PyList_GetItem(row,1));
+        dest[10]=(float)PyFloat_AsDouble(PyList_GetItem(row,2));
+        dest[11]=(float)PyFloat_AsDouble(PyList_GetItem(row,3));
+      }
+      row = PyList_GetItem(src,3);    
+      if(row&&PyList_Check(row)) {
+        dest[12]=(float)PyFloat_AsDouble(PyList_GetItem(row,0));
+        dest[13]=(float)PyFloat_AsDouble(PyList_GetItem(row,1));
+        dest[14]=(float)PyFloat_AsDouble(PyList_GetItem(row,2));
+        dest[15]=(float)PyFloat_AsDouble(PyList_GetItem(row,3));
+      }
+  }
 }
 
 
