@@ -516,8 +516,9 @@ void RayTransformFirst(CRay *I)
 
   backface_cull = (int)SettingGet(cSetting_backface_cull);
   
-  if(SettingGet(cSetting_two_sided_lighting)||
-     SettingGet(cSetting_ray_interior_color)>=0)
+  if((SettingGet(cSetting_two_sided_lighting)||
+      (SettingGet(cSetting_transparency_mode)==1)||
+      SettingGet(cSetting_ray_interior_color)>=0))
     backface_cull=0;
 
   basis0 = I->Basis;
@@ -879,37 +880,36 @@ void RayRenderPOV(CRay *I,int width,int height,char **headerVLA_ptr,
 /*========================================================================*/
 void RayProjectTriangle(CRay *I,RayInfo *r,float *light,float *v0,float *n0,float scale)
 {
-  float w2;
+  register float w2;
   float d1[3],d2[3],d3[3];
   float p1[3],p2[3],p3[3];
-  int c=0;
+  register int c=0;
+  register const float _0 = 0.0F;
+  register float *impact = r->impact;
 
-  if(dot_product3f(light,n0-3)>=0.0F) c++;  
-  if(dot_product3f(light,n0)>=0.0F) c++;
-  if(dot_product3f(light,n0+3)>=0.0F) c++;
-  if(dot_product3f(light,n0+6)>=0.0F) c++;
+  if(dot_product3f(light,n0-3)>=_0) c++;  
+  else if(dot_product3f(light,n0)>=_0) c++;
+  else if(dot_product3f(light,n0+3)>=_0) c++;
+  else if(dot_product3f(light,n0+6)>=_0) c++;
   
   if(c) {
 
     w2 = 1.0F-(r->tri1+r->tri2);
     
-    subtract3f(v0,r->impact,d1);
+    subtract3f(v0,impact,d1);
+    subtract3f(v0+3,impact,d2);
+    subtract3f(v0+6,impact,d3);
     project3f(d1,n0,p1);
-    scale3f(p1,w2,d1);
-    
-    subtract3f(v0+3,r->impact,d2);
     project3f(d2,n0+3,p2);
-    scale3f(p2,r->tri1,d2);
-    
-    subtract3f(v0+6,r->impact,d3);
     project3f(d3,n0+6,p3);
+    scale3f(p1,w2,d1);
+    scale3f(p2,r->tri1,d2);
     scale3f(p3,r->tri2,d3);
-    
     add3f(d1,d2,d2);
     add3f(d2,d3,d3);
     scale3f(d3,scale,d3);
-    if(dot_product3f(r->surfnormal,d3)>=0.0F)
-      add3f(d3,r->impact,r->impact);
+    if(dot_product3f(r->surfnormal,d3)>=_0)
+      add3f(d3,impact,impact);
   }
 }
 
@@ -1079,7 +1079,7 @@ int RayTraceThread(CRayThreadInfo *T)
 	float fog;
 	float *inter=NULL;
 	float fog_start=0.0F;
-	float gamma,inp,sig=1.0F;
+   /*	float gamma,inp,sig=1.0F;*/
 	float persist,persist_inv;
 	float new_front;
 	int pass;
@@ -1091,6 +1091,7 @@ int RayTraceThread(CRayThreadInfo *T)
 	float excl_trans;
 	int shadows;
 	int trans_shadows;
+   int trans_mode;
 	float first_excess;
 	int pixel_flag;
 	float ray_trans_spec;
@@ -1099,6 +1100,7 @@ int RayTraceThread(CRayThreadInfo *T)
 	int interior_flag;
 	int interior_shadows;
 	int interior_wobble;
+   float interior_reflect;
 	int wobble_save;
 	float		settingPower, settingReflectPower,settingSpecPower,settingSpecReflect, _0, _1, _p5, _255, _persistLimit, _inv3;
 	float		invHgt, invFrontMinusBack, inv1minusFogStart,invWdth,invHgtRange;
@@ -1113,6 +1115,7 @@ int RayTraceThread(CRayThreadInfo *T)
    unsigned int edge_avg[4];
    int edge_cnt=0;
    float base[2];
+   float interior_normal[3];
    float edge_width = 0.35356F;
    float edge_height = 0.35356F;
    float trans_spec_cut,trans_spec_scale;
@@ -1120,7 +1123,10 @@ int RayTraceThread(CRayThreadInfo *T)
    float red_blend=0.0F;
    float blue_blend=0.0F;
    float green_blend=0.0F;
+   float trans_cont;
+   int trans_cont_flag = false;
    int blend_colors;
+   int max_pass;
 
 	_0		= 0.0F;
 	_1		= 1.0F;
@@ -1136,21 +1142,29 @@ int RayTraceThread(CRayThreadInfo *T)
 	*/
 	I = T->ray;
 	
-	interior_shadows	= (int)SettingGet(cSetting_ray_interior_shadows);
-	interior_wobble	= (int)SettingGet(cSetting_ray_interior_texture);
-	interior_color		= (int)SettingGet(cSetting_ray_interior_color);
+	interior_shadows	= SettingGetGlobal_i(cSetting_ray_interior_shadows);
+	interior_wobble	= SettingGetGlobal_i(cSetting_ray_interior_texture);
+	interior_color		= SettingGetGlobal_i(cSetting_ray_interior_color);
+   interior_reflect  = 1.0F - SettingGet(cSetting_ray_interior_reflect);
+
 	project_triangle	= SettingGet(cSetting_ray_improve_shadows);
-	shadows				= (int)SettingGet(cSetting_ray_shadows);
-	trans_shadows		= (int)SettingGet(cSetting_ray_transparency_shadows);
-	backface_cull		= (int)SettingGet(cSetting_backface_cull);
-	opaque_back			= (int)SettingGet(cSetting_ray_opaque_background);
-	two_sided_lighting	= (int)SettingGet(cSetting_two_sided_lighting);
+	shadows				= SettingGetGlobal_i(cSetting_ray_shadows);
+	trans_shadows		= SettingGetGlobal_i(cSetting_ray_transparency_shadows);
+	backface_cull		= SettingGetGlobal_i(cSetting_backface_cull);
+	opaque_back			= SettingGetGlobal_i(cSetting_ray_opaque_background);
+	two_sided_lighting	= SettingGetGlobal_i(cSetting_two_sided_lighting);
 	ray_trans_spec		= SettingGet(cSetting_ray_transparency_specular);
+   trans_cont        = SettingGetGlobal_f(cSetting_ray_transparency_contrast);
+   trans_mode        = SettingGetGlobal_i(cSetting_transparency_mode);
+   if(trans_mode==1) two_sided_lighting = true;
+   if(trans_cont>1.0F)
+     trans_cont_flag = true;
 	ambient				= SettingGet(cSetting_ambient);
 	lreflect			= SettingGet(cSetting_reflect);
 	direct				= SettingGet(cSetting_direct);
    trans_spec_cut = SettingGet(cSetting_ray_transparency_spec_cut);
-   blend_colors    = (int)SettingGet(cSetting_ray_blend_colors);
+   blend_colors    = SettingGetGlobal_i(cSetting_ray_blend_colors);
+   max_pass = SettingGetGlobal_i(cSetting_ray_max_passes);
    if(blend_colors) {
      red_blend = SettingGet(cSetting_ray_blend_red);
      green_blend = SettingGet(cSetting_ray_blend_green);
@@ -1173,22 +1187,28 @@ int RayTraceThread(CRayThreadInfo *T)
      settingSpecReflect = 0.0F;
    }
     
-	if((interior_color>=0)||(two_sided_lighting))
+	if((interior_color>=0)||(two_sided_lighting)||(trans_mode==1))
 		backface_cull	= 0;
 
 	shadow_fudge = SettingGet(cSetting_ray_shadow_fudge);
 	
-	gamma = SettingGet(cSetting_gamma);
+   /*	gamma = SettingGet(cSetting_gamma);
 	if(gamma > R_SMALL4)
 		gamma	= _1/gamma;
 	else
 		gamma	= _1;
-	
+   */
+
 	inv1minusFogStart	= _1;
 	
 	fog = SettingGet(cSetting_ray_trace_fog);
-   if(fog<0.0F)
-     fog = SettingGet(cSetting_depth_cue);
+   if(fog<0.0F) {
+     if(SettingGet(cSetting_depth_cue)) {
+       fog = SettingGet(cSetting_fog);
+     } else 
+       fog = _0;
+   }
+   
 	if(fog != _0) 
 	{
 		fogFlag	= true;
@@ -1202,8 +1222,6 @@ int RayTraceThread(CRayThreadInfo *T)
 		inv1minusFogStart	= _1 / (_1 - fog_start);
 	}
 
-	if(interior_color>=0)
-		inter = ColorGet(interior_color);
 
 	/* ray-trace */
 	
@@ -1232,6 +1250,14 @@ int RayTraceThread(CRayThreadInfo *T)
      offset = (T->phase * render_height/T->n_thread);
      offset = offset - (offset % T->n_thread) + T->phase;
    }
+
+	if(interior_color>=0) {
+		inter = ColorGet(interior_color);
+      interior_normal[0] = interior_reflect*bp2->LightNormal[0];
+      interior_normal[1] = interior_reflect*bp2->LightNormal[1];
+      interior_normal[2] = 1.0F+interior_reflect*bp2->LightNormal[2];
+      normalize3f(interior_normal);
+   }
      
 	r1.base[2]	= _0;
 
@@ -1363,10 +1389,9 @@ int RayTraceThread(CRayThreadInfo *T)
               excl_trans		= _0;
               pass			= 0;
               new_front		= T->front;
-              while((persist > _persistLimit) && (pass < 25))
+              while((persist > _persistLimit) && (pass <= max_pass))
                 {
                   pixel_flag		= false;
-                  
                   SceneCall.except = exclude;
                   SceneCall.front = new_front;
                   SceneCall.excl_trans = excl_trans;
@@ -1376,17 +1401,15 @@ int RayTraceThread(CRayThreadInfo *T)
 
                   interior_flag = SceneCall.interior_flag;
                   
-                  if((i >= 0) || interior_flag) 
+                  if(((i >= 0) || interior_flag) && (pass < max_pass))
                     {
                       pixel_flag		= true;
                       n_hit++;
                       r1.trans = r1.prim->trans;
                       if(interior_flag)
                         {
+                          copy3f(interior_normal,r1.surfnormal);
                           copy3f(r1.base,r1.impact);
-                          r1.surfnormal[0]	= _0;
-                          r1.surfnormal[1]	= _0;
-                          r1.surfnormal[2]	= _1;
                           r1.impact[2]	-= T->front;
                           
                           if(interior_wobble >= 0) 
@@ -1454,9 +1477,7 @@ int RayTraceThread(CRayThreadInfo *T)
                               if((!two_sided_lighting) && (interior_color>=0)) 
                                 {
                                   interior_flag		= true;
-                                  r1.surfnormal[0]	= _0;
-                                  r1.surfnormal[1]	= _0;
-                                  r1.surfnormal[2]	= _1;
+                                  copy3f(interior_normal,r1.surfnormal);
                                   copy3f(r1.base,r1.impact);
                                   r1.impact[2]		-= T->front;
                                   r1.dist				= T->front;
@@ -1584,7 +1605,9 @@ int RayTraceThread(CRayThreadInfo *T)
                     }
                   else if(pass) 
                     {
-                      /* hit nothing, and we're on on second or greater pass */
+                      /* hit nothing, and we're on on second or greater pass,
+                         or we're on the last pass of a dead-end loop */
+                      i=-1;
 
                       fc[0] = first_excess+T->bkrd[0];
                       fc[1] = first_excess+T->bkrd[1];
@@ -1599,12 +1622,15 @@ int RayTraceThread(CRayThreadInfo *T)
                       ffact1m = 0.0F;
                       
                       pixel_flag	= true;
+                      if(trans_cont_flag)
+                        persist = pow(persist,trans_cont);
+                      
                     }
-                  
+
                   if(pixel_flag)
                     {
+                      /*
                       inp	= (fc[0]+fc[1]+fc[2]) * _inv3;
-                      
                       if(inp < R_SMALL4) 
                         sig = _1;
                       else
@@ -1613,7 +1639,12 @@ int RayTraceThread(CRayThreadInfo *T)
                       cc0 = (uint)(sig * fc[0] * _255);
                       cc1 = (uint)(sig * fc[1] * _255);
                       cc2 = (uint)(sig * fc[2] * _255);
-                      
+                      */
+
+                      cc0 = (uint)(fc[0] * _255);
+                      cc1 = (uint)(fc[1] * _255);
+                      cc2 = (uint)(fc[2] * _255);
+
                       if(cc0 > 255) cc0 = 255;
                       if(cc1 > 255) cc1 = 255;
                       if(cc2 > 255) cc2 = 255;
@@ -1642,7 +1673,12 @@ int RayTraceThread(CRayThreadInfo *T)
                       float mix_in;
                       if(i>=0) {
                         if(fogFlag) {
-                          mix_in = persist * (_1 - r1.trans*ffact);
+                          if(trans_cont_flag&&(ffact>_p5)) {
+                            mix_in = 2*(persist*(_1-ffact)+(pow(persist,trans_cont)*(ffact-_p5)))
+                              * (_1 - r1.trans*ffact);                            
+                          } else {
+                            mix_in = persist * (_1 - r1.trans*ffact);
+                          }
                         } else {
                           mix_in = persist * (_1 - r1.trans);
                         }
@@ -1710,14 +1746,15 @@ int RayTraceThread(CRayThreadInfo *T)
                       if(r1.prim->type == cPrimSausage)	/* carry ray through the stick */
                         excl_trans	= new_front+(2*r1.surfnormal[2]*r1.prim->r1);
                       
-                      if(!backface_cull) 
+                      if((!backface_cull)&&(trans_mode!=2))
                         persist	= persist * r1.trans;
                       else 
                         {
-                          if((persist < 0.9999) && (r1.trans))	/* don't combine transparent surfaces */
+                          if((persist < 0.9999) && (r1.trans))	/* don't combine transparent surfaces */ {
                             *pixel	= last_pixel;
-                          else
+                          } else {
                             persist	= persist * r1.trans;
+                          }
                         }
                       
                     }
@@ -1735,7 +1772,7 @@ int RayTraceThread(CRayThreadInfo *T)
                     }
                   
                 } /* end of ray while */
-              
+
               if(blend_colors) {
                 
                 float red_min = _0;
@@ -2203,14 +2240,14 @@ void RayRender(CRay *I,int width,int height,unsigned int *image,
   int antialias;
   int opaque_back=0;
   int n_hit=0;
-  float *bkrd;
+  float *bkrd_ptr,bkrd[3];
   double now;
   int shadows;
   float spec_vector[3];
   int n_thread;
   int mag=1;
   int oversample_cutoff;
-
+  
 #ifdef PROFILE_BASIS
   n_cells = 0;
   n_prims = 0;
@@ -2221,15 +2258,15 @@ void RayRender(CRay *I,int width,int height,unsigned int *image,
   n_skipped = 0;
 #endif
 
-  n_thread  = (int)SettingGet(cSetting_max_threads);
+  n_thread  = SettingGetGlobal_i(cSetting_max_threads);
   if(n_thread<1)
     n_thread=1;
   if(n_thread>MAX_RAY_THREADS)
     n_thread = MAX_RAY_THREADS;
-  opaque_back = (int)SettingGet(cSetting_ray_opaque_background);
+  opaque_back = SettingGetGlobal_i(cSetting_ray_opaque_background);
   BasisSetFudge(SettingGet(cSetting_ray_triangle_fudge));
-  shadows = (int)SettingGet(cSetting_ray_shadows);
-  antialias = (int)SettingGet(cSetting_antialias);
+  shadows = SettingGetGlobal_i(cSetting_ray_shadows);
+  antialias = SettingGetGlobal_i(cSetting_antialias);
   if(antialias<0) antialias=0;
   if(antialias>4) antialias=4;
   mag = antialias;
@@ -2245,7 +2282,26 @@ void RayRender(CRay *I,int width,int height,unsigned int *image,
   } else {
     buffer_size = width*height;
   }
-  bkrd=SettingGetfv(cSetting_bg_rgb);
+  bkrd_ptr=SettingGetfv(cSetting_bg_rgb);
+  copy3f(bkrd_ptr,bkrd);
+  { /* adjust bkrd to offset the effect of gamma correction */
+    float gamma = SettingGet(cSetting_gamma);
+    register float inp;
+    register float sig;
+
+    inp = (bkrd[0]+bkrd[1]+bkrd[2])/3.0F;
+    if(inp < R_SMALL4) 
+      sig = 1.0F;
+    else
+      sig = (float)(pow(inp,gamma))/inp;
+    bkrd[0] *= sig;
+    bkrd[1] *= sig;
+    bkrd[2] *= sig;
+    if(bkrd[0]>1.0F) bkrd[0] = 1.0F;
+    if(bkrd[1]>1.0F) bkrd[1] = 1.0F;
+    if(bkrd[2]>1.0F) bkrd[2] = 1.0F;
+  }
+
   if(opaque_back) {
     if(I->BigEndian)
       back_mask = 0x000000FF;
@@ -2261,14 +2317,14 @@ void RayRender(CRay *I,int width,int height,unsigned int *image,
   }
   if(I->BigEndian) {
      background = back_mask|
-      ((0xFF& ((unsigned int)(bkrd[0]*255))) <<24)|
-      ((0xFF& ((unsigned int)(bkrd[1]*255))) <<16)|
-      ((0xFF& ((unsigned int)(bkrd[2]*255))) <<8 );
+      ((0xFF& ((unsigned int)(bkrd[0]*255+0.499))) <<24)|
+      ((0xFF& ((unsigned int)(bkrd[1]*255+0.499))) <<16)|
+      ((0xFF& ((unsigned int)(bkrd[2]*255+0.499))) <<8 );
   } else {
     background = back_mask|
-      ((0xFF& ((unsigned int)(bkrd[2]*255))) <<16)|
-      ((0xFF& ((unsigned int)(bkrd[1]*255))) <<8)|
-      ((0xFF& ((unsigned int)(bkrd[0]*255))) );
+      ((0xFF& ((unsigned int)(bkrd[2]*255+0.499))) <<16)|
+      ((0xFF& ((unsigned int)(bkrd[1]*255+0.499))) <<8)|
+      ((0xFF& ((unsigned int)(bkrd[0]*255+0.499))) );
   }
 
   OrthoBusyFast(3,20);
@@ -2417,7 +2473,7 @@ void RayRender(CRay *I,int width,int height,unsigned int *image,
       if(x_stop>width) x_stop = width;
       if(y_stop>height) y_stop = height;
 
-      oversample_cutoff = (int)SettingGet(cSetting_ray_oversample_cutoff);
+      oversample_cutoff = SettingGetGlobal_i(cSetting_ray_oversample_cutoff);
 
       if(!antialias)
         oversample_cutoff = 0;
@@ -3051,9 +3107,9 @@ void RayPrepare(CRay *I,float v0,float v1,float v2,
 {
   int a;
   if(!I->Primitive) 
-	 I->Primitive=VLACacheAlloc(CPrimitive,10000,0,cCache_ray_primitive);  
+	 I->Primitive=VLACacheAlloc(CPrimitive,10000,3,cCache_ray_primitive);  
   if(!I->Vert2Prim) 
-	 I->Vert2Prim=VLACacheAlloc(int,10000,0,cCache_ray_vert2prim);
+	 I->Vert2Prim=VLACacheAlloc(int,10000,3,cCache_ray_vert2prim);
   I->Volume[0]=v0;
   I->Volume[1]=v1;
   I->Volume[2]=v2;
