@@ -31,11 +31,24 @@ Z* -------------------------------------------------------------------
 #include"main.h"
 #include"PConv.h"
 #include"Wizard.h"
+#include"Seq.h"
 
 CSetting Setting;
 
 static void *SettingPtr(CSetting *I,int index,unsigned int size);
 
+int SettingSetSmart_i(CSetting *set1,CSetting *set2,int index, int value)
+{
+  int dummy;
+  if(set1 && SettingGetIfDefined_i(set1,index,&dummy)) {
+    return SettingSet_i(set1,index,value);
+  }
+  if(set2 && SettingGetIfDefined_i(set2,index,&dummy)) {
+    return SettingSet_i(set2,index,value);
+  }
+  return SettingSetGlobal_i(index,value);
+}
+
 int SettingSetGlobalsFromPyList(PyObject *list)
 {
   int ok=true;
@@ -44,6 +57,7 @@ int SettingSetGlobalsFromPyList(PyObject *list)
     if(PyList_Check(list)) 
       ok = SettingFromPyList(I,list);
   SettingSet_i(I,cSetting_security,Security); /* always override Security setting with global variable */
+  SettingSet_b(I,cSetting_stereo,0); /* don't start in stereo mode */
   return(ok);
 }
 
@@ -1046,9 +1060,26 @@ void SettingGenerateSideEffects(int index,char *sele,int state)
     inv_sele = sele;
   }
   switch(index) {
+  case cSetting_stereo:
+    SceneUpdateStereo();
+    break;
+  case cSetting_seq_view:
+  case cSetting_seq_view_label_spacing:
+  case cSetting_seq_view_label_mode:
+  case cSetting_seq_view_label_start:
+  case cSetting_seq_view_format:
+  case cSetting_seq_view_color:
+    SeqChanged();
+    break;
+  case cSetting_seq_view_location:
+  case cSetting_seq_view_overlay:
+    PParse("cmd.viewport(-1,-1)");
+    break;
   case cSetting_stereo_mode:
     SceneUpdateStereoMode();
     break;
+  case cSetting_dot_lighting:
+  case cSetting_mesh_lighting:
   case cSetting_light:
   case cSetting_fog:
   case cSetting_fog_start:
@@ -1071,6 +1102,7 @@ void SettingGenerateSideEffects(int index,char *sele,int state)
     break;
   case cSetting_min_mesh_spacing:
   case cSetting_mesh_mode:
+  case cSetting_mesh_type:
   case cSetting_mesh_solvent:
   case cSetting_mesh_quality:
     ExecutiveInvalidateRep(inv_sele,cRepMesh,cRepInvRep);
@@ -1189,6 +1221,7 @@ void SettingGenerateSideEffects(int index,char *sele,int state)
     SceneChanged();
     break;
   case cSetting_surface_color:
+  case cSetting_surface_ramp_above_mode:
     ExecutiveInvalidateRep(inv_sele,cRepSurface,cRepInvColor);
     SceneChanged();
     break;
@@ -1203,7 +1236,17 @@ void SettingGenerateSideEffects(int index,char *sele,int state)
   case cSetting_surface_quality:
   case cSetting_surface_mode:
   case cSetting_surface_type:
+  case cSetting_surface_carve_state:
+  case cSetting_surface_carve_cutoff:
+  case cSetting_surface_carve_selection:
+  case cSetting_surface_clear_state:
+  case cSetting_surface_clear_cutoff:
+  case cSetting_surface_clear_selection:
+  case cSetting_surface_trim_cutoff:
+  case cSetting_surface_trim_factor:
+  case cSetting_surface_solvent:
   case cSetting_surface_proximity:
+  case cSetting_cavity_cull:
     ExecutiveInvalidateRep(inv_sele,cRepSurface,cRepInvRep);
     SceneChanged();
     break;
@@ -1226,6 +1269,7 @@ void SettingGenerateSideEffects(int index,char *sele,int state)
   case cSetting_cartoon_transparency:
   case cSetting_cartoon_trace:
   case cSetting_cartoon_refine:
+  case cSetting_cartoon_nucleic_acid_mode:
   case cSetting_cartoon_sampling:
   case cSetting_cartoon_loop_quality:
   case cSetting_cartoon_loop_radius:
@@ -1295,6 +1339,8 @@ void SettingGenerateSideEffects(int index,char *sele,int state)
   case cSetting_text:
     OrthoDirty();
     break;
+  case cSetting_internal_gui_mode:
+    break;
   case cSetting_internal_gui_width:
   case cSetting_internal_gui:
   case cSetting_internal_feedback:
@@ -1308,8 +1354,8 @@ void SettingGenerateSideEffects(int index,char *sele,int state)
   case cSetting_security:
     Security = (int)SettingGet(cSetting_security);
     break;
-  case cSetting_frame:
   case cSetting_state:
+  case cSetting_frame:
     SceneChanged();
     break;
   default:
@@ -1586,7 +1632,7 @@ void SettingInitGlobal(int alloc,int reset_gui)
 
   SettingSet_f(I,cSetting_test2, 1.0F);
 
-  SettingSet_f(I,cSetting_surface_best, 0.2F);
+  SettingSet_f(I,cSetting_surface_best, 0.25F);
 
   SettingSet_f(I,cSetting_surface_normal, 0.5F);
 
@@ -1650,7 +1696,7 @@ void SettingInitGlobal(int alloc,int reset_gui)
 
   SettingSet_color(I,cSetting_label_color, "-1");
 
-  SettingSet_i(I,cSetting_ray_trace_fog, -1);
+  SettingSet_f(I,cSetting_ray_trace_fog, -1.0F);
 
   SettingSet_f(I,cSetting_spheroid_scale, 1.0F);
 
@@ -1688,9 +1734,9 @@ void SettingInitGlobal(int alloc,int reset_gui)
 
   SettingSet_b(I,cSetting_static_singletons, 1);
 
-  SettingSet_i(I,cSetting_max_triangles, 1000000);
+  SettingSet_i(I,cSetting_max_triangles, 1000000); /* no longer used */
 
-  SettingSet_f(I,cSetting_depth_cue, 1.0F);
+  SettingSet_b(I,cSetting_depth_cue, 1);
 
   SettingSet_f(I,cSetting_specular, 1.0F);
 
@@ -1778,7 +1824,7 @@ void SettingInitGlobal(int alloc,int reset_gui)
 
   SettingSet_b(I,cSetting_normalize_ccp4_maps, 1);  
 
-  SettingSet_f(I,cSetting_surface_poor, 0.89F);  
+  SettingSet_f(I,cSetting_surface_poor, 0.85F);  
 
   SettingSet_i(I,cSetting_internal_feedback, PyMOLOption->internal_feedback);
 
@@ -1796,7 +1842,7 @@ void SettingInitGlobal(int alloc,int reset_gui)
 
   SettingSet_f(I,cSetting_valence_size, 0.050F);
 
-  SettingSet_f(I,cSetting_surface_miserable, 0.8F);
+  SettingSet_f(I,cSetting_surface_miserable, 2.0F);
 
   SettingSet_b(I,cSetting_ray_opaque_background, 1);
 
@@ -1940,7 +1986,7 @@ void SettingInitGlobal(int alloc,int reset_gui)
 
   SettingSet_i(I,cSetting_mesh_solvent,0);
   
-  SettingSet_i(I,cSetting_dot_solvent,0);
+  SettingSet_b(I,cSetting_dot_solvent,0);
 
   SettingSet_f(I,cSetting_ray_shadow_fudge,0.001F);
 
@@ -2188,7 +2234,8 @@ void SettingInitGlobal(int alloc,int reset_gui)
   SettingSet_i(I,cSetting_pdb_reformat_names_mode,0); /*
                                                         0 = no reformatting, 
                                                         1 = pdb compliant,
-                                                        2 = amber compliant 
+                                                        2 = amber compliant,
+                                                        3 = pdb I/O, but iupac inside
                                                       */
   SettingSet_b(I,cSetting_ray_pixel_scale_to_window,1);
   SettingSet_i(I,cSetting_label_font_id,0);
@@ -2198,6 +2245,41 @@ void SettingInitGlobal(int alloc,int reset_gui)
   SettingSet_b(I,cSetting_dot_normals,1);
   SettingSet_b(I,cSetting_session_migration,1);
   SettingSet_b(I,cSetting_mesh_normals,1);
+  SettingSet_b(I,cSetting_mesh_type,0);
+
+  SettingSet_b(I,cSetting_dot_lighting,1);
+  SettingSet_b(I,cSetting_mesh_lighting,0);
+  SettingSet_b(I,cSetting_surface_solvent,0);
+  SettingSet_i(I,cSetting_triangle_max_passes,5);
+  SettingSet_f(I,cSetting_ray_interior_reflect,0.4F);
+  SettingSet_i(I,cSetting_internal_gui_mode,0);
+  SettingSet_s(I,cSetting_surface_carve_selection,"");
+  SettingSet_i(I,cSetting_surface_carve_state,0);
+  SettingSet_f(I,cSetting_surface_carve_cutoff,0.0F);
+  SettingSet_s(I,cSetting_surface_clear_selection,"");
+  SettingSet_i(I,cSetting_surface_clear_state,0);
+  SettingSet_f(I,cSetting_surface_clear_cutoff,0.0F);
+  SettingSet_f(I,cSetting_surface_trim_cutoff,0.2F);
+  SettingSet_f(I,cSetting_surface_trim_factor,2.0F);
+  SettingSet_i(I,cSetting_ray_max_passes,25);
+  SettingSet_b(I,cSetting_active_selections,true);
+  SettingSet_f(I,cSetting_ray_transparency_contrast,1.0F);
+  SettingSet_b(I,cSetting_seq_view,0);
+  SettingSet_i(I,cSetting_mouse_selection_mode,1);
+  SettingSet_i(I,cSetting_seq_view_label_spacing,5);
+  SettingSet_i(I,cSetting_seq_view_label_start,1);
+  SettingSet_i(I,cSetting_seq_view_format,0);
+  SettingSet_i(I,cSetting_seq_view_location,0);
+  SettingSet_b(I,cSetting_seq_view_overlay,0);
+  SettingSet_b(I,cSetting_auto_classify_atoms,1);
+  SettingSet_i(I,cSetting_cartoon_nucleic_acid_mode,0);
+  SettingSet_color(I,cSetting_seq_view_color, "-1");
+  SettingSet_i(I,cSetting_seq_view_label_mode,2);
+  SettingSet_i(I,cSetting_surface_ramp_above_mode,0);
+  SettingSet_b(I,cSetting_stereo,0);
+  SettingSet_i(I,cSetting_wizard_prompt_mode,1);
+  SettingSet_f(I,cSetting_coulomb_cutoff,0.0F);
+
 }
 
 
