@@ -82,6 +82,8 @@ float *SettingGetGlobal_3fv(int index); /* always succeed */
 int   SettingSetGlobal_b(int index,int value);
 int   SettingSetGlobal_i(int index,int value);
 int   SettingSetGlobal_f(int index,float value);
+
+int   SettingSetSmart_i(CSetting *set1,CSetting *set2,int index, int value);
 /* more to come */
 
 int SettingGetIfDefined_i(CSetting *set1,int index,int *value);
@@ -476,10 +478,43 @@ CSetting *SettingNewFromPyList(PyObject *list);
 #define cSetting_dot_normals                332
 #define cSetting_session_migration          333
 #define cSetting_mesh_normals               334
+#define cSetting_mesh_type                  335
+#define cSetting_dot_lighting               336
+#define cSetting_mesh_lighting              337
+#define cSetting_surface_solvent            338 
+#define cSetting_triangle_max_passes        339
+#define cSetting_ray_interior_reflect       340
+#define cSetting_internal_gui_mode          341
+#define cSetting_surface_carve_selection    342
+#define cSetting_surface_carve_state        343
+#define cSetting_surface_carve_cutoff       344
+#define cSetting_surface_clear_selection    345
+#define cSetting_surface_clear_state        346
+#define cSetting_surface_clear_cutoff       347
+#define cSetting_surface_trim_cutoff        348
+#define cSetting_surface_trim_factor        349
+#define cSetting_ray_max_passes             350
+#define cSetting_active_selections          351
+#define cSetting_ray_transparency_contrast  352
+#define cSetting_seq_view                   353
+#define cSetting_mouse_selection_mode       354
+#define cSetting_seq_view_label_spacing     355
+#define cSetting_seq_view_label_start       356
+#define cSetting_seq_view_format            357
+#define cSetting_seq_view_location          358
+#define cSetting_seq_view_overlay           359
+#define cSetting_auto_classify_atoms        360
+#define cSetting_cartoon_nucleic_acid_mode  361
+#define cSetting_seq_view_color             362
+#define cSetting_seq_view_label_mode        363
+#define cSetting_surface_ramp_above_mode    364
+#define cSetting_stereo                     365
+#define cSetting_wizard_prompt_mode         366
+#define cSetting_coulomb_cutoff             367
 
 /* cSetting_ss_INIT must always be last setting_index +1 */
 
-#define cSetting_INIT                        335
+#define cSetting_INIT                       368
 
 #endif
 
