@@ -37,6 +37,14 @@ Z* -------------------------------------------------------------------
 #define cWizTypeButton 2
 #define cWizTypePopUp  3
 
+#define cWizEventPick    1
+#define cWizEventSelect  2
+#define cWizEventKey     4
+#define cWizEventSpecial 8
+#define cWizEventScene   16
+#define cWizEventState   32
+#define cWizEventFrame   64
+
 typedef struct {
   int type;
   WordType text;
@@ -50,6 +58,11 @@ typedef struct {
   int NLine;
   int Stack;
   int Pressed;
+  int EventMask;
+  int Dirty;
+  int LastUpdatedState;
+  int LastUpdatedFrame;
+  
 }  CWizard;
 
 CWizard Wizard;
@@ -58,6 +71,40 @@ CWizard Wizard;
 #define cWizardTopMargin 0
 #define cWizardClickOffset 2
 
+void WizardDirty(void)
+{
+  CWizard *I=&Wizard;
+  I->Dirty=true;
+  OrthoDirty();
+}
+
+int WizardUpdate(void)
+{
+  CWizard *I=&Wizard;
+  int result = false;
+
+  {
+    int frame = SettingGetGlobal_i(cSetting_frame);
+    int state = SettingGetGlobal_i(cSetting_state);
+    if(frame!=I->LastUpdatedFrame) {
+      I->LastUpdatedFrame = frame;
+      WizardDoFrame();
+    }
+    if(state!=I->LastUpdatedState) {
+      I->LastUpdatedState = state;
+      WizardDoState();
+    }
+  }
+
+  if(I->Dirty) {
+    WizardRefresh();
+    I->Dirty=false;
+    result = true;
+  }
+
+  return result;
+}
+
 void WizardPurgeStack(void)
 {
   int blocked;
@@ -70,22 +117,25 @@ void WizardPurgeStack(void)
   PAutoUnblock(blocked);
 
 }
-void WizardDoSelect(char *name)
+int WizardDoSelect(char *name)
 {
   OrthoLineType buf;
   CWizard *I=&Wizard;
+  int result = false;
 
-  if(I->Stack>=0)
-    if(I->Wiz[I->Stack]) {
-      sprintf(buf,"cmd.get_wizard().do_select('''%s''')",name);
-      PLog(buf,cPLog_pym);
-      PBlock(); 
-      if(PyObject_HasAttrString(I->Wiz[I->Stack],"do_select")) {
-        PXDecRef(PyObject_CallMethod(I->Wiz[I->Stack],"do_select","s",name));
+  if(I->EventMask & cWizEventSelect) 
+    if(I->Stack>=0)
+      if(I->Wiz[I->Stack]) {
+        sprintf(buf,"cmd.get_wizard().do_select('''%s''')",name);
+        PLog(buf,cPLog_pym);
+        PBlock(); 
+        if(PyObject_HasAttrString(I->Wiz[I->Stack],"do_select")) {
+          result = PTruthCallStr(I->Wiz[I->Stack],"do_select",name);
         if(PyErr_Occurred()) PyErr_Print();
+        }
+        PUnblock();
       }
-    PUnblock();
-  }
+  return result;
 }
 /*========================================================================*/
 void WizardRefresh(void)
@@ -119,7 +169,16 @@ void WizardRefresh(void)
   I->NLine = 0;
   if(I->Stack>=0)
     if(I->Wiz[I->Stack]) {
+
+      I->EventMask = cWizEventPick + cWizEventSelect;
       
+      if(PyObject_HasAttrString(I->Wiz[I->Stack],"get_event_mask")) {      
+        i = PyObject_CallMethod(I->Wiz[I->Stack],"get_event_mask","");
+        if(!PConvPyIntToInt(i,&I->EventMask))
+          I->EventMask = cWizEventPick + cWizEventSelect;
+        Py_XDECREF(i);
+      }
+
       if(PyObject_HasAttrString(I->Wiz[I->Stack],"get_panel")) {
         P_list = PyObject_CallMethod(I->Wiz[I->Stack],"get_panel","");
         if(PyErr_Occurred()) PyErr_Print();
@@ -210,27 +269,151 @@ Block *WizardGetBlock(void)
   {return(I->Block);}
 }
 /*========================================================================*/
-void WizardDoPick(int bondFlag)
+int WizardDoPick(int bondFlag)
 {
   CWizard *I=&Wizard;
-  if(I->Stack>=0) 
-    if(I->Wiz[I->Stack]) {
-      if(bondFlag)
-        PLog("cmd.get_wizard().do_pick(1)",cPLog_pym);
-      else
-        PLog("cmd.get_wizard().do_pick(0)",cPLog_pym);
-      
-      PBlock(); 
-      if(I->Stack>=0)
-        if(I->Wiz[I->Stack]) {
-          if(PyObject_HasAttrString(I->Wiz[I->Stack],"do_pick")) {
-            PXDecRef(PyObject_CallMethod(I->Wiz[I->Stack],"do_pick","i",bondFlag));
-            if(PyErr_Occurred()) PyErr_Print();
+  int result=false;
+  if(I->EventMask & cWizEventPick) 
+    if(I->Stack>=0) 
+      if(I->Wiz[I->Stack]) {
+        if(bondFlag)
+          PLog("cmd.get_wizard().do_pick(1)",cPLog_pym);
+        else
+          PLog("cmd.get_wizard().do_pick(0)",cPLog_pym);
+        
+        PBlock(); 
+        if(I->Stack>=0)
+          if(I->Wiz[I->Stack]) {
+            if(PyObject_HasAttrString(I->Wiz[I->Stack],"do_pick")) {
+              result = PTruthCallStr1i(I->Wiz[I->Stack],"do_pick",bondFlag);
+              if(PyErr_Occurred()) PyErr_Print();
+            }
           }
-        }
-      PUnblock();
-    }
+        PUnblock();
+      }
+  return result;
 }
+
+int WizardDoKey(unsigned char k, int x, int y, int mod)
+{
+  CWizard *I=&Wizard;
+  int result=false;
+  if(I->EventMask & cWizEventKey) 
+    if(I->Stack>=0) 
+      if(I->Wiz[I->Stack]) {
+        OrthoLineType buffer;
+        sprintf(buffer,"cmd.get_wizard().do_key(%d,%d,%d,%d)",k,x,y,mod);
+        PLog(buffer,cPLog_pym);
+        PBlock(); 
+        if(I->Stack>=0)
+          if(I->Wiz[I->Stack]) {
+            if(PyObject_HasAttrString(I->Wiz[I->Stack],"do_key")) {
+              result = PTruthCallStr4i(I->Wiz[I->Stack],"do_key",k,x,y,mod);
+              if(PyErr_Occurred()) PyErr_Print();
+            }
+          }
+        PUnblock();
+      }
+  return result;
+}
+
+int WizardDoScene(void)
+{
+  CWizard *I=&Wizard;
+  int result=false;
+  if(I->EventMask & cWizEventScene) 
+    if(I->Stack>=0) 
+      if(I->Wiz[I->Stack]) {
+        OrthoLineType buffer;
+        sprintf(buffer,"cmd.get_wizard().do_scene()");
+        PLog(buffer,cPLog_pym);
+        PBlock(); 
+        if(I->Stack>=0)
+          if(I->Wiz[I->Stack]) {
+            if(PyObject_HasAttrString(I->Wiz[I->Stack],"do_scene")) {
+              result = PTruthCallStr0(I->Wiz[I->Stack],"do_scene");
+              if(PyErr_Occurred()) PyErr_Print();
+            }
+          }
+        PUnblock();
+      }
+  return result;
+}
+
+
+int WizardDoState(void)
+{
+  CWizard *I=&Wizard;
+  int result=false;
+  if(I->EventMask & cWizEventState) 
+    if(I->Stack>=0) 
+      if(I->Wiz[I->Stack]) {
+        OrthoLineType buffer;
+        int state = SettingGetGlobal_i(cSetting_state) + 1;
+        sprintf(buffer,"cmd.get_wizard().do_state(%d)",state);
+        PLog(buffer,cPLog_pym);
+        PBlock(); 
+        if(I->Stack>=0)
+          if(I->Wiz[I->Stack]) {
+            if(PyObject_HasAttrString(I->Wiz[I->Stack],"do_state")) {
+              result = PTruthCallStr1i(I->Wiz[I->Stack],"do_state",state);
+              if(PyErr_Occurred()) PyErr_Print();
+            }
+          }
+        PUnblock();
+      }
+  return result;
+}
+
+
+int WizardDoFrame(void)
+{
+  CWizard *I=&Wizard;
+  int result=false;
+  if(I->EventMask & cWizEventFrame) 
+    if(I->Stack>=0) 
+      if(I->Wiz[I->Stack]) {
+        OrthoLineType buffer;
+        int frame = SettingGetGlobal_i(cSetting_frame) + 1;
+        sprintf(buffer,"cmd.get_wizard().do_frame(%d)",frame);
+        PLog(buffer,cPLog_pym);
+        PBlock(); 
+        if(I->Stack>=0)
+          if(I->Wiz[I->Stack]) {
+            if(PyObject_HasAttrString(I->Wiz[I->Stack],"do_frame")) {
+              result = PTruthCallStr1i(I->Wiz[I->Stack],"do_frame",frame);
+              if(PyErr_Occurred()) PyErr_Print();
+            }
+          }
+        PUnblock();
+      }
+  return result;
+}
+
+int WizardDoSpecial(int k, int x, int y, int mod)
+{
+  CWizard *I=&Wizard;
+  int result=false;
+
+  if(I->EventMask & cWizEventSpecial) 
+    if(I->Stack>=0) 
+      if(I->Wiz[I->Stack]) {
+        OrthoLineType buffer;
+        sprintf(buffer,"cmd.get_wizard().do_special(%d,%d,%d,%d)",k,x,y,mod);
+        PLog(buffer,cPLog_pym);
+        PBlock(); 
+        if(I->Stack>=0)
+          if(I->Wiz[I->Stack]) {
+            if(PyObject_HasAttrString(I->Wiz[I->Stack],"do_special")) {
+              result = PTruthCallStr4i(I->Wiz[I->Stack],"do_special",k,x,y,mod);
+              if(PyErr_Occurred()) PyErr_Print();
+            }
+          }
+        PUnblock();
+      }
+  return result;
+}
+
 /*========================================================================*/
 static int WizardClick(Block *block,int button,int x,int y,int mod)
 {
@@ -425,7 +608,8 @@ static void WizardDraw(Block *block)
     BlockFill(I->Block);
     
     glColor3fv(I->Block->TextColor);
-
+    
+    
     x = I->Block->rect.left+cWizardLeftMargin;
     y = (I->Block->rect.top-LineHeight)-cWizardTopMargin;
 
@@ -546,12 +730,15 @@ void WizardInit(void)
   I->Block->TextColor[1]=1.0F;
   I->Block->TextColor[2]=0.2F;
 
+  I->LastUpdatedState = -1;
+  I->LastUpdatedFrame = -1;
+
   OrthoAttach(I->Block,cOrthoTool);
 
   I->Line = VLAlloc(WizardLine,10);
   I->NLine = 0;
   I->Pressed = -1;
-
+  I->EventMask = 0;
   I->Stack = -1;
   I->Wiz = VLAlloc(PyObject*,10);
 }
