@@ -27,6 +27,516 @@ Z* -------------------------------------------------------------------
 
 
 static int NColor,CarbColor,HColor,OColor,SColor,MColor,IColor;
+/*========================================================================*/
+void AtomInfoGetPDB3LetHydroName(char *resn, char *iname, char *oname) 
+{
+  oname[0]=' ';	
+  strcpy(oname+1,iname);
+
+  switch(resn[0]) {
+  case 'A':
+    switch(resn[1]) {
+    case 'L': 
+      if(resn[2]=='A')
+        if((iname[0]=='H')&&
+           (iname[1]=='B')&&
+           ((iname[2]>='0')&&(iname[2]<='9'))) {
+          oname[0]=iname[2]; oname[1]=iname[0]; oname[2]=iname[1]; oname[3]=0;
+        }
+      break;
+    case 'R': 
+      if(resn[2]=='G')
+        if((iname[0]=='H')&&
+           ((iname[1]=='B')||(iname[1]=='G')||(iname[1]=='D'))&&
+           ((iname[2]>='0')&&(iname[2]<='9'))) {
+          oname[0]=iname[2]; oname[1]=iname[0]; oname[2]=iname[1]; oname[3]=0;
+        }
+      break;
+    case 'S': 
+      switch(resn[2]) {
+      case 'P':
+        if((iname[0]=='H')&&
+           (iname[1]=='B')&&
+           ((iname[2]>='0')&&(iname[2]<='9'))) {
+          oname[0]=iname[2]; oname[1]=iname[0]; oname[2]=iname[1]; oname[3]=0;
+        }
+        break;
+      case 'N':
+        if((iname[0]=='H')&&
+           (iname[1]=='B')&&
+           ((iname[2]>='0')&&(iname[2]<='9'))) {
+          oname[0]=iname[2]; oname[1]=iname[0]; oname[2]=iname[1]; oname[3]=0;
+        }
+        break;
+      }
+      break;
+    }
+    break;
+  case 'C':
+    switch(resn[1]) {
+    case 'Y': 
+      switch(resn[2]) {
+      case 'S':
+      case 'X':
+        if((iname[0]=='H')&&
+           (iname[1]=='B')&&
+           ((iname[2]>='0')&&(iname[2]<='9'))) {
+          oname[0]=iname[2]; oname[1]=iname[0]; oname[2]=iname[1]; oname[3]=0;
+        }
+        break;
+      }
+      break;
+    }
+    break;
+  case 'G':
+    switch(resn[1]) {
+    case 'L': 
+      switch(resn[2]) {
+      case 'N':
+        if((iname[0]=='H')&&
+           ((iname[1]=='B')||(iname[1]=='G'))&&
+           ((iname[2]>='0')&&(iname[2]<='9'))) {
+          oname[0]=iname[2]; oname[1]=iname[0]; oname[2]=iname[1]; oname[3]=0;
+        }
+        break;
+      case 'U':
+        if((iname[0]=='H')&&
+           ((iname[1]=='B')||(iname[1]=='G'))&&
+           ((iname[2]>='0')&&(iname[2]<='9'))) {
+          oname[0]=iname[2]; oname[1]=iname[0]; oname[2]=iname[1]; oname[3]=0;
+        }
+        break;
+      case 'Y':
+        if((iname[0]=='H')&&
+           (iname[1]=='A')&&
+           ((iname[2]>='0')&&(iname[2]<='9'))) {
+          oname[0]=iname[2]; oname[1]=iname[0]; oname[2]=iname[1]; oname[3]=0;
+        }
+        break;
+      }
+    }
+    break;
+  case 'H':
+    switch(resn[1]) {
+    case 'I': 
+      switch(resn[2]) {
+      case 'S':
+      case 'D':
+      case 'E':
+      case 'P':
+        if((iname[0]=='H')&&
+           (iname[1]=='B')&&
+           ((iname[2]>='0')&&(iname[2]<='9'))) {
+          oname[0]=iname[2]; oname[1]=iname[0]; oname[2]=iname[1]; oname[3]=0;
+        }
+        break;
+      }
+      break;
+    case 'O': 
+      switch(resn[2]) {
+      case 'H':
+        break;
+      }
+      break;
+    case '2': 
+      switch(resn[2]) {
+      case 'O':
+        break;
+      }
+      break;
+    }
+  case 'I':
+    switch(resn[1]) {
+    case 'L': 
+      switch(resn[2]) {
+      case 'E':
+        break;
+      }
+    }
+    break;
+  case 'L':
+    switch(resn[1]) {
+    case 'E': 
+      switch(resn[2]) {
+      case 'U':
+        if((iname[0]=='H')&&
+           (iname[1]=='B')&&
+           ((iname[2]>='0')&&(iname[2]<='9'))) {
+          oname[0]=iname[2]; oname[1]=iname[0]; oname[2]=iname[1]; oname[3]=0;
+        }
+        break;
+      }
+      break;
+    case 'Y': 
+      switch(resn[2]) {
+      case 'S':
+        if((iname[0]=='H')&&
+           ((iname[1]=='B')||(iname[1]=='G')||(iname[1]=='D')||(iname[1]=='E')||(iname[1]=='Z'))&&
+           ((iname[2]>='0')&&(iname[2]<='9'))) {
+          oname[0]=iname[2]; oname[1]=iname[0]; oname[2]=iname[1]; oname[3]=0;
+        }
+        break;
+      }
+      break;
+    }
+    break;
+  case 'M':
+    switch(resn[1]) {
+    case 'E': 
+      switch(resn[2]) {
+      case 'T':
+        if((iname[0]=='H')&&
+           ((iname[1]=='B')||(iname[1]=='G')||(iname[1]=='E'))&&
+           ((iname[2]>='0')&&(iname[2]<='9'))) {
+          oname[0]=iname[2]; oname[1]=iname[0]; oname[2]=iname[1]; oname[3]=0;
+        }
+        break;
+      }
+    }
+    break;
+  case 'P':
+    switch(resn[1]) {
+    case 'H':
+      switch(resn[2]) {
+      case 'E':
+        if((iname[0]=='H')&&
+           (iname[1]=='B')&&
+           ((iname[2]>='0')&&(iname[2]<='9'))) {
+          oname[0]=iname[2]; oname[1]=iname[0]; oname[2]=iname[1]; oname[3]=0;
+        }
+        break;
+      }
+      break;     
+    case 'R': 
+      switch(resn[2]) {
+      case 'O':
+        if((iname[0]=='H')&&
+           ((iname[1]=='B')||(iname[1]=='G')||(iname[1]=='D'))&&
+           ((iname[2]>='0')&&(iname[2]<='9'))) {
+          oname[0]=iname[2]; oname[1]=iname[0]; oname[2]=iname[1]; oname[3]=0;
+        }
+        break;
+      }
+      break;
+    }
+    break;
+  case 'S':
+    switch(resn[1]) {
+    case 'E': 
+      switch(resn[2]) {
+      case 'R':
+        if((iname[0]=='H')&&
+           (iname[1]=='B')&&
+           ((iname[2]>='0')&&(iname[2]<='9'))) {
+          oname[0]=iname[2]; oname[1]=iname[0]; oname[2]=iname[1]; oname[3]=0;
+        }
+        break;
+      }
+      break;
+    }
+    break;
+  case 'T':
+    switch(resn[1]) {
+    case 'H': 
+      switch(resn[2]) {
+      case 'R':
+        break;
+      }
+      break;
+    case 'I': 
+      switch(resn[2]) {
+      case 'P':
+        break;
+      }
+      break;
+    case 'R': 
+      switch(resn[2]) {
+      case 'P':
+        if((iname[0]=='H')&&
+           (iname[1]=='B')&&
+           ((iname[2]>='0')&&(iname[2]<='9'))) {
+          oname[0]=iname[2]; oname[1]=iname[0]; oname[2]=iname[1]; oname[3]=0;
+        }
+        break;
+      }
+      break;
+    case 'Y': 
+      switch(resn[2]) {
+      case 'R':
+        if((iname[0]=='H')&&
+           (iname[1]=='B')&&
+           ((iname[2]>='0')&&(iname[2]<='9'))) {
+          oname[0]=iname[2]; oname[1]=iname[0]; oname[2]=iname[1]; oname[3]=0;
+        }
+        break;
+      }
+      break;
+    }
+    break;
+  case 'V':
+    switch(resn[1]) {
+    case 'A': 
+      switch(resn[2]) {
+      case 'L':
+        break;
+      }
+      break;
+    }
+    break;
+  case 'W':
+    switch(resn[1]) {
+    case 'A': 
+      switch(resn[2]) {
+      case 'T':
+        break;
+      }
+      break;
+    }
+    break;
+  }
+}
+
+int AtomInfoKnownWaterResName(char *resn) 
+{
+
+  switch(resn[0]) {
+  case 'H':
+    switch(resn[1]) {
+    case 'O': 
+      switch(resn[2]) {
+      case 'H':
+        return true;
+        break;
+      }
+      break;
+    case '2': 
+      switch(resn[2]) {
+      case 'O':
+        return true;
+        break;
+      }
+      break;
+    }
+  case 'T':
+    switch(resn[1]) {
+    case 'I': 
+      switch(resn[2]) {
+      case 'P':
+        return true;
+        break;
+      }
+      break;
+    }
+    break;
+  case 'W':
+    switch(resn[1]) {
+    case 'A': 
+      switch(resn[2]) {
+      case 'T':
+        return true;
+        break;
+      }
+      break;
+    }
+    break;
+  }
+  return false;
+}
+
+int AtomInfoKnownPolymerResName(char *resn) 
+{
+  switch(resn[0]) {
+  case 'A':
+    switch(resn[1]) {
+    case 0:
+      return true;
+    case 'L': 
+      if(resn[2]=='A')
+        return true;
+      break;
+    case 'R': 
+      if(resn[2]=='G')
+        return true;
+      break;
+    case 'S': 
+      switch(resn[2]) {
+      case 'P':
+        return true;
+        break;
+      case 'N':
+        return true;
+        break;
+      }
+      break;
+    }
+    break;
+  case 'C':
+    switch(resn[1]) {
+    case 0:
+      return true;
+    case 'Y': 
+      switch(resn[2]) {
+      case 'S':
+      case 'X':
+        return true;
+        break;
+      }
+      break;
+    }
+    break;
+  case 'G':
+    switch(resn[1]) {
+    case 0:
+      return true;
+      break;
+    case 'L': 
+      switch(resn[2]) {
+      case 'N':
+        return true;
+        break;
+      case 'U':
+        return true;
+        break;
+      case 'Y':
+        return true;
+        break;
+      }
+    }
+    break;
+  case 'H':
+    switch(resn[1]) {
+    case 'I': 
+      switch(resn[2]) {
+      case 'S':
+      case 'D':
+      case 'E':
+      case 'P':
+        return true;
+        break;
+      }
+      break;
+    }
+  case 'I':
+    switch(resn[1]) {
+    case 'L': 
+      switch(resn[2]) {
+      case 'E':
+        return true;
+        break;
+      }
+    }
+    break;
+  case 'L':
+    switch(resn[1]) {
+    case 'E': 
+      switch(resn[2]) {
+      case 'U':
+        return true;
+        break;
+      }
+      break;
+    case 'Y': 
+      switch(resn[2]) {
+      case 'S':
+        return true;
+        break;
+      }
+      break;
+    }
+    break;
+  case 'M':
+    switch(resn[1]) {
+    case 'E': 
+      switch(resn[2]) {
+      case 'T':
+        return true;
+        break;
+      }
+    case 'S':
+      switch(resn[2]) {
+      case 'E':
+        return true;
+        break;
+      }
+    }
+    break;
+  case 'P':
+    switch(resn[1]) {
+    case 'H':
+      switch(resn[2]) {
+      case 'E':
+        return true;
+        break;
+      }
+      break;     
+    case 'R': 
+      switch(resn[2]) {
+      case 'O':
+        return true;
+        break;
+      }
+      break;
+    }
+    break;
+  case 'S':
+    switch(resn[1]) {
+    case 'E': 
+      switch(resn[2]) {
+      case 'R':
+        return true;
+        break;
+      }
+      break;
+    }
+    break;
+  case 'T':
+    switch(resn[1]) {
+    case 0:
+      return true;
+    case 'H': 
+      switch(resn[2]) {
+      case 'R':
+        return true;
+        break;
+      }
+      break;
+    case 'R': 
+      switch(resn[2]) {
+      case 'P':
+        return true;
+        break;
+      }
+      break;
+    case 'Y': 
+      switch(resn[2]) {
+      case 'R':
+        return true;
+        break;
+      }
+      break;
+    }
+    break;
+  case 'U':
+    switch(resn[1]) {
+    case 0:
+      return true;
+      break;
+    } 
+    break;
+  case 'V':
+    switch(resn[1]) {
+    case 'A': 
+      switch(resn[2]) {
+      case 'L':
+        return true;
+        break;
+      }
+      break;
+    }
+    break;
+  }
+  return false;
+}
 
 
 /*========================================================================*/
@@ -36,12 +546,26 @@ int AtomInfoInOrigOrder(AtomInfoType *atom,int atom1,int atom2);
 int AtomInfoInOrderIgnoreHet(AtomInfoType *atom,int atom1,int atom2);
 int AtomNameCompare(char *name1,char *name2);
 
+int AtomResvFromResi(char *resi)
+{
+  int result = 1;
+  char *start = resi;
+  while(*start) {
+    if(sscanf(start,"%d",&result))
+      break;
+    else
+      result = 1;
+    start++;
+  }
+  return result;
+}
+
 /*========================================================================*/
 PyObject *AtomInfoAsPyList(AtomInfoType *I)
 {
   PyObject *result = NULL;
 
-  result = PyList_New(34);
+  result = PyList_New(36);
   PyList_SetItem(result, 0,PyInt_FromLong(I->resv));
   PyList_SetItem(result, 1,PyString_FromString(I->chain));
   PyList_SetItem(result, 2,PyString_FromString(I->alt));
@@ -76,7 +600,8 @@ PyObject *AtomInfoAsPyList(AtomInfoType *I)
   PyList_SetItem(result,31,PyInt_FromLong((int)I->protons));
   PyList_SetItem(result,32,PyInt_FromLong(I->sculpt_id));
   PyList_SetItem(result,33,PyInt_FromLong(I->stereo));
-
+  PyList_SetItem(result,34,PyInt_FromLong(I->discrete_state));
+  PyList_SetItem(result,35,PyFloat_FromDouble(I->bohr_radius));
   return(PConvAutoNone(result));
 }
 
@@ -84,7 +609,9 @@ int AtomInfoFromPyList(AtomInfoType *I,PyObject *list)
 {
   int ok=true;
   int hetatm;
+  int ll = 0;
   if(ok) ok = PyList_Check(list);
+  if(ok) ll = PyList_Size(list);
   if(ok) ok = PConvPyIntToInt(PyList_GetItem(list, 0),&I->resv);
   if(ok) ok = PConvPyStrToStr(PyList_GetItem(list, 1),I->chain,sizeof(Chain));
   if(ok) ok = PConvPyStrToStr(PyList_GetItem(list, 2),I->alt,sizeof(Chain));
@@ -120,6 +647,8 @@ int AtomInfoFromPyList(AtomInfoType *I,PyObject *list)
   if(ok) ok = PConvPyIntToChar(PyList_GetItem(list,31),(char*)&I->protons);
   if(ok) ok = PConvPyIntToInt(PyList_GetItem(list,32),&I->sculpt_id);
   if(ok) ok = PConvPyIntToInt(PyList_GetItem(list,33),&I->stereo);
+  if(ok&&(ll>34)) ok = PConvPyIntToInt(PyList_GetItem(list,34),&I->discrete_state);
+  if(ok&&(ll>35)) ok = PConvPyFloatToFloat(PyList_GetItem(list,35),&I->bohr_radius);
   return(ok);
 }
 
@@ -134,6 +663,7 @@ void AtomInfoCombine(AtomInfoType *dst,AtomInfoType *src,int mask)
   if(mask&cAIC_b) dst->b = src->b;
   if(mask&cAIC_q) dst->q = src->q;
   if(mask&cAIC_id) dst->id = src->id;
+  if(mask&cAIC_state) dst->discrete_state = src->discrete_state;
   dst->temp1 = src->temp1;
   dst->sculpt_id = src->sculpt_id;
   /* keep all existing names, identifiers, etc. */
@@ -637,18 +1167,24 @@ int AtomInfoCompare(AtomInfoType *at1,AtomInfoType *at2)
           if(!wc) {
             wc=WordCompare(at1->resn,at2->resn,true);
             if(!wc) {
-              if(at1->priority==at2->priority) {
-                if(at1->alt[0]==at2->alt[0]) {
-                  result=AtomNameCompare(at1->name,at2->name);
-                } else if((!at2->alt[0])||(at1->alt[0]&&((at1->alt[0]<at2->alt[0])))) {
+              if(at1->discrete_state==at2->discrete_state) {
+                if(at1->priority==at2->priority) {
+                  if(at1->alt[0]==at2->alt[0]) {
+                    result=AtomNameCompare(at1->name,at2->name);
+                  } else if((!at2->alt[0])||(at1->alt[0]&&((at1->alt[0]<at2->alt[0])))) {
+                    result=-1;
+                  } else {
+                    result=1;
+                  }
+                } else if(at1->priority<at2->priority) {
                   result=-1;
                 } else {
                   result=1;
                 }
-              } else if(at1->priority<at2->priority) {
-                result=-1;
+              } else if(at1->discrete_state<at2->discrete_state) {
+                result = -1;
               } else {
-                result=1;
+                result = 1;
               }
             } else {
               result=wc;
@@ -713,18 +1249,24 @@ int AtomInfoCompareIgnoreHet(AtomInfoType *at1,AtomInfoType *at2)
         if(!wc) {
           wc=WordCompare(at1->resn,at2->resn,true);
           if(!wc) {
-            if(at1->priority==at2->priority) {
-              if(at1->alt[0]==at2->alt[0]) {
-                result=AtomNameCompare(at1->name,at2->name);
-              } else if((!at2->alt[0])||(at1->alt[0]&&((at1->alt[0]<at2->alt[0])))) {
+            if(at1->discrete_state==at2->discrete_state) {
+              if(at1->priority==at2->priority) {
+                if(at1->alt[0]==at2->alt[0]) {
+                  result=AtomNameCompare(at1->name,at2->name);
+                } else if((!at2->alt[0])||(at1->alt[0]&&((at1->alt[0]<at2->alt[0])))) {
+                  result=-1;
+                } else {
+                  result=1;
+                }
+              } else if(at1->priority<at2->priority) {
                 result=-1;
               } else {
                 result=1;
               }
-            } else if(at1->priority<at2->priority) {
-              result=-1;
+            } else if(at1->discrete_state<at2->discrete_state) {
+              result = -1;
             } else {
-              result=1;
+              result = 1;
             }
           } else {
             result=wc;
@@ -810,9 +1352,11 @@ int AtomInfoSameResidue(AtomInfoType *at1,AtomInfoType *at2)
   if(at1->hetatm==at2->hetatm)
     if(at1->chain[0]==at2->chain[0])
       if(at1->resv==at2->resv)
-        if(WordMatch(at1->resi,at2->resi,true)<0) 
-          if(WordMatch(at1->segi,at2->segi,true)<0) 
-            return 1;
+        if(at1->discrete_state==at2->discrete_state)
+          if(WordMatch(at1->resi,at2->resi,true)<0) 
+            if(WordMatch(at1->segi,at2->segi,true)<0) 
+              if(WordMatch(at1->resn,at2->resn,true)<0) 
+                return 1;
   return 0;
 }
 
@@ -822,9 +1366,28 @@ int AtomInfoSameResidueP(AtomInfoType *at1,AtomInfoType *at2)
     if(at1->hetatm==at2->hetatm)
       if(at1->chain[0]==at2->chain[0])
         if(at1->resv==at2->resv)
-          if(WordMatch(at1->resi,at2->resi,true)<0) 
-            if(WordMatch(at1->segi,at2->segi,true)<0) 
-              return 1;
+          if(at1->discrete_state==at2->discrete_state)
+            if(WordMatch(at1->resi,at2->resi,true)<0) 
+              if(WordMatch(at1->segi,at2->segi,true)<0) 
+                if(WordMatch(at1->resn,at2->resn,true)<0) 
+                  return 1;
+  return 0;
+}
+
+int AtomInfoSameChainP(AtomInfoType *at1,AtomInfoType *at2)
+{
+  if(at1&&at2)
+    if(at1->chain[0]==at2->chain[0])
+        if(WordMatch(at1->segi,at2->segi,true)<0) 
+          return 1;
+  return 0;
+}
+
+int AtomInfoSameSegmentP(AtomInfoType *at1,AtomInfoType *at2)
+{
+  if(at1&&at2)
+    if(WordMatch(at1->segi,at2->segi,true)<0) 
+      return 1;
   return 0;
 }
 
@@ -1185,7 +1748,17 @@ void AtomInfoAssignParameters(AtomInfoType *I)
   while((*e>='0')&&(*e<='9')&&(*(e+1))) e++;
   switch ( *e )
     {
-    case 'N' : vdw=1.55F;  break;
+    case 'N':
+      switch (*(e+1)) {
+      case 0:
+        vdw=1.55F;  break;
+      case 'a':
+      case 'A':
+        vdw=2.27F; break;
+      default:
+        vdw = 1.80F; break;
+      }
+      break;
     case 'C' :	
       switch (*(e+1)) 
         {
@@ -1205,7 +1778,17 @@ void AtomInfoAssignParameters(AtomInfoType *I)
     case 'I' :	vdw=1.98F; break;
     case 'P' :	vdw=1.80F; break;
     case 'B' :	vdw=1.85F; break; /* incl B, BR */
-    case 'S' :	vdw=1.80F; break;
+    case 'S' :	
+      switch (*(e+1)) {
+      case 'e':
+      case 'E':
+        vdw=1.9F;
+        break;
+      default:
+        vdw=1.80F; break;
+
+      }
+      break;
     case 'F' : 
       switch (*(e+1))
         {
@@ -1219,10 +1802,38 @@ void AtomInfoAssignParameters(AtomInfoType *I)
           vdw=1.35F; break;
         }
       break;
+    case 'K':
+      switch (*(e+1)) {
+      case 0:
+        vdw=2.75F; break;
+      default:
+        vdw = 1.80F; break;
+      }
+      break;
+    case 'M':
+      switch (*(e+1)) {
+      case 'n':
+      case 'N':
+      case 'g':
+      case 'G':
+        vdw=1.73F; break;
+      default:
+        vdw = 1.80F; break;
+      }
+      break;
     case 'H' :
     case 'D' :
       vdw = 1.2F; /* WLD */
       break;
+    case 'Z':
+      switch (*(e+1)) {
+      case 'n':
+      case 'N':
+        vdw=1.39F; break;
+      default:
+        vdw = 1.80F; break;
+      }
+      break;
     default:
       vdw=1.8F;
       break;
@@ -1307,6 +1918,7 @@ void AtomInfoAssignParameters(AtomInfoType *I)
   }
   if(I->vdw==0.0)
     I->vdw = vdw;
+
   /*  printf("I->name %s I->priority %d\n",I->name,I->priority);*/
 }
 
