@@ -54,6 +54,88 @@ typedef struct {
 } MyArrayObject;
 #endif
 
+int ObjectMapStateGetExcludedStats(ObjectMapState *ms,float *vert_vla, float beyond,float within, float *level)
+{
+  double sum=0.0,sumsq=0.0;
+  float mean,stdev;  
+  int cnt = 0;
+  int list_size = VLAGetSize(vert_vla)/3;
+  float cutoff = beyond;
+  MapType *voxelmap = NULL;
+
+  if(cutoff<within)
+    cutoff = within;
+
+  if(list_size) 
+    voxelmap=MapNew(-cutoff,vert_vla,list_size,NULL);
+  if(voxelmap||(!list_size)) {
+    int a,b,c;
+    int h,k,l,i,j;
+    int *fdim = ms->FDim;
+    float *v,f_val;
+    int within_flag, within_default=false;
+    int beyond_flag;
+    
+    Isofield *field = ms->Field;
+
+    if(list_size)
+      MapSetupExpress(voxelmap);  
+
+    within_flag=true;
+    beyond_flag=true;
+
+    if(within<R_SMALL4)
+      within_default = true;
+    for(c=0;c<fdim[2];c++) {
+      for(b=0;b<fdim[1];b++) {
+        for(a=0;a<fdim[0];a++) {
+          if(list_size) {
+            within_flag = within_default;
+            beyond_flag = true;
+
+            v = F4Ptr(field->points,a,b,c,0);
+            
+            MapLocus(voxelmap,v,&h,&k,&l);
+            i=*(MapEStart(voxelmap,h,k,l));
+            if(i) {
+              j=voxelmap->EList[i++];
+              while(j>=0) {
+                if(!within_flag) {
+                  if(within3f(vert_vla+3*j,v,within)) {                  
+                    within_flag=true;
+                  }
+                }
+                if(within3f(vert_vla+3*j,v,beyond)) {
+                  beyond_flag=false;
+                  break;
+                }
+                j=voxelmap->EList[i++];
+              }
+            }
+          }
+
+          if(within_flag&&beyond_flag) { /* point isn't too close to any vertex */
+            f_val = F3(field->data,a,b,c);
+            sum+=f_val;
+            sumsq+=(f_val*f_val);
+            cnt++;
+          }
+        }
+      }
+    }
+    if(voxelmap) 
+      MapFree(voxelmap);
+  }
+  if(cnt) {
+    mean = (float)(sum/cnt);
+    stdev = (float)sqrt1d((sumsq - (sum*sum/cnt))/(cnt));
+    level[1] = mean;
+    level[0] = mean-stdev;
+    level[2] = mean+stdev;
+  }
+  return cnt;
+}
+
 int ObjectMapInterpolate(ObjectMap *I,int state,float *array,float *result,int n)
 {
   int ok=false;
@@ -84,7 +166,7 @@ static int ObjectMapStateDouble(ObjectMapState *ms)
   case cMapSourceBRIX:
   case cMapSourceGRD:
     for(a=0;a<3;a++) {
-      div[a]=ms->Div[a]*2-1;
+      div[a]=ms->Div[a]*2;
       min[a]=ms->Min[a]*2;
       max[a]=ms->Max[a]*2;
       fdim[a]=ms->FDim[a]*2-1;
@@ -130,7 +212,7 @@ static int ObjectMapStateDouble(ObjectMapState *ms)
   case cMapSourceDesc:
 
     for(a=0;a<3;a++) {
-      div[a]=ms->Div[a]*2-1;
+      div[a]=ms->Div[a]*2;
       grid[a]=ms->Grid[a]/2.0F;
       min[a]=ms->Min[a]*2;
       max[a]=ms->Max[a]*2;
@@ -791,12 +873,14 @@ ObjectMapState *ObjectMapNewStateFromDesc(ObjectMap *I,ObjectMapDesc *md,int sta
 
     copy3f(md->MinCorner,ms->Origin);
     copy3f(md->Grid,ms->Grid);
-    for(a=0;a<3;a++) ms->Range[a] = md->Grid[a] * md->Dim[a];
+    for(a=0;a<3;a++) ms->Range[a] = md->Grid[a] * (md->Dim[a]-1);
 
     /* these maps start at zero */
-    for(a=0;a<3;a++) ms->Min[a]=0; 
-    copy3f(md->Dim,ms->Max);
-
+    for(a=0;a<3;a++) {
+      ms->Min[a]=0; 
+      ms->Max[a]=md->Dim[a]-1;
+    }
+    
     /* define corners */
 
     for(a=0;a<8;a++) copy3f(ms->Origin,ms->Corner[a]);
@@ -815,7 +899,7 @@ ObjectMapState *ObjectMapNewStateFromDesc(ObjectMap *I,ObjectMapDesc *md,int sta
         }
       }
     }
-    for(a=0;a<3;a++) ms->FDim[a] = ms->Max[a];
+    for(a=0;a<3;a++) ms->FDim[a] = ms->Max[a]-ms->Min[a]+1;
     ms->FDim[3] = 3; 
 
     ms->Field=IsosurfFieldAlloc(ms->FDim);
@@ -1095,15 +1179,15 @@ int ObjectMapCCP4StrToMap(ObjectMap *I,char *CCP4Str,int bytes,int state) {
 
   ms->FDim[mapc] = nc;
   ms->Min[mapc] = ncstart;
-  ms->Max[mapc] = nc+ncstart;
+  ms->Max[mapc] = nc+ncstart-1;
 
   ms->FDim[mapr] = nr;
   ms->Min[mapr] = nrstart;
-  ms->Max[mapr] = nr+nrstart;
+  ms->Max[mapr] = nr+nrstart-1;
 
   ms->FDim[maps] = ns;
   ms->Min[maps] = nsstart;
-  ms->Max[maps] = ns+nsstart;
+  ms->Max[maps] = ns+nsstart-1;
 
   if(Feedback(FB_ObjectMap,FB_Blather)) {
     dump3i(ms->Div,"ms->Div");
@@ -1257,7 +1341,7 @@ static int ObjectMapPHIStrToMap(ObjectMap *I,char *PHIStr,int bytes,int state) {
 
   ParseNCopy(cc,p,20);
   PRINTFB(FB_ObjectMap,FB_Details)
-    " PHIMapToStr: %s\n",cc
+    " PHIStrToMap: %s\n",cc
     ENDFB;
   p+=20;
   p+=4;
@@ -1265,12 +1349,12 @@ static int ObjectMapPHIStrToMap(ObjectMap *I,char *PHIStr,int bytes,int state) {
   p+=4;
   ParseNCopy(cc,p,10);
   PRINTFB(FB_ObjectMap,FB_Details)
-    " PHIMapToStr: %s\n",cc
+    " PHIStrToMap: %s\n",cc
     ENDFB;
   p+=10;
   ParseNCopy(cc,p,60);
   PRINTFB(FB_ObjectMap,FB_Details)
-    " PHIMapToStr: %s\n",cc
+    " PHIStrToMap: %s\n",cc
     ENDFB;
   p+=60;
   p+=4;
@@ -1297,7 +1381,7 @@ static int ObjectMapPHIStrToMap(ObjectMap *I,char *PHIStr,int bytes,int state) {
     map_dim = 65;
 
   PRINTFB(FB_Details,FB_ObjectMap) 
-      " PHIMapToStr: Map Size %d x %d x %d\n",map_dim,map_dim,map_dim
+      " PHIStrToMap: Map Size %d x %d x %d\n",map_dim,map_dim,map_dim
       ENDFB;
   p+=4;
 
@@ -1350,7 +1434,7 @@ static int ObjectMapPHIStrToMap(ObjectMap *I,char *PHIStr,int bytes,int state) {
   p+=4;
   ParseNCopy(cc,p,16);
   PRINTFB(FB_ObjectMap,FB_Details)
-    " PHIMapToStr: %s\n",cc
+    " PHIStrToMap: %s\n",cc
     ENDFB;
   p+=16;
   p+=4;
@@ -1849,7 +1933,7 @@ static int ObjectMapFLDStrToMap(ObjectMap *I,char *PHIStr,int bytes,int state)
     ms->FDim[3] = 3;
 
     PRINTFB(FB_Details,FB_ObjectMap) 
-      " FLDMapToStr: Map Size %d x %d x %d\n",ms->FDim[0],ms->FDim[1],ms->FDim[2]
+      " FLDStrToMap: Map Size %d x %d x %d\n",ms->FDim[0],ms->FDim[1],ms->FDim[2]
       ENDFB;
       
     for(a=0;a<3;a++) {
@@ -2538,9 +2622,9 @@ static int ObjectMapGRDStrToMap(ObjectMap *I,char *GRDStr,int bytes,int state)
     ms->Div[1]= ms->Div[1] - ms->Min[1];
     ms->Div[2]= ms->Div[2] - ms->Min[2];
 
-    ms->Max[0]=ms->Min[0]+ms->FDim[0];
-    ms->Max[1]=ms->Min[1]+ms->FDim[1];
-    ms->Max[2]=ms->Min[2]+ms->FDim[2];
+    ms->Max[0]=ms->Min[0]+ms->FDim[0]-1;
+    ms->Max[1]=ms->Min[1]+ms->FDim[1]-1;
+    ms->Max[2]=ms->Min[2]+ms->FDim[2]-1;
 
     ms->FDim[3]=3;
 
@@ -2645,7 +2729,7 @@ static int ObjectMapGRDStrToMap(ObjectMap *I,char *GRDStr,int bytes,int state)
     transform33f3f(ms->Crystal->FracToReal,v,ms->ExtentMax);
     
     PRINTFB(FB_Details,FB_ObjectMap) 
-      " GRDXMapToStr: Map Size %d x %d x %d\n",ms->FDim[0],ms->FDim[1],ms->FDim[2]
+      " GRDXStrToMap: Map Size %d x %d x %d\n",ms->FDim[0],ms->FDim[1],ms->FDim[2]
       ENDFB;
     
     ms->Active=true;
@@ -2901,6 +2985,302 @@ ObjectMap *ObjectMapLoadPHIFile(ObjectMap *obj,char *fname,int state)
     }
   return(I);
 
+}
+/*========================================================================*/
+static int ObjectMapDXStrToMap(ObjectMap *I,char *DXStr,int bytes,int state) {
+
+  int n_items = 0;
+
+  char *p;
+  float dens;
+  int a,b,c,d,e;
+  float v[3],maxd,mind;
+  int ok = true;
+  /* DX named from their docs */
+
+  int stage = 0;
+
+  ObjectMapState *ms;
+
+  char cc[MAXLINELEN];
+
+
+  if(state<0) state=I->NState;
+  if(I->NState<=state) {
+    VLACheck(I->State,ObjectMapState,state);
+    I->NState=state+1;
+  }
+  ms=&I->State[state];
+  ObjectMapStateInit(ms);
+
+  ms->Origin=Alloc(float,3);
+  ms->Grid = Alloc(float,3);
+
+  maxd = FLT_MIN;
+  mind = FLT_MAX;
+  p=DXStr;
+
+  /* get the dimensions */
+
+  ms->FDim[3] = 3;
+
+  while(ok&&(*p)&&(stage==0)) {
+    p = ParseNCopy(cc,p,35);
+    if(strcmp(cc,"object 1 class gridpositions counts")==0) {
+      p = ParseWordCopy(cc,p,10);      
+      if(sscanf(cc,"%d",&ms->FDim[0])==1) {
+        p = ParseWordCopy(cc,p,10);      
+        if(sscanf(cc,"%d",&ms->FDim[1])==1) {
+          p = ParseWordCopy(cc,p,10);      
+          if(sscanf(cc,"%d",&ms->FDim[2])==1) {
+            stage = 1;
+          }
+        }
+      }
+    }
+    p = ParseNextLine(p);
+  }
+
+  if(ok&&(stage==1)) {
+    PRINTFB(FB_ObjectMap,FB_Details)
+      " DXStrToMap: Dimensions: %d %d %d\n",ms->FDim[0],ms->FDim[1],ms->FDim[2]
+      ENDFB;
+  }
+
+  /* get the origin */
+
+  while(ok&&(*p)&&(stage==1)) {  
+    p = ParseNCopy(cc,p,6);
+    if(strcmp(cc,"origin")==0) {
+      p = ParseWordCopy(cc,p,20);      
+      if(sscanf(cc,"%f",&ms->Origin[0])==1) {
+        p = ParseWordCopy(cc,p,20);      
+        if(sscanf(cc,"%f",&ms->Origin[1])==1) {
+          p = ParseWordCopy(cc,p,20);      
+          if(sscanf(cc,"%f",&ms->Origin[2])==1) {
+            stage = 2;
+          }
+        }
+      }
+    }
+    p = ParseNextLine(p);
+    
+  }
+
+  if(ok&&(stage==2)) {
+    PRINTFB(FB_ObjectMap,FB_Details)
+      " DXStrToMap: Origin %8.3f %8.3f %8.3f\n",ms->Origin[0],ms->Origin[1],ms->Origin[2]
+      ENDFB;
+  }
+
+
+  while(ok&&(*p)&&(stage==2)) {  
+    p = ParseNCopy(cc,p,5);
+    if(strcmp(cc,"delta")==0) {
+      p = ParseWordCopy(cc,p,20);      
+      if(sscanf(cc,"%f",&ms->Grid[0])==1) {
+        p = ParseNextLine(p);
+        p = ParseWordCopy(cc,p,20);      
+        p = ParseWordCopy(cc,p,20);      
+        p = ParseWordCopy(cc,p,20);      
+        if(sscanf(cc,"%f",&ms->Grid[1])==1) {
+        p = ParseNextLine(p);
+          p = ParseWordCopy(cc,p,20);      
+          p = ParseWordCopy(cc,p,20);      
+          p = ParseWordCopy(cc,p,20);      
+          p = ParseWordCopy(cc,p,20);      
+          if(sscanf(cc,"%f",&ms->Grid[2])==1) {
+            stage = 3;
+          }
+        }
+      }
+    }
+    p = ParseNextLine(p);
+    
+  }
+
+  if(ok&&(stage==3)) {
+    PRINTFB(FB_ObjectMap,FB_Details)
+      " DXStrToMap: Grid %8.3f %8.3f %8.3f\n",ms->Grid[0],ms->Grid[1],ms->Grid[2]
+      ENDFB;
+  }
+
+  while(ok&&(*p)&&(stage==3)) {  
+    p = ParseNCopy(cc,p,6);
+    if(strcmp(cc,"object")==0) {
+      p = ParseWordCopy(cc,p,20);      
+      p = ParseNTrim(cc,p,29);      
+      if(strcmp(cc,"class array type double rank")==0) {
+        p = ParseWordCopy(cc,p,20);      
+        p = ParseWordCopy(cc,p,20);      
+        p = ParseWordCopy(cc,p,20);      
+        if(sscanf(cc,"%d",&n_items)==1) {
+          if(n_items == ms->FDim[0]*ms->FDim[1]*ms->FDim[2])
+            stage = 4;
+        }
+      }
+    }
+    p = ParseNextLine(p);
+  }
+
+  if(stage == 4) {
+
+    if(ok&&(stage==4)) {
+      PRINTFB(FB_ObjectMap,FB_Details)
+        " DXStrToMap: %d data points.\n",n_items
+        ENDFB;
+    }
+    
+    ms->Field=IsosurfFieldAlloc(ms->FDim);
+    ms->MapSource = cMapSourcePHI;
+    ms->Field->save_points=false;
+    
+    for(a=0;a<3;a++) {
+      ms->Div[a] = ms->FDim[a] - 1;
+      ms->Min[a] = 0;
+      ms->Max[a] = ms->FDim[a] - 1;
+    }
+    
+    for(a=0;a<ms->FDim[0];a++) {
+      for(b=0;b<ms->FDim[1];b++) {
+        for(c=0;c<ms->FDim[2];c++) { 
+          
+          p = ParseWordCopy(cc,p,20);      
+          if(!cc[0]) {
+            p = ParseNextLine(p);           
+            p = ParseWordCopy(cc,p,20);      
+          }
+          if(sscanf(cc,"%f",&dens)==1) {
+            if(maxd<dens) maxd = dens;
+            if(mind>dens) mind = dens;
+            F3(ms->Field->data,a,b,c) = dens;
+          } else {
+            ok=false;
+          }
+        }
+      }
+
+    }
+    
+    for(e=0;e<3;e++) {
+      ms->ExtentMin[e] = ms->Origin[e]+ms->Grid[e]*ms->Min[e];
+      ms->ExtentMax[e] = ms->Origin[e]+ms->Grid[e]*ms->Max[e];
+    }
+    
+    for(c=0;c<ms->FDim[2];c++) {
+      v[2]=ms->Origin[2]+ms->Grid[2]*(c+ms->Min[2]);
+      for(b=0;b<ms->FDim[1];b++) {
+        v[1]=ms->Origin[1]+ms->Grid[1]*(b+ms->Min[1]);
+        for(a=0;a<ms->FDim[0];a++) {
+          v[0]=ms->Origin[0]+ms->Grid[0]*(a+ms->Min[0]);
+          for(e=0;e<3;e++) {
+            F4(ms->Field->points,a,b,c,e) = v[e];
+          }
+        }
+      }
+    }
+    
+    d=0;
+    for(c=0;c<ms->FDim[2];c+=ms->FDim[2]-1) {
+      v[2]=ms->Origin[2]+ms->Grid[2]*(c+ms->Min[2]);
+      
+      for(b=0;b<ms->FDim[1];b+=ms->FDim[1]-1) {
+        v[1]=ms->Origin[1]+ms->Grid[1]*(b+ms->Min[1]);
+        
+        for(a=0;a<ms->FDim[0];a+=ms->FDim[0]-1) {
+          v[0]=ms->Origin[0]+ms->Grid[0]*(a+ms->Min[0]);
+          copy3f(v,ms->Corner[d]);
+          d++;
+        }
+      }
+    }
+    if(ok)
+      stage = 5;
+  }
+
+  if(stage!=5)
+    ok=false;
+
+  if(!ok) {
+    ErrMessage("ObjectMap","Error reading map");
+  } else {
+    ms->Active=true;
+    ObjectMapUpdateExtents(I);
+    printf(" ObjectMap: Map Read.  Range = %5.6f to %5.6f\n",mind,maxd);
+  }
+  return(ok);
+}
+
+/*========================================================================*/
+static ObjectMap *ObjectMapReadDXStr(ObjectMap *I,char *MapStr,int bytes,int state)
+{
+  int ok=true;
+  int isNew = true;
+
+  if(!I) 
+	 isNew=true;
+  else 
+	 isNew=false;
+  if(ok) {
+	 if(isNew) {
+		I=(ObjectMap*)ObjectMapNew();
+		isNew = true;
+	 } else {
+		isNew = false;
+	 }
+    ObjectMapDXStrToMap(I,MapStr,bytes,state);
+    SceneChanged();
+    SceneCountFrames();
+  }
+  return(I);
+}
+
+/*========================================================================*/
+ObjectMap *ObjectMapLoadDXFile(ObjectMap *obj,char *fname,int state)
+{
+  ObjectMap *I = NULL;
+  int ok=true;
+  FILE *f;
+  long size;
+  char *buffer,*p;
+  float mat[9];
+
+  f=fopen(fname,"rb");
+  if(!f)
+	 ok=ErrMessage("ObjectMapLoadDXFile","Unable to open file!");
+  else
+	 {
+		if(Feedback(FB_ObjectMap,FB_Actions))
+		  {
+			printf(" ObjectMapLoadDXFile: Loading from '%s'.\n",fname);
+		  }
+		
+		fseek(f,0,SEEK_END);
+      size=ftell(f);
+		fseek(f,0,SEEK_SET);
+
+		buffer=(char*)mmalloc(size);
+		ErrChkPtr(buffer);
+		p=buffer;
+		fseek(f,0,SEEK_SET);
+		fread(p,size,1,f);
+		fclose(f);
+
+		I=ObjectMapReadDXStr(obj,buffer,size,state);
+
+		mfree(buffer);
+      if(state<0)
+        state=I->NState-1;
+      if(state<I->NState) {
+        ObjectMapState *ms;
+        ms = &I->State[state];
+        if(ms->Active) {
+          multiply33f33f(ms->Crystal->FracToReal,ms->Crystal->RealToFrac,mat);
+        }
+      }
+    }
+  return(I);
+
 }
 
 /*========================================================================*/
