@@ -57,6 +57,8 @@ typedef struct ObjectMapDesc { /* information for creating a new map */
 
 ObjectMap *ObjectMapNew(void);
 ObjectMapState *ObjectMapNewStateFromDesc(ObjectMap *I,ObjectMapDesc *md,int state);
+int ObjectMapStateGetExcludedStats(ObjectMapState *ms,float *vert_vla,
+                                   float beyond, float within, float *level);
 
 ObjectMap *ObjectMapLoadXPLORFile(ObjectMap *obj,char *fname,int state,int is_file);
 ObjectMap *ObjectMapReadXPLORStr(ObjectMap *I,char *XPLORStr,int state);
@@ -66,6 +68,7 @@ ObjectMap *ObjectMapLoadCCP4File(ObjectMap *obj,char *fname,int state);
 ObjectMap *ObjectMapReadCCP4Str(ObjectMap *I,char *XPLORStr,int bytes,int state);
 int ObjectMapCCP4StrToMap(ObjectMap *I,char *XPLORStr,int bytes,int state);
 
+ObjectMap *ObjectMapLoadDXFile(ObjectMap *obj,char *fname,int state);
 ObjectMap *ObjectMapLoadPHIFile(ObjectMap *obj,char *fname,int state);
 ObjectMap *ObjectMapLoadFLDFile(ObjectMap *obj,char *fname,int state);
 ObjectMap *ObjectMapLoadBRIXFile(ObjectMap *obj,char *fname,int state);
