@@ -337,6 +337,8 @@ static void ObjectMeshUpdate(ObjectMesh *I)
 
   int *n;
   float *v;
+  float carve_buffer;
+  int avoid_flag=false;
   int *old_n;
   float *old_v;
   int n_cur;
@@ -400,8 +402,14 @@ static void ObjectMeshUpdate(ObjectMesh *I)
           }
           if(ms->CarveFlag&&ms->AtomVertex&&
              VLAGetSize(ms->N)&&VLAGetSize(ms->V)) {
+            carve_buffer = ms->CarveBuffer;
+            if(ms->CarveBuffer<0.0F) {
+              avoid_flag=true;
+              carve_buffer = -carve_buffer;
+            }
+              
             /* cull my friend, cull */
-            voxelmap=MapNew(-ms->CarveBuffer,ms->AtomVertex,VLAGetSize(ms->AtomVertex)/3,NULL);
+            voxelmap=MapNew(-carve_buffer,ms->AtomVertex,VLAGetSize(ms->AtomVertex)/3,NULL);
             if(voxelmap) {
               
               MapSetupExpress(voxelmap);  
@@ -427,13 +435,15 @@ static void ObjectMeshUpdate(ObjectMesh *I)
                     if(i) {
                       j=voxelmap->EList[i++];
                       while(j>=0) {
-                        if(within3f(ms->AtomVertex+3*j,v,ms->CarveBuffer)) {
+                        if(within3f(ms->AtomVertex+3*j,v,carve_buffer)) {
                           flag=true;
                           break;
                         }
                         j=voxelmap->EList[i++];
                       }
                     }
+                    if(avoid_flag)
+                      flag = !flag;
                     if(flag&&(!last_flag)) {
                       VLACheck(ms->V,float,3*(n_line+1));
                       copy3f(v,ms->V+n_line*3);
@@ -479,6 +489,7 @@ static void ObjectMeshRender(ObjectMesh *I,int state,CRay *ray,Pickable **pick,i
 {
   float *v = NULL;
   float *vc;
+  float radius;
   int *n = NULL;
   int c;
   int a=0;
@@ -511,12 +522,19 @@ static void ObjectMeshRender(ObjectMesh *I,int state,CRay *ray,Pickable **pick,i
           if(ms->UnitCellCGO&&(I->Obj.RepVis[cRepCell]))
             CGORenderRay(ms->UnitCellCGO,ray,ColorGet(I->Obj.Color),
                          I->Obj.Setting,NULL);
-          ms->Radius=SettingGet_f(I->Obj.Setting,NULL,cSetting_mesh_radius);
-
-
-          if(ms->Radius==0.0F) {
-            ms->Radius = ray->PixelRadius*SettingGet_f(I->Obj.Setting,NULL,cSetting_mesh_width)/2.0F;
-          } 
+          if(!ms->DotFlag) {
+            radius=SettingGet_f(I->Obj.Setting,NULL,cSetting_mesh_radius);
+            
+            if(radius==0.0F) {
+              radius = ray->PixelRadius*SettingGet_f(I->Obj.Setting,NULL,cSetting_mesh_width)/2.0F;
+            } 
+          } else {
+            radius=SettingGet_f(I->Obj.Setting,NULL,cSetting_dot_radius);            
+            if(radius==0.0F) {
+              radius = ray->PixelRadius*SettingGet_f(I->Obj.Setting,NULL,cSetting_dot_width)/1.4142F;
+            } 
+          }
+                  
 
           if(n&&v&&I->Obj.RepVis[cRepMesh]) {
             vc = ColorGet(I->Obj.Color);
@@ -530,7 +548,7 @@ static void ObjectMeshRender(ObjectMesh *I,int state,CRay *ray,Pickable **pick,i
                       v+=3;
                       while(c--)
                         {
-                          ray->fSphere3fv(ray,v,ms->Radius);
+                          ray->fSphere3fv(ray,v,radius);
                           v+=3;
                         }
                     }
@@ -544,7 +562,7 @@ static void ObjectMeshRender(ObjectMesh *I,int state,CRay *ray,Pickable **pick,i
                       v+=3;
                       while(c--)
                         {
-                          ray->fSausage3fv(ray,v-3,v,ms->Radius,vc,vc);
+                          ray->fSausage3fv(ray,v-3,v,radius,vc,vc);
                           v+=3;
                         }
                     }
@@ -577,7 +595,10 @@ static void ObjectMeshRender(ObjectMesh *I,int state,CRay *ray,Pickable **pick,i
               }
 
             if(n&&v&&I->Obj.RepVis[cRepMesh]) {
-              glLineWidth(SettingGet_f(I->Obj.Setting,NULL,cSetting_mesh_width));
+              if(ms->DotFlag) 
+                glPointSize(SettingGet_f(I->Obj.Setting,NULL,cSetting_dot_width));
+              else
+                glLineWidth(SettingGet_f(I->Obj.Setting,NULL,cSetting_mesh_width));
               while(*n)
                 {
                   c=*(n++);
@@ -695,7 +716,7 @@ ObjectMesh *ObjectMeshFromBox(ObjectMesh *obj,ObjectMap *map,
     IsosurfGetRange(oms->Field,oms->Crystal,mn,mx,ms->Range);
     ms->ExtentFlag = true;
   }
-  if(carve>=0.0) {
+  if(carve!=0.0) {
     ms->CarveFlag=true;
     ms->CarveBuffer = carve;
     ms->AtomVertex = vert_vla;
