@@ -269,6 +269,70 @@ void EditorGetNextMultiatom(char *name)
 }
 
 /*========================================================================*/
+int EditorLogState(int pkresi)
+{
+  CEditor *I = &Editor;
+  if(SettingGet(cSetting_logging)) {
+
+    OrthoLineType buffer,buf1="None",buf2="None",buf3="None",buf4="None";
+    int pkbond = 1;
+    
+    if(!EditorActive()) {
+      PLog("edit",cPLog_pml);
+    } else {
+      int sele1,sele2,sele3,sele4;
+      ObjectMolecule *obj1=NULL,*obj2=NULL,*obj3=NULL,*obj4=NULL;
+      int index1,index2,index3,index4;
+      
+      sele1 = SelectorIndexByName(cEditorSele1);
+      sele2 = SelectorIndexByName(cEditorSele2);
+      sele3 = SelectorIndexByName(cEditorSele3);
+      sele4 = SelectorIndexByName(cEditorSele4);
+
+      obj1 = SelectorGetFastSingleAtomObjectIndex(sele1,&index1);
+      obj2 = SelectorGetFastSingleAtomObjectIndex(sele2,&index2);
+      obj3 = SelectorGetFastSingleAtomObjectIndex(sele3,&index3);
+      obj4 = SelectorGetFastSingleAtomObjectIndex(sele4,&index4);
+
+      if((sele1>=0) && (sele2>=0) && I->BondMode && obj1 && obj2) {
+
+        /* bond mode */
+        ObjectMoleculeGetAtomSeleLog(obj1,index1,buf1,true);
+        ObjectMoleculeGetAtomSeleLog(obj2,index2,buf2,true);
+        
+      } else {
+
+        /* atom mode */
+        pkbond = 0;
+
+        if(obj1) {
+          ObjectMoleculeGetAtomSeleLog(obj1,index1,buf1,true);
+        }
+
+        if(obj2) {
+          ObjectMoleculeGetAtomSeleLog(obj2,index2,buf2,true);
+        }
+
+        if(obj3) {
+          ObjectMoleculeGetAtomSeleLog(obj3,index3,buf3,true);
+        }
+
+        if(obj4) {
+          ObjectMoleculeGetAtomSeleLog(obj4,index4,buf4,true);
+        }
+      }
+  
+      sprintf(buffer,"cmd.edit(%s,%s,%s,%s,pkresi=%d,pkbond=%d)",
+              buf1,buf2,buf3,buf4,pkresi ? 1: 0, pkbond ? 1: 0);
+
+      PLog(buffer,cPLog_pym);
+
+    }
+  }
+  return 1;
+}
+/*========================================================================*/
+
 int EditorInvert(int quiet)
 {
   CEditor *I = &Editor;
@@ -544,7 +608,7 @@ int EditorIsAnActiveObject(ObjectMolecule *obj) {
   return false;
 }
 /*========================================================================*/
-void EditorCycleValence(void)
+void EditorCycleValence(int quiet)
 {
   CEditor *I = &Editor;
   int sele0,sele1;
@@ -567,7 +631,8 @@ void EditorCycleValence(void)
 }
 
 /*========================================================================*/
-void EditorAttach(char *elem,int geom,int valence,char *name)
+void EditorAttach(char *elem,int geom,int valence,
+                  char *name,int quiet)
 {
   int i0;
   int sele0,sele1;
@@ -590,7 +655,7 @@ void EditorAttach(char *elem,int geom,int valence,char *name)
         if(obj1) {
           if(obj0==obj1) {
             /* bond mode - behave like replace */
-            EditorReplace(elem,geom,valence,name);
+            EditorReplace(elem,geom,valence,name,quiet);
           }
         } else {
           /* atom mode */
@@ -611,7 +676,7 @@ void EditorAttach(char *elem,int geom,int valence,char *name)
   VLAFreeP(ai); /* safety */
 }
 /*========================================================================*/
-void EditorRemove(int hydrogen)
+void EditorRemove(int hydrogen,int quiet)
 {
   int sele0,sele1;
   int i0;
@@ -642,13 +707,13 @@ void EditorRemove(int hydrogen)
         if(SelectorGetFastSingleAtomObjectIndex(sele0,&i0)) {
           /* atom mode */
           if(i0>=0) {
-            ExecutiveRemoveAtoms(cEditorSele1);
+            ExecutiveRemoveAtoms(cEditorSele1,quiet);
             EditorInactivate();
           }
         }
 
         if(h_flag) {
-          ExecutiveRemoveAtoms(cEditorRemoveSele);
+          ExecutiveRemoveAtoms(cEditorRemoveSele,quiet);
           SelectorDelete(cEditorRemoveSele);
         }
 
@@ -658,7 +723,7 @@ void EditorRemove(int hydrogen)
 #undef cEditorRemoveSele
 }
 /*========================================================================*/
-void EditorHFill(void)
+void EditorHFill(int quiet)
 {
   int sele0,sele1;
   int i0;
@@ -679,11 +744,11 @@ void EditorHFill(void)
           sprintf(buffer,"((neighbor %s) and (elem h))",
                   cEditorSele1);
         SelectorGetTmp(buffer,s1);
-        ExecutiveRemoveAtoms(s1);
+        ExecutiveRemoveAtoms(s1,quiet);
         SelectorFreeTmp(s1);
         i0 = ObjectMoleculeGetAtomIndex(obj0,sele0); 
         obj0->AtomInfo[i0].chemFlag=false;
-        ExecutiveAddHydrogens(cEditorSele1);
+        ExecutiveAddHydrogens(cEditorSele1,quiet);
       }
       
       if(sele1>=0) {
@@ -695,18 +760,18 @@ void EditorHFill(void)
           sprintf(buffer,"((neighbor %s) and (elem h))",
                   cEditorSele2);
         SelectorGetTmp(buffer,s1);
-        ExecutiveRemoveAtoms(s1);
+        ExecutiveRemoveAtoms(s1,quiet);
         SelectorFreeTmp(s1);
         i0 = ObjectMoleculeGetAtomIndex(obj1,sele1); 
         obj1->AtomInfo[i0].chemFlag=false;
-        ExecutiveAddHydrogens(cEditorSele2);
+        ExecutiveAddHydrogens(cEditorSele2,quiet);
       }
     }
   }
   
 }
 /*========================================================================*/
-void EditorReplace(char *elem,int geom,int valence,char *name)
+void EditorReplace(char *elem,int geom,int valence,char *name,int quiet)
 {
   int i0;
   int sele0;
@@ -821,6 +886,70 @@ static void draw_bond(float *v0,float *v1)
   glEnd();
 
 }
+#if 0
+static void draw_dist(float *v0,float *v1)
+{
+  SceneResetNormal(true);
+  glLineWidth(SettingGet(cSetting_line_width)*2);
+  glBegin(GL_LINES);
+  glVertex3fv(v0);
+  glVertex3fv(v1);
+  glEnd();
+}
+
+static void draw_arc(float *v0,float *v1,float *v2) 
+{
+  int nEdge = (int)SettingGet(cSetting_stick_quality)/2;
+  if(nEdge>20)
+    nEdge=20;
+  if(nEdge<3)
+    nEdge=3;
+  /*
+  int a;
+  if(n<3) {n=3;}
+  for(a=0;a<=n;a++)
+	 {
+		x[a]=(float)cos(a*2*PI/n);
+		y[a]=(float)sin(a*2*PI/n);
+	 }
+  */
+  SceneResetNormal(true);
+  glLineWidth(SettingGet(cSetting_line_width)*2);
+  glBegin(GL_LINE_STRIP);
+  glVertex3fv(v0);
+  glVertex3fv(v1);
+  glVertex3fv(v2);
+  glEnd();
+
+}
+
+static void draw_torsion(float *v0,float *v1,float *v2,float *v3) 
+{
+  int nEdge = (int)SettingGet(cSetting_stick_quality)/2;
+  if(nEdge>20)
+    nEdge=20;
+  if(nEdge<3)
+    nEdge=3;
+  /*
+  int a;
+  if(n<3) {n=3;}
+  for(a=0;a<=n;a++)
+	 {
+		x[a]=(float)cos(a*2*PI/n);
+		y[a]=(float)sin(a*2*PI/n);
+	 }
+  */
+  SceneResetNormal(true);
+  glLineWidth(SettingGet(cSetting_line_width)*2);
+  glBegin(GL_LINE_STRIP);
+  glVertex3fv(v0);
+  glVertex3fv(v1);
+  glVertex3fv(v2);
+  glVertex3fv(v3);
+  glEnd();
+
+}
+#endif
 
 static void draw_globe(float *v2,int number)
 {
@@ -986,10 +1115,12 @@ void EditorRender(int state)
 {
   CEditor *I=&Editor;
   int sele1,sele2,sele3,sele4;
-  float v0[3],v1[3],v2[3];
+  float v0[3],v1[3];
+  float vp[12],*vv;
+  /*  int v_cnt;*/
   ObjectMolecule *obj1=NULL,*obj2=NULL,*obj3=NULL,*obj4=NULL;
   int index1,index2,index3,index4;
-
+  
   if(EditorActive()) {
 
     PRINTFD(FB_Editor)
@@ -1021,26 +1152,53 @@ void EditorRender(int state)
         
       } else {
         /* atom mode */
-        
+
+        vv = vp;
+
         if(obj1) {
-          if(ObjectMoleculeGetAtomVertex(obj1,state,index1,v2))
-            draw_globe(v2,1);
+          if(ObjectMoleculeGetAtomVertex(obj1,state,index1,vv)) {
+            draw_globe(vv,1);
+            vv+=3;
+          }
         }
 
         if(obj2) {
-          if(ObjectMoleculeGetAtomVertex(obj2,state,index2,v2))
-            draw_globe(v2,2);
+          if(ObjectMoleculeGetAtomVertex(obj2,state,index2,vv)) {
+            draw_globe(vv,2);
+            vv+=3;
+          }
         }
 
         if(obj3) {
-          if(ObjectMoleculeGetAtomVertex(obj3,state,index3,v2))
-            draw_globe(v2,3);
+          if(ObjectMoleculeGetAtomVertex(obj3,state,index3,vv)) {
+            draw_globe(vv,3);
+            vv+=3;
+          }
         }
 
         if(obj4) {
-          if(ObjectMoleculeGetAtomVertex(obj4,state,index4,v2))
-            draw_globe(v2,4);
+          if(ObjectMoleculeGetAtomVertex(obj4,state,index4,vv)) {
+            draw_globe(vv,4);
+            vv+=3;
+          }
         }
+
+#if 0        
+        v_cnt = (vv-vp)/3;
+        
+        switch(v_cnt) {
+        case 2:
+          draw_dist(vp,vp+3);
+          break;
+        case 3:
+          draw_arc(vp,vp+3,vp+6);
+          break;
+        case 4:
+          draw_torsion(vp,vp+3,vp+6,vp+9);
+          break;
+        }
+#endif
+
       }
       /*
         if(I->ShowFrags) {
@@ -1144,7 +1302,8 @@ void EditorPrepareDrag(ObjectMolecule *obj,int index,int state)
   int log_trans = (int)SettingGet(cSetting_log_conformations);
   int drag_sele = -1;
   PRINTFD(FB_Editor)
-    " EditorPrepareDrag-Debug: entered. obj %p index %d",obj,index
+    " EditorPrepareDrag-Debug: entered. obj %p index %d",
+    (void*)obj,index
     ENDFD;
 
   state = EditorGetEffectiveState(obj,state);
@@ -1173,7 +1332,7 @@ void EditorPrepareDrag(ObjectMolecule *obj,int index,int state)
     }
     if(seleFlag) { /* normal selection */
       
-      PRINTFB(FB_Editor,FB_Actions)
+      PRINTFB(FB_Editor,FB_Blather)
         " Editor: grabbing (%s).",name
         ENDFB;
 
@@ -1344,7 +1503,8 @@ void EditorPrepareDrag(ObjectMolecule *obj,int index,int state)
 
   PRINTFD(FB_Editor)
     " EditorPrepDrag-Debug: leaving Index %d Sele %d Object %p\n Axis %d Base %d BondFlag %d SlowFlag %d seleFlag %d\n",
-    I->DragIndex,I->DragSelection,I->DragObject,I->DragHaveAxis,I->DragHaveBase,
+    I->DragIndex,I->DragSelection,(void*)I->DragObject,
+    I->DragHaveAxis,I->DragHaveBase,
     I->DragBondFlag,I->DragSlowFlag,seleFlag
     ENDFD;
 }
@@ -1361,8 +1521,10 @@ void EditorDrag(ObjectMolecule *obj,int index,int mode,int state,
 
 
   PRINTFD(FB_Editor)
-    " EditorDrag-Debug: entered. obj %p state %d index %d mode %d \nIndex %d Sele %d Object %p\n Axis %d Base %d BondFlag %d SlowFlag %d\n", obj,state,index,mode,
-    I->DragIndex,I->DragSelection,I->DragObject,I->DragHaveAxis,I->DragHaveBase,
+    " EditorDrag-Debug: entered. obj %p state %d index %d mode %d \nIndex %d Sele %d Object %p\n Axis %d Base %d BondFlag %d SlowFlag %d\n", 
+    (void*)obj,state,index,mode,
+    I->DragIndex,I->DragSelection,(void*)I->DragObject,
+    I->DragHaveAxis,I->DragHaveBase,
     I->DragBondFlag,I->DragSlowFlag
     ENDFD;
 
