@@ -59,6 +59,8 @@
 #include"Wizard.h"
 #include"Text.h"
 #include"Character.h"
+#include"Seq.h"
+#include"Seeker.h"
 
 void MainFree(void);
 void MainTest(void);
@@ -66,6 +68,7 @@ void MainBusyIdle(void);
 static void MainInit(void);
 void MainReshape(int width, int height);
 static void MainDrawLocked(void);
+static void MainDrag(int x,int y);
 
 GLuint obj;
 
@@ -96,6 +99,7 @@ typedef struct {
   int IdleCount;
   int ReshapeFlag;
   int DragDirtyFlag;
+  int DragPassive;
 } CMain;
 
 static CMain Main;
@@ -275,6 +279,24 @@ int MainSceneCopy(int width,int height,int rowbytes,void *ptr)
   return result;
 }
 /*========================================================================*/
+void MainDoCommand(char *str1)
+{
+  PLockAPIAsGlut();
+  if(str1[0]!='_') { /* suppress internal call-backs */
+    if(strncmp(str1,"cmd._",5)) {
+      OrthoAddOutput("PyMOL>");
+      OrthoAddOutput(str1);
+      OrthoNewLine(NULL,true);
+    }
+    PDo(str1);
+  } else if(str1[1]==' ') { /* "_ command" suppresses echoing of command, but it is still logged */
+    PDo(str1+2);    
+  } else { 
+    PDo(str1);
+  }
+  PUnlockAPIAsGlut();
+}
+/*========================================================================*/
 void MainRunCommand(char *str1)
 {
   PLockAPIAsGlut();
@@ -288,10 +310,10 @@ void MainRunCommand(char *str1)
     }
     PParse(str1);
   } else if(str1[1]==' ') { /* "_ command" suppresses echoing of command, but it is still logged */
-    if(WordMatch(str1+2,"quit",true)==0) /* don't log quit */
+    if(WordMatch(str1+2,"quit",true)>=0) /* don't log quit */
       PLog(str1+2,cPLog_pml);
     PParse(str1+2);    
-  } else {
+  } else { 
     PParse(str1);
   }
   PUnlockAPIAsGlut();
@@ -340,7 +362,7 @@ void MainOnExit(void)
 #ifdef WIN32
 	TerminateProcess(GetCurrentProcess(),0); /* only way to avoid a crash */
 #endif
-    exit(0);
+    exit(EXIT_SUCCESS);
   }
 }
 /*========================================================================*/
@@ -372,6 +394,12 @@ void MainSwapBuffers(void)
   I->SwapFlag=true;
 }
 /*========================================================================*/
+void MainSetPassiveDrag(int onOrOff)
+{
+  CMain *I = &Main;
+  I->DragPassive = onOrOff;
+}
+/*========================================================================*/
 void MainTest(void)
 {
 }
@@ -379,29 +407,34 @@ void MainTest(void)
 static void MainButton(int button,int state,int x,int y)
 {
   static int glMod;  
-  /*  CMain *I = &Main;*/
+  CMain *I = &Main;
 
   glMod = p_glutGetModifiers();
 
   PLockAPIAsGlut();
 
-  /* stay blocked here because Clicks->SexFrame->PParse */
-
-  y=WinY-y;
-
-  Modifiers = ((glMod&P_GLUT_ACTIVE_SHIFT) ? cOrthoSHIFT : 0) |
-	 ((glMod&P_GLUT_ACTIVE_CTRL) ? cOrthoCTRL : 0) |
-	 ((glMod&P_GLUT_ACTIVE_ALT) ? cOrthoALT : 0);
-  
-  switch(button) {
-  case P_GLUT_BUTTON_SCROLL_FORWARD:
-  case P_GLUT_BUTTON_SCROLL_BACKWARD:
-    x=1;y=WinY-1; /* force into scene */
-    break;
-  }
-  if(!OrthoButton(button,state,x,y,Modifiers))
-    {
+  if(I->DragPassive) {
+    I->DragPassive = false;
+    MainDrag(x,y);
+  } else {
+    /* stay blocked here because Clicks->SexFrame->PParse */
+    
+    y=WinY-y;
+    
+    Modifiers = ((glMod&P_GLUT_ACTIVE_SHIFT) ? cOrthoSHIFT : 0) |
+      ((glMod&P_GLUT_ACTIVE_CTRL) ? cOrthoCTRL : 0) |
+      ((glMod&P_GLUT_ACTIVE_ALT) ? cOrthoALT : 0);
+    
+    switch(button) {
+    case P_GLUT_BUTTON_SCROLL_FORWARD:
+    case P_GLUT_BUTTON_SCROLL_BACKWARD:
+      x=WinX/2;y=WinY/2; /* force into scene */
+      break;
     }
+    if(!OrthoButton(button,state,x,y,Modifiers))
+      {
+      }
+  }
 
   PUnlockAPIAsGlut();
 
@@ -416,7 +449,7 @@ static void MainDrag(int x,int y)
   y=WinY-y;
   if(!OrthoDrag(x,y,Modifiers))
     {
-	 }
+    }
   
   if(I->DirtyFlag)
     if(PMGUI) {
@@ -427,6 +460,32 @@ static void MainDrag(int x,int y)
   PUnlockAPIAsGlut();
 
 }
+/*========================================================================*/
+static void MainPassive(int x,int y)
+{
+  CMain *I = &Main;
+
+  if(I->DragPassive) { /* a harmless race condition -- we don't want
+                          to slow Python down buy locking */
+
+    PLockAPIAsGlut();
+    
+    y=WinY-y;
+    if(!OrthoDrag(x,y,Modifiers))
+      {
+      }
+    
+    if(I->DirtyFlag)
+      if(PMGUI) {
+        p_glutPostRedisplay();
+        I->DirtyFlag=false;
+      }
+    
+    PUnlockAPIAsGlut();
+  }
+  
+}
+
 /*========================================================================*/
 static void MainDrawLocked(void)
 {
@@ -441,7 +500,7 @@ static void MainDrawLocked(void)
   ExecutiveDrawNow();
 
   if(PMGUI) {
-    if(Feedback(FB_Main,FB_Errors)) {
+    if(Feedback(FB_OpenGL,FB_Debugging)) {
       PyMOLCheckOpenGLErr("During Rendering");
     }
   }
@@ -489,7 +548,8 @@ static void MainKey(unsigned char k, int x, int y)
 	 ((glMod&P_GLUT_ACTIVE_CTRL) ? cOrthoCTRL : 0) |
 	 ((glMod&P_GLUT_ACTIVE_ALT) ? cOrthoALT : 0);
 
-  OrthoKey(k,x,y,Modifiers);
+  if(!WizardDoKey(k,x,y,Modifiers))
+    OrthoKey(k,x,y,Modifiers);
 
   PUnlockAPIAsGlut();
   
@@ -509,6 +569,9 @@ static void MainSpecial(int k, int x, int y)
 	 ((glMod&P_GLUT_ACTIVE_CTRL) ? cOrthoCTRL : 0) |
 	 ((glMod&P_GLUT_ACTIVE_ALT) ? cOrthoALT : 0);
 
+  if(!grabbed)
+    grabbed = WizardDoKey((unsigned char)k,x,y,Modifiers);
+
   switch(k) {
     case P_GLUT_KEY_UP:
     case P_GLUT_KEY_DOWN:
@@ -523,6 +586,7 @@ static void MainSpecial(int k, int x, int y)
       }
       break;
   }
+
   if(!grabbed) {
     sprintf(buffer,"_special %d,%d,%d,%d",k,x,y,Modifiers);
     PLog(buffer,cPLog_pml);
@@ -544,7 +608,8 @@ void MainReshape(int width, int height) /* called by Glut */
   h = ((float)height)/width;
   if(PMGUI) glViewport(0, 0, (GLint) width, (GLint) height);
   
-  OrthoReshape(width,height);
+  OrthoReshape(width,height,false);
+
 }
 /*========================================================================*/
 
@@ -554,6 +619,8 @@ PyObject *MainAsPyList(void)
   int width,height;
   result = PyList_New(2);
   BlockGetSize(SceneGetBlock(),&width,&height);
+  if(SettingGetGlobal_b(cSetting_seq_view)&&!SettingGetGlobal_b(cSetting_seq_view_overlay))
+    height+=SeqGetHeight();
   PyList_SetItem(result,0,PyInt_FromLong(width));
   PyList_SetItem(result,1,PyInt_FromLong(height));
   return(PConvAutoNone(result));
@@ -583,10 +650,13 @@ void MainDoReshape(int width, int height) /* called internally */
 {
   int h,w;
   int internal_feedback;
+  int force =false;
+
   if(width<0) {
     BlockGetSize(SceneGetBlock(),&width,&h);
-    if(SettingGet(cSetting_internal_gui))
-      width+=(int)SettingGet(cSetting_internal_gui_width);
+    if(SettingGetGlobal_b(cSetting_internal_gui))
+      width+=SettingGetGlobal_i(cSetting_internal_gui_width);
+    force = true;
   }
 
   if(height<0) { 
@@ -594,12 +664,15 @@ void MainDoReshape(int width, int height) /* called internally */
     internal_feedback = (int)SettingGet(cSetting_internal_feedback);
     if(internal_feedback)
       height+=(internal_feedback-1)*cOrthoLineHeight+cOrthoBottomSceneMargin;
+    if(SettingGetGlobal_b(cSetting_seq_view)&&!SettingGetGlobal_b(cSetting_seq_view_overlay))
+      height+=SeqGetHeight();
+    force = true;
   }
   if(PMGUI) {
     p_glutReshapeWindow(width,height);
     glViewport(0, 0, (GLint) width, (GLint) height);
   }
-  OrthoReshape(width,height);
+  OrthoReshape(width,height,force);
   if(SettingGet(cSetting_full_screen))
     p_glutFullScreen();
 
@@ -617,8 +690,10 @@ static void MainInit(void)
   I->IdleMode=2;
   I->IdleTime=(float)UtilGetSeconds();
   I->IdleCount = 0;
-  I->ReshapeFlag=false;
+  I->ReshapeFlag = false;
   I->DragDirtyFlag=0;
+  I->DragPassive = false;
+
   if(PMGUI) {
 
     /* get us into a well defined GL state */
@@ -649,6 +724,10 @@ static void MainInit(void)
   SelectorInit();
   MovieInit();
   SceneInit();
+  SeqInit();
+  SeekerInit();
+  ButModeInit();
+  ControlInit();
   SculptCacheInit();
   VFontInit();
   ExecutiveInit();
@@ -671,6 +750,10 @@ void MainFree(void)
   ExecutiveFree();
   VFontFree();
   SculptCacheFree();
+  ButModeFree();
+  ControlFree();
+  SeekerFree();
+  SeqFree();
   SceneFree();
   MovieFree();
   SelectorFree();
@@ -689,8 +772,26 @@ void MainFree(void)
   }
   
 #ifdef WIN32
+  if(PMGUI) p_glutDestroyWindow(TheWindow);
   TerminateProcess(GetCurrentProcess(),0); /* only way to avoid a crash */
 #endif
+#ifdef _PYMOL_OSX
+  if(PMGUI) {
+    if(GameMode) {
+      p_glutLeaveGameMode();
+      /* force a full-screen refresh to eliminate garbage on screen */
+      p_glutInitWindowPosition(0,0);
+      p_glutInitWindowSize(640,480);
+      p_glutInitDisplayMode(P_GLUT_RGBA | P_GLUT_DEPTH | P_GLUT_DOUBLE );            
+      if(p_glutGet(P_GLUT_DISPLAY_MODE_POSSIBLE)) {
+        TheWindow = p_glutCreateWindow("PyMOL Viewer");
+        p_glutFullScreen();
+        p_glutDestroyWindow(TheWindow);
+      }
+    } else 
+      p_glutDestroyWindow(TheWindow);
+  }
+#endif
 
 }
 /*========================================================================*/
@@ -717,6 +818,10 @@ void MainRefreshNow(void)
 }
 /*========================================================================*/
 
+#ifdef _PYMOL_SHARP3D
+static int Sharp3DLastWindowX = -1000;
+#endif
+
 void MainBusyIdle(void) 
 {
   /* This is one of the few places in the program where we can be sure 
@@ -726,10 +831,27 @@ void MainBusyIdle(void)
 
   CMain *I = &Main;
 
+
   PRINTFD(FB_Main)
     " MainBusyIdle: called.\n"
     ENDFD;
 
+#ifdef _PYMOL_SHARP3D
+// keep the window on even coordinates to preserve L/R stereo...
+ {
+   int x,y;
+   x = glutGet(GLUT_WINDOW_X);
+   if(x!=Sharp3DLastWindowX) {
+     Sharp3DLastWindowX=x;
+     if(x&0x1) {
+       y = glutGet(GLUT_WINDOW_Y);
+       glutPositionWindow(x-1,y);
+     }
+   }
+ }
+#endif
+
+
   /* flush command and output queues */
   
   /*  PRINTFD(FB_Main)
@@ -820,9 +942,16 @@ void MainBusyIdle(void)
         #endif*/
 
         PRunString("adapt_to_hardware()");
+
+        if(PyMOLOption->incentive_product) { /* perform incentive product initialization (if any) */
+          PyRun_SimpleString("try:\n   import ipymol\nexcept:\n   pass\n");
+        }
+
         PRunString("exec_deferred()");
 #ifdef _PYMOL_SHARP3D
-PParse("load $TUT/1hpv.pdb;hide;show sticks;show surface;set surface_color,white;set transparency,0.5;stereo on");
+        //PParse("load $TUT/1hpv.pdb;hide;show sticks;show surface;set surface_color,white;set transparency,0.5;stereo on");
+        PParse("stereo on");
+        //PParse("wizard demo,cartoon");
 #endif
         PUnblock();
       }
@@ -918,12 +1047,12 @@ SetConsoleCtrlHandler(
 
     case 0: /* default/autodetect (stereo on win/unix; mono on macs) */
 #ifdef _PYMOL_SHARP3D
-      WinX = 1024;
-      WinY = 768;
-      WinPX = 0;
-      WinPY = 0;
+      WinX = 794+220;
+      WinY = 547;
+      // WinPX = 0;
+      //WinPY = 0;
 
-      glutInitDisplayMode( P_GLUT_RGBA| P_GLUT_MULTISAMPLE| P_GLUT_DOUBLE| P_GLUT_STENCIL );
+      glutInitDisplayMode( P_GLUT_RGBA| P_GLUT_DOUBLE| P_GLUT_STENCIL );
       StereoCapable = 1;
 #else
 #ifndef _PYMOL_OSX
@@ -947,7 +1076,7 @@ SetConsoleCtrlHandler(
     case 1: /* force stereo (if possible) */
       p_glutInitDisplayMode(P_GLUT_RGBA | P_GLUT_DEPTH | P_GLUT_DOUBLE | P_GLUT_STEREO );
       if(!p_glutGet(P_GLUT_DISPLAY_MODE_POSSIBLE)) {
-        p_glutInitDisplayMode(P_GLUT_RGBA | P_GLUT_DEPTH | P_GLUT_DOUBLE );            
+
         StereoCapable = 0;
       } else {
         StereoCapable = 1;
@@ -964,7 +1093,7 @@ SetConsoleCtrlHandler(
         #endif
       }
       p_glutInitWindowSize(WinX, WinY);
-
+ 
       TheWindow = p_glutCreateWindow("PyMOL Viewer");
     } else {
       char str[255];
@@ -974,23 +1103,27 @@ SetConsoleCtrlHandler(
     }
   } 
 
-  #ifdef _PYMOL_SHARP3D
+#ifdef _PYMOL_SHARP3D
   	// Setup OpenGL
 	glEnable(GL_DEPTH_TEST);
 	glDepthFunc(GL_LESS);
-
-  #endif
+#endif
 
   MainInit();
   PInit();
 
+#ifdef _PYMOL_SHARP3D
+  SettingSetGlobal_b(cSetting_overlay,1);
+  SettingSetGlobal_b(cSetting_overlay_lines,1);
+#endif
+
   if(PMGUI) {
     p_glutDisplayFunc(         MainDraw );
     p_glutReshapeFunc(         MainReshape );
     p_glutKeyboardFunc(        MainKey );
     p_glutMouseFunc(           MainButton );
     p_glutMotionFunc(          MainDrag );
-    /*  p_glutPassiveMotionFunc(   MainMove );*/
+    p_glutPassiveMotionFunc(   MainPassive );
     p_glutSpecialFunc(         MainSpecial );
     p_glutIdleFunc(            MainBusyIdle );
 
@@ -1085,7 +1218,7 @@ int was_main(void)
   PyMOLMultisample = PyMOLOption->multisample; /* currently only used by MacOSX */
 
 #ifdef _PYMOL_SHARP3D
-  InternalGUI = 0;
+  //  InternalGUI = 0;
   InternalFeedback = 0;
   ShowSplash = 0;
 #endif
