@@ -84,16 +84,17 @@ if __name__=='pymol.cmd':
    #######################################################################
 
    file_ext_re= re.compile(string.join([
-      "\.pdb$|\.ent$|\.mol$|\.p5m$|",
-      r"\.PDB$|\.ENT$|\.MOL$|\.P5M$|",
-      r"\.mmod$|\.mmd$|\.dat$|\.out$|",
-      r"\.MMOD$|\.MMD$|\.DAT$|\.OUT$|",
-      r"\.xplor$|\.pkl$|\.sdf$|", 
-      r"\.XPLOR$|\.PKL$|\.SDF$|",                        
+      "\.pdb$|\.pdb1$|\.ent$|\.mol$|\.p5m$|",
+      r"\.PDB$|\.pdb1$|\.ENT$|\.MOL$|\.P5M$|",
+      r"\.mmod$|\.mmd$|\.dat$|\.out$|\.mol2$|",
+      r"\.MMOD$|\.MMD$|\.DAT$|\.OUT$|\.MOL2$|",
+      r"\.xplor$|\.pkl$|\.sdf$|\.pqr|", 
+      r"\.XPLOR$|\.PKL$|\.SDF$|\.PQR|",                        
       r"\.r3d$|\.xyz$|\.xyz_[0-9]*$|", 
       r"\.R3D$|\.XYZ$|\.XYZ_[0-9]*$|",
       r"\.cc1$|\.cc2$|", # ChemDraw 3D
       r"\.CC1$|\.CC2$|",
+      r"\.DX$|\.dx$|", # DX files (APBS)
       r"\.pse$|\.PSE$|", # PyMOL session (pickled dictionary)
       r"\.pmo$|", # Experimental molecular object format
       r"\.PMO$|",
@@ -149,11 +150,13 @@ if __name__=='pymol.cmd':
 
 
    palette_dict = {
-      'rainbow_cycle'           : ('s',3,0  ,999),
-      'rainbow_cycle_rev'       : ('s',3,999,  0),      
-      'rainbow'                 : ('s',3,167,833),
-      'rainbow_rev'             : ('s',3,833,167),
-
+      'rainbow_cycle'           : ('o',3,0  ,999), # perceptive rainbow
+      'rainbow_cycle_rev'       : ('o',3,999,  0),      
+      'rainbow'                 : ('o',3,107,893),
+      'rainbow_rev'             : ('o',3,893,107),
+      'rainbow2'                : ('s',3,167, 833), # cartesian rainbow 
+      'rainbow2_rev'            : ('s',3,833,167),
+      
       'gcbmry' : ('r',3,166,999),
       'yrmbcg' : ('r',3,999,166),
       
@@ -199,41 +202,41 @@ if __name__=='pymol.cmd':
       'red_white_cyan'          : ('w',3,916,999),
       'cyan_white_red'          : ('w',3,999,916),      
 
-      'yellow_blue'       : ('w',3,  0, 83),
-      'blue_yellow'       : ('w',3, 83,  0),      
+      'yellow_blue'       : ('c',3,  0, 83),
+      'blue_yellow'       : ('c',3, 83,  0),      
       
-      'blue_red'          : ('w',3, 83,167),
-      'red_blue'          : ('w',3,167, 83),
+      'blue_red'          : ('c',3, 83,167),
+      'red_blue'          : ('c',3,167, 83),
       
-      'red_green'         : ('w',3,167,250),
-      'green_red'         : ('w',3,250,167),
+      'red_green'         : ('c',3,167,250),
+      'green_red'         : ('c',3,250,167),
       
-      'green_magenta'     : ('w',3,250,333),
-      'magenta_green'     : ('w',3,333,250),      
+      'green_magenta'     : ('c',3,250,333),
+      'magenta_green'     : ('c',3,333,250),      
       
-      'magenta_cyan'      : ('w',3,333,417),
-      'cyan_magenta'      : ('w',3,417,333),
+      'magenta_cyan'      : ('c',3,333,417),
+      'cyan_magenta'      : ('c',3,417,333),
       
-      'cyan_yellow'       : ('w',3,417,500),
-      'yellow_cyan'       : ('w',3,500,417),
+      'cyan_yellow'       : ('c',3,417,500),
+      'yellow_cyan'       : ('c',3,500,417),
       
-      'yellow_green'      : ('w',3,500,583),
-      'green_yellow'      : ('w',3,583,500),
+      'yellow_green'      : ('c',3,500,583),
+      'green_yellow'      : ('c',3,583,500),
       
-      'green_blue'        : ('w',3,583,667),
-      'blue_green'        : ('w',3,667,583),      
+      'green_blue'        : ('c',3,583,667),
+      'blue_green'        : ('c',3,667,583),      
       
-      'blue_magenta'      : ('w',3,667,750),
-      'magenta_blue'      : ('w',3,750,667),
+      'blue_magenta'      : ('c',3,667,750),
+      'magenta_blue'      : ('c',3,750,667),
       
-      'magenta_yellow'    : ('w',3,750,833),
-      'yellow_magenta'    : ('w',3,833,750),
+      'magenta_yellow'    : ('c',3,750,833),
+      'yellow_magenta'    : ('c',3,833,750),
       
-      'yellow_red'        : ('w',3,833,917),
-      'red_yellow'        : ('w',3,817,833),
+      'yellow_red'        : ('c',3,833,917),
+      'red_yellow'        : ('c',3,817,833),
       
-      'red_cyan'          : ('w',3,916,999),
-      'cyan_red'          : ('w',3,999,916),      
+      'red_cyan'          : ('c',3,916,999),
+      'cyan_red'          : ('c',3,999,916),      
       }
 
    palette_sc = Shortcut(palette_dict.keys())
@@ -672,7 +675,7 @@ DEVELOPMENT TO DO
 
    # loading
 
-   def _load(oname,finfo,state,ftype,finish,discrete):
+   def _load(oname,finfo,state,ftype,finish,discrete,quiet=1):
       # WARNING: internal routine, subject to change
       # caller must already hold API lock
       # NOTE: state index assumes 1-based state
@@ -683,31 +686,31 @@ DEVELOPMENT TO DO
             obj = cgo.from_r3d(finfo)
             if obj:
                _cmd.load_object(str(oname),obj,int(state)-1,loadable.cgo,
-                                int(finish),int(discrete))
+                                int(finish),int(discrete),int(quiet))
             else:
                print " load: couldn't load raster3d file."
          elif ftype == loadable.cc1: # ChemDraw 3D
             obj = io.cc1.fromFile(finfo)
             if obj:
                _cmd.load_object(str(oname),obj,int(state)-1,loadable.model,
-                                int(finish),int(discrete))            
+                                int(finish),int(discrete),int(quiet))            
          else:
             r = _cmd.load(str(oname),finfo,int(state)-1,int(ftype),
-                          int(finish),int(discrete))
+                          int(finish),int(discrete),int(quiet))
       else:
          try:
             x = io.pkl.fromFile(finfo)
             if isinstance(x,types.ListType) or isinstance(x,types.TupleType):
                for a in x:
                   r = _cmd.load_object(str(oname),a,int(state)-1,
-                                       int(ftype),0,int(discrete))
+                                       int(ftype),0,int(discrete),int(quiet))
                   if(state>0):
                      state = state + 1
                _cmd.finish_object(str(oname))
             else:
                r = _cmd.load_object(str(oname),x,
                                     int(state)-1,int(ftype),
-                                    int(finish),int(discrete))
+                                    int(finish),int(discrete),int(quiet))
          except:
             print 'Error: can not load file "%s"' % finfo
       return r
@@ -776,6 +779,16 @@ DEVELOPMENT TO DO
       # WARNING: internal routine, subject to change
       try:
          lock()
+         try: # flush and close log if possible to avoid threading exception
+            if pymol._log_file!=None:
+               try:
+                  pymol._log_file.flush()
+               except:
+                  pass
+               pymol._log_file.close()
+               del pymol._log_file
+         except:
+            pass
          if reaper!=None:
             try:
                reaper.join()
@@ -1082,6 +1095,7 @@ SEE ALSO
 
    #--------------------------------------------------------------------
    from importing import \
+        finish_object,      \
         load,               \
         load_brick,         \
         load_callback,      \
@@ -1146,6 +1160,7 @@ SEE ALSO
         distance,           \
         export_dots,        \
         find_pairs,         \
+        get_angle,          \
         get_area,           \
         get_chains,         \
         get_color_indices,  \
@@ -1153,6 +1168,7 @@ SEE ALSO
         get_color_tuple,    \
         get_atom_coords,    \
         get_dihedral,       \
+        get_distance,       \
         get_extent,         \
         get_model,          \
         get_movie_locked,   \
@@ -1175,8 +1191,10 @@ SEE ALSO
    from selecting import \
         deselect,           \
         indicate,           \
-        select             
-
+        select,             \
+        select_list,        \
+        pop
+   
    #--------------------------------------------------------------------
    from exporting import \
         png,                \
@@ -1190,6 +1208,7 @@ SEE ALSO
    import editing
    from editing import \
         alter,              \
+        alter_list,         \
         alter_state,        \
         attach,             \
         bond,               \
@@ -1254,6 +1273,7 @@ SEE ALSO
         replace_wizard,     \
         set_wizard,         \
         set_wizard_stack,   \
+        dirty_wizard,       \
         wizard
 
    #--------------------------------------------------------------------
@@ -1275,7 +1295,9 @@ SEE ALSO
         publication
 
    #--------------------------------------------------------------------
+   import moving
    from moving import \
+        madd,              \
         mset,              \
         mclear,            \
         mdo,               \
@@ -1285,6 +1307,7 @@ SEE ALSO
         accept,            \
         decline,           \
         mpng,              \
+        mview,             \
         forward,           \
         backward,          \
         rewind,            \
@@ -1464,10 +1487,12 @@ SEE ALSO
       'full_screen'   : [ full_screen       , 0 , 0 , ''  , parsing.STRICT ],
       'fuse'          : [ fuse              , 0 , 0 , ''  , parsing.STRICT ],
       'frame'         : [ frame             , 0 , 0 , ''  , parsing.STRICT ],
-      'get'           : [ get               , 0 , 0 , ''  , parsing.STRICT ],      
+      'get'           : [ get               , 0 , 0 , ''  , parsing.STRICT ],
+      'get_angle'     : [ get_angle         , 0 , 0 , ''  , parsing.STRICT ],      
       'get_area'      : [ get_area          , 0 , 0 , ''  , parsing.STRICT ],
       'get_chains'    : [ get_chains        , 0 , 0 , ''  , parsing.STRICT ],
       'get_dihedral'  : [ get_dihedral      , 0 , 0 , ''  , parsing.STRICT ],
+      'get_distance'  : [ get_distance      , 0 , 0 , ''  , parsing.STRICT ],
       'get_extent'    : [ get_extent        , 0 , 0 , ''  , parsing.STRICT ],
       'get_position'  : [ get_position      , 0 , 0 , ''  , parsing.STRICT ],
       'get_symmetry'  : [ get_symmetry      , 0 , 0 , ''  , parsing.STRICT ],
@@ -1503,7 +1528,8 @@ SEE ALSO
       'log'           : [ log               , 0 , 0 , ''  , parsing.STRICT ],
       'log_close'     : [ log_close         , 0 , 0 , ''  , parsing.STRICT ],
       'log_open'      : [ log_open          , 0 , 0 , ''  , parsing.STRICT ],
-      'ls'            : [ ls                , 0 , 0 , ''  , parsing.STRICT ],  
+      'ls'            : [ ls                , 0 , 0 , ''  , parsing.STRICT ],
+      'madd'          : [ madd              , 0 , 0 , ''  , parsing.STRICT ],
       'mask'          : [ mask              , 0 , 0 , ''  , parsing.STRICT ],
       'map_set_border': [ map_set_border    , 0 , 0 , ''  , parsing.STRICT ],
       'map_double'    : [ map_double        , 0 , 0 , ''  , parsing.STRICT ],      
@@ -1524,13 +1550,15 @@ SEE ALSO
       'minimize'      : [ minimize          , 0 , 4 , ',' , parsing.SIMPLE ], # TO REMOVE
       'mmatrix'       : [ mmatrix           , 0 , 0 , ''  , parsing.STRICT ],
       'mouse'         : [ mouse             , 0 , 0 , ''  , parsing.STRICT ],
-      'multisave'     : [ multisave         , 0 , 0 , ''  , parsing.STRICT ],   
+      'multisave'     : [ multisave         , 0 , 0 , ''  , parsing.STRICT ],
+      'mview'         : [ mview             , 0 , 0 , ''  , parsing.STRICT ],
       'origin'        : [ origin            , 0 , 0 , ''  , parsing.STRICT ],
       'orient'        : [ orient            , 0 , 0 , ''  , parsing.STRICT ],
       'overlap'       : [ overlap           , 0 , 0 , ''  , parsing.STRICT ],
       'pair_fit'      : [ pair_fit          , 2 ,98 , ',' , parsing.SIMPLE ],
       'pass'          : [ python_help       , 0 , 0 , ''  , parsing.PYTHON ],
       'phi_psi'       : [ phi_psi           , 0 , 0 , ''  , parsing.STRICT ],
+      'pop'           : [ pop               , 0 , 0 , ''  , parsing.STRICT ],
       'protect'       : [ protect           , 0 , 0 , ''  , parsing.STRICT ],
       'push_undo'     : [ push_undo         , 0 , 0 , ''  , parsing.STRICT ],   
       'pwd'           : [ pwd               , 0 , 0 , ''  , parsing.STRICT ],
@@ -1628,6 +1656,7 @@ SEE ALSO
       'movie.load'    : [ movie.load        , 0 , 0 , ''  , parsing.STRICT ],
       'movie.zoom'    : [ movie.zoom        , 0 , 0 , ''  , parsing.STRICT ],
       'movie.screw'   : [ movie.screw       , 0 , 0 , ''  , parsing.STRICT ],
+      'movie.sweep'   : [ movie.sweep       , 0 , 0 , ''  , parsing.STRICT ],      
       'movie.nutate'  : [ movie.nutate      , 0 , 0 , ''  , parsing.STRICT ],
       'movie.tdroll'  : [ movie.tdroll      , 0 , 0 , ''  , parsing.STRICT ],
    # activate metaphorics extensions
@@ -1775,7 +1804,7 @@ SEE ALSO
       'A' : [ redo                   , () , {}],
       'B' : [ replace                , ('Br',1,1), {} ],
       'C' : [ replace                , ('C',4,4), {} ],
-      'D' : [ remove_picked          , () , {} ],
+      'D' : [ remove_picked          , () , {'quiet':0} ],
       'E' : [ invert                 , () , {'quiet':0} ],      
       'F' : [ replace                , ('F',1,1), {} ],   
       'G' : [ replace                , ('H',1,1), {} ],
@@ -1841,10 +1870,11 @@ SEE ALSO
    selection_sc = lambda sc=Shortcut,gn=get_names:sc(gn('public')+['all'])
    object_sc = lambda sc=Shortcut,gn=get_names:sc(gn('objects'))
    map_sc = lambda sc=Shortcut,gnot=get_names_of_type:sc(gnot('object:map'))
-
+   
    # Table for argument autocompletion
 
    auto_arg =[
+# 1st
       {
       'bg'             : [ _get_color_sc          , 'color'           , ''   ],      
       'color'          : [ _get_color_sc          , 'color'           , ', ' ],
@@ -1866,9 +1896,11 @@ SEE ALSO
       'zoom'           : [ selection_sc           , 'selection'       , ''   ],
       'origin'         : [ selection_sc           , 'selection'       , ''   ],
       'center'         : [ selection_sc           , 'selection'       , ''   ],   
+      'indicate'       : [ selection_sc           , 'selection'       , ''   ],   
       'protect'        : [ selection_sc           , 'selection'       , ''   ],
       'deprotect'      : [ selection_sc           , 'selection'       , ''   ],   
       'mask'           : [ selection_sc           , 'selection'       , ''   ],
+      'mview'          : [ moving.mview_action_sc , 'action'          , ''   ],
       'unmask'         : [ selection_sc           , 'selection'       , ''   ],
       'delete'         : [ selection_sc           , 'selection'       , ''   ],
       'alter'          : [ selection_sc           , 'selection'       , ''   ],
@@ -1878,8 +1910,10 @@ SEE ALSO
       'help'           : [ help_sc                , 'selection'       , ''   ],
       'unset'          : [ setting.setting_sc     , 'setting'         , ','  ],
       'view'           : [ viewing.view_dict_sc   , 'view'            , ''   ],                              
-      'scene'          : [ viewing.scene_dict_sc  , 'scene'           , ''   ],                        
+      'scene'          : [ viewing.scene_dict_sc  , 'scene'           , ''   ],
+      'ramp_new'       : [ object_sc              , 'ramp'            , ''   ],
       },
+# 2nd
       {
       'align'          : [ selection_sc           , 'selection'       , ''   ],
       'feedback'       : [ fb_module_sc           , 'module'          , ', ' ],
@@ -1901,8 +1935,10 @@ SEE ALSO
       'isosurface'     : [ map_sc                 , 'map object'      , ', ' ],
       'view'           : [ viewing.view_sc        , 'view action'     , ''   ],
       'scene'          : [ viewing.view_sc        , 'scene action'    , ','   ],                  
-      'unset'          : [ selection_sc           , 'selection'        , ','  ],   
+      'unset'          : [ selection_sc           , 'selection'        , ','  ],
+      'ramp_new'       : [ map_sc                 , 'map object'       , ''   ],      
       },
+#3rd
       {
       'spectrum'       : [ selection_sc           , 'selection'       , ''   ],
       'feedback'       : [ fb_mask_sc             , 'mask'            , ''   ],
@@ -1910,6 +1946,7 @@ SEE ALSO
       'flag'           : [ editing.flag_action_sc , 'flag action'     , ''   ],
       'set'            : [ selection_sc            , 'selection'         , ','  ],
       },
+#4th
       {
       'map_new'        : [ selection_sc           , 'selection'       , ', ' ],
       'isosurface'     : [ selection_sc           , 'selection'       , ', ' ],
