@@ -18,7 +18,8 @@ if __name__=='pymol.exporting':
    import selector
    import string
    import re
-
+   import copy
+   
    import pymol
    import cmd
    from cmd import _cmd,lock,unlock,Shortcut,QuietException
@@ -48,7 +49,7 @@ NOTES
       r = None
       try:
          lock()   
-         r = _cmd.get_pdb(str(selection),int(state)-1)
+         r = _cmd.get_pdb(str(selection),int(state)-1,0)
       finally:
          unlock()
       return r
@@ -63,6 +64,10 @@ NOTES
                r = _cmd.get_session(session)
             finally:
                unlock()
+            try:
+               session['session'] = copy.deepcopy(pymol.session)
+            except:
+               traceback.print_exc()
          else:
             try:
                r = apply(a,(session,))
@@ -89,7 +94,7 @@ PYMOL API
       if thread.get_ident() ==pymol.glutThread:
          r = cmd._png(str(filename),int(quiet))
       else:
-         r = _cmd.do("cmd._png('"+str(filename)+"')")
+         r = _cmd.do("cmd._png('"+str(filename)+"')",0)
       return r
 
    def export_coords(obj,state): # experimental
@@ -130,6 +135,12 @@ PYMOL API
 
    cmd.save(file, selection, state, format)
 
+NOTES
+
+   When saving a session file, then "state" has no effect.
+   When state = 0 (default), only the current state is written.
+   When state = -1, then a multi-state output file is written (PDB only).
+   
 SEE ALSO
 
    load, get_model
@@ -143,6 +154,8 @@ SEE ALSO
          lc_filename=string.lower(filename)
          if re.search("\.pdb$|\.ent$",lc_filename):
             format = 'pdb'
+         elif re.search("\.pqr$",lc_filename):
+            format = 'pqr'
          elif re.search("\.mol$",lc_filename):
             format = 'mol'
          elif re.search("\.pkl$",lc_filename):
@@ -163,12 +176,25 @@ SEE ALSO
          format = str(format)
       filename = os.path.expanduser(filename)
       filename = os.path.expandvars(filename)
-      if format=='pdb':
+      if format=='pdb': # standard PDB file 
          f=open(filename,"w")
          if f:
             try:
                lock()
-               st = _cmd.get_pdb("("+str(selection)+")",int(state)-1)
+               st = _cmd.get_pdb("("+str(selection)+")",int(state)-1,0)
+            finally:
+               unlock()
+               f.write(st)
+               f.close()
+            r = None
+            if not quiet:
+               print " Save: wrote \""+filename+"\"."
+      elif format=='pqr': # PQR (modified PDB file)
+         f=open(filename,"w")
+         if f:
+            try:
+               lock()
+               st = _cmd.get_pdb("("+str(selection)+")",int(state)-1,1)
             finally:
                unlock()
                f.write(st)
