@@ -31,6 +31,8 @@ if __name__=='pymol.importing':
    from chempy.sdf import SDF,SDFRec
    from chempy import io
    import pymol
+   import copy
+   import traceback
    
    class loadable:
       pdb = 0
@@ -61,6 +63,9 @@ if __name__=='pymol.importing':
       fld = 28      # AVS field format (not yet general -- just uniform allowed)
       brix = 29     # BRIX/DSN6/O map format
       grd = 30      # Insight II Grid format
+      pqr = 31      # PQR file (modified PDB file for APBS)
+      dx = 32       # DX file (APBS)
+      mol2 = 33     # MOL2 file (TRIPOS)
       
    loadable_sc = Shortcut(loadable.__dict__.keys()) 
 
@@ -74,6 +79,13 @@ if __name__=='pymol.importing':
                r = _cmd.set_session(session)
             finally:
                unlock()
+            try:
+               if session.has_key('session'):
+                  pymol.session = copy.deepcopy(session['session'])
+               else:
+                  pymol.session = pymol.Session_Storage()
+            except:
+               traceback.print_exc()
          else:
             r = apply(a,(session,))
          if not r: break
@@ -81,7 +93,7 @@ if __name__=='pymol.importing':
          cmd.wizard("security")
       return r
 
-   def load_object(type,object,name,state=0,finish=1,discrete=0):
+   def load_object(type,object,name,state=0,finish=1,discrete=0,quiet=1):
       '''
 DESCRIPTION
 
@@ -97,12 +109,13 @@ PYMOL API
    name = object name (string)
    finish = perform (1) or defer (0) post-processing of structure after load
    discrete = treat each state as an independent, unrelated set of atoms
+   quiet = suppress chatter (default is yes)
       '''
       r = 1
       try:
          lock()   
          r = _cmd.load_object(str(name),object,int(state)-1,
-                                 int(type),int(finish),int(discrete))
+                                 int(type),int(finish),int(discrete),int(quiet))
       finally:
          unlock()
       return r
@@ -327,7 +340,7 @@ SEE ALSO
 
 
 
-   def load(filename,object='',state=0,format='',finish=1,discrete=0):
+   def load(filename,object='',state=0,format='',finish=1,discrete=None,quiet=1):
       '''
 DESCRIPTION
 
@@ -365,7 +378,9 @@ NOTES
    'top' : AMBER topology file 'rst' : AMBER restart file
    'cex' : Metaphorics CEX format
    'pse' : PyMOL Session file
-
+   'pqr' : PQR (a modified PDB file with charges and radii)
+   'mol2' : MOL2
+   
 SEE ALSO
 
    save
@@ -377,14 +392,19 @@ SEE ALSO
          ftype = 0
          state = int(state)
          finish = int(finish)
-         discrete = int(discrete)
+         if discrete==None:
+            discrete_default = 1
+            discrete=0
+         else:
+            discrete_default = 0
+            discrete = int(discrete)
          fname = filename
          fname = os.path.expanduser(fname)
          fname = os.path.expandvars(fname)
 
          if not len(str(type)):
             # determine file type if possible
-            if re.search("\.pdb$|\.ent$|\.p5m",filename,re.I):
+            if re.search("\.pdb$|\.pdb1$|\.ent$|\.p5m",filename,re.I):
                ftype = loadable.pdb
             elif re.search("\.mol$",filename,re.I):
                ftype = loadable.mol
@@ -422,8 +442,14 @@ SEE ALSO
                ftype = loadable.pse
             elif re.search("\.phi$",filename,re.I):
                ftype = loadable.phi
+            elif re.search("\.mol2$",filename,re.I):
+               ftype = loadable.mol2
+            elif re.search("\.dx$",filename,re.I):
+               ftype = loadable.dx
             elif re.search("\.fld$",filename,re.I):
                ftype = loadable.fld
+            elif re.search("\.pqr$",filename,re.I):
+               ftype = loadable.pqr
             elif re.search("\.o$|\.dsn6$|\.brix$|\.omap$",filename,re.I):
                ftype = loadable.brix
             elif re.search("\.grd$",filename,re.I):
@@ -460,10 +486,11 @@ SEE ALSO
             while 1:
                rec = sdf.read()
                if not rec: break
-               r = _load(oname,string.join(rec.get('MOL'),''),state,ftype,0,1)
+               r = _load(oname,string.join(rec.get('MOL'),''),state,ftype,0,1,quiet)
             del sdf
             _cmd.finish_object(str(oname))
-            _cmd.do("zoom (%s)"%oname) 
+            _cmd.do("zoom (%s)"%oname,0)
+            _cmd.do("set seq_view_format,4,"+oname,0)
             ftype = -1
 
    # special handling for AMBER trj failes
@@ -485,15 +512,21 @@ SEE ALSO
             ftype = -1
             cmd.set_session(io.pkl.fromFile(fname))
 
+   # special handling for mol2 files
+
+         if ftype == loadable.mol2:
+            if discrete_default==1: # make mol2 files discrete by default
+               discrete = 1
+               
    # standard file handling
 
          if ftype>=0:
-            r = _load(oname,fname,state,ftype,finish,discrete)
+            r = _load(oname,fname,state,ftype,finish,discrete,quiet)
       finally:
          unlock()
       return r
 
-   def read_molstr(molstr,name,state=0,finish=1,discrete=1):
+   def read_molstr(molstr,name,state=0,finish=1,discrete=1,quiet=1):
       '''
 DESCRIPTION
 
@@ -520,12 +553,12 @@ NOTES
       try:
          lock()
          r = _cmd.load(str(name),str(molstr),int(state)-1,
-                       loadable.molstr,int(finish),int(discrete))
+                       loadable.molstr,int(finish),int(discrete),int(quiet))
       finally:
          unlock()
       return r
 
-   def read_mmodstr(*arg):
+   def read_mmodstr(*arg,**kw):
       '''
 DESCRIPTION
 
@@ -537,19 +570,23 @@ DESCRIPTION
       try:
          lock()   
          ftype = 6
+         if kw.has_key('quiet'):
+            quiet = int(kw['quiet'])
+         else:
+            quiet = 1
          if len(arg)==2:
             oname = string.strip(arg[1])
-            r = _cmd.load(str(oname),arg[0],-1,int(ftype),1,1)
+            r = _cmd.load(str(oname),arg[0],-1,int(ftype),1,1,quiet)
          elif len(arg)==3:
             oname = string.strip(arg[1])
-            r = _cmd.load(str(oname),arg[0],int(arg[2])-1,int(ftype),1,1)
+            r = _cmd.load(str(oname),arg[0],int(arg[2])-1,int(ftype),1,1,quiet)
          else:
             print "argument error."
       finally:
          unlock()
       return r
 
-   def read_pdbstr(pdb,name,state=0,finish=1,discrete=0):
+   def read_pdbstr(pdb,name,state=0,finish=1,discrete=0,quiet=1):
       '''
 DESCRIPTION
 
@@ -580,12 +617,12 @@ NOTES
          ftype = loadable.pdbstr
          oname = string.strip(str(name))
          r = _cmd.load(str(oname),pdb,int(state)-1,int(ftype),
-                          int(finish),int(discrete))
+                          int(finish),int(discrete),int(quiet))
       finally:
          unlock()
       return r
 
-   def read_xplorstr(xplor,name,state=0,finish=1,discrete=0):
+   def read_xplorstr(xplor,name,state=0,finish=1,discrete=0,quiet=1):
       '''
 DESCRIPTION
 
@@ -609,7 +646,7 @@ NOTES
          ftype = loadable.xplorstr
          oname = string.strip(str(name))
          r = _cmd.load(str(oname),xplor,int(state)-1,int(ftype),
-                          int(finish),int(discrete))
+                          int(finish),int(discrete),int(quiet))
       finally:
          unlock()
       return r
