@@ -246,10 +246,87 @@ def presets(s):
            [ 1, 'default'   ,'preset.default("'+s+'")'          ],           
            ]
 
+#def hydrogens(s):
+#   return [[ 2, 'Hydrogens:'       ,''                        ],     
+#           [ 1, 'add'   ,'cmd.h_add("'+s+'")'          ],           
+#           [ 1, 'remove'   ,'cmd.remove("('+s+') and hydro")'          ],
+#           ]
+
+def state(s):
+   return [[ 2, 'State:'       ,''                        ],
+           [ 1, 'freeze'  ,'cmd.set("state",cmd.get_state(),"'+s+'")'        ],
+           [ 1, 'thaw'  ,'cmd.set("state",0,"'+s+'")'        ],           
+           ]
+   
+def movement(s):
+   return [[ 2, 'Movement:'       ,''                        ],     
+           [ 1, 'protect'   ,'cmd.protect("'+s+'")'          ],
+           [ 1, 'deprotect'   ,'cmd.deprotect("'+s+'")'          ],           
+           ]
+   
+def selection(s):
+   return [[ 2, 'Selection:'       ,''                        ],     
+           [ 1, 'mask'   ,'cmd.mask("'+s+'")'          ],
+           [ 1, 'unmask'   ,'cmd.unmask("'+s+'")'          ],           
+           ]
+
+def compute(s):
+   return [[ 2, 'Compute:'       ,''                        ],     
+           [ 1, 'atom count'   ,'cmd.count_atoms("'+s+'",quiet=0)'          ],
+           [ 0, ''               ,''                             ],           
+           [ 1, 'formal charge sum'   ,'util.sum_formal_charges("'+s+'",quiet=0)'          ],
+           [ 1, 'partial charges sum'   ,'util.sum_partial_charges("'+s+'",quiet=0)'          ],                      
+           ]
+
+def vacuum(s):
+   return [[ 2, 'Vacuum Electrostatics:'       ,''                        ],
+           [ 2, '\\955WARNING:\\595 Unvalidated and experimental code!', '' ],
+           [ 1, 'protein surface potential (absolute)', 'util.protein_vacuum_esp("'+s+'",quiet=0)'          ],
+           [ 1, 'protein surface potential (relative)', 'util.protein_vacuum_esp("'+s+'",absolute=0,quiet=0)'          ],
+           [ 2, '\\955NOTE:\\559 Due to absence of solvent dielectric', ''],
+           [ 2, '\\559"screening", vacuum electrostatic potentials', ''],
+           [ 2, '\\559for macromolecules are only qualitatively', ''],
+           [ 2, '\\559meaningful.  Please view with skepticism!', '' ],
+           ]
+   
+def mol_assign(s):
+   return [[ 2, 'Assign:'       ,''                        ],     
+           [ 1, 'Amber 99 atomic properties',  'util.assign_amber99("'+s+'")' ],
+           ]
+   
+def mol_generate(s):
+   return [[ 2, 'Generate:'       ,''                        ],
+           [ 1, 'vacuum electrostatics', vacuum(s) ],           
+#           [ 1, 'assign', mol_assign(s) ],
+           ]
+   
+def invert(s):
+   return [[ 2, 'Invert:'       ,''                        ],     
+           [ 1, 'within object(s)'     ,'cmd.select("'+s+'","((byobj '+s+') and not '+s+')",show=1)'    ],
+           [ 1, 'within segments(s)'     ,'cmd.select("'+s+'","((byseg '+s+') and not '+s+')",show=1)'    ],           
+           [ 1, 'within chains(s)'     ,'cmd.select("'+s+'","((bychain '+s+') and not '+s+')",show=1)'    ],
+           [ 1, 'within residues(s)'   ,'cmd.select("'+s+'","((byres '+s+') and not '+s+')",show=1)'    ],                      
+           [ 0, ''               ,''                             ],
+           [ 1, 'within molecule(s)'     ,'cmd.select("'+s+'","((bymol '+s+') and not '+s+')",show=1)'    ],
+           [ 0, ''               ,''                             ],
+           [ 1, 'within any'     ,'cmd.select("'+s+'","(not '+s+')",show=1)'    ],
+           ]
+
+def complete(s):
+   return [[ 2, 'Complete:'       ,''                        ],     
+
+           [ 1, 'resides'  ,'cmd.select("'+s+'","(byres '+s+')",show=1)'      ],
+           [ 1, 'chains'  ,'cmd.select("'+s+'","(bychain '+s+')",show=1)'      ],
+           [ 1, 'segments'  ,'cmd.select("'+s+'","(byseg '+s+')",show=1)'      ],
+           [ 1, 'objects'  ,'cmd.select("'+s+'","(byobj '+s+')",show=1)'      ],
+           [ 0, ''               ,''                             ],           
+           [ 1, 'molecules'  ,'cmd.select("'+s+'","(bymol '+s+')",show=1)'      ],
+           [ 0, ''               ,''                             ],
+           [ 1, 'C-alphas'  ,'cmd.select("'+s+'","(bycalpha '+s+')",show=1)'      ],           
+           ]
+
 def expand(s):
    return [[ 2, 'Expand:'       ,''                        ],     
-           [ 1, 'complete resides'  ,'cmd.select("'+s+'","(byres '+s+')",show=1)'      ],
-           [ 0, ''               ,''                             ],           
            [ 1, 'by 4 A'  ,'cmd.select("'+s+'","('+s+' expand 4)",show=1)' ],
            [ 1, 'by 6 A'  ,'cmd.select("'+s+'","('+s+' expand 6)",show=1)' ],           
            [ 1, 'by 8 A'  ,'cmd.select("'+s+'","('+s+' expand 8)",show=1)' ],
@@ -268,14 +345,17 @@ def expand(s):
 def around(s):
    return [[ 2, 'Around:'       ,''                        ],     
            [ 1, 'atoms within 4 A'  ,'cmd.select("'+s+'","('+s+' around 4)",show=1)' ],
-           [ 1, 'atoms within 5 A'  ,'cmd.select("'+s+'","('+s+' around 5)",show=1)' ],           
            [ 1, 'atoms within 6 A'  ,'cmd.select("'+s+'","('+s+' around 6)",show=1)' ],           
            [ 1, 'atoms within 8 A'  ,'cmd.select("'+s+'","('+s+' around 8)",show=1)' ],
+           [ 1, 'atoms within 12 A'  ,'cmd.select("'+s+'","('+s+' around 12)",show=1)' ],
+           [ 1, 'atoms within 16 A'  ,'cmd.select("'+s+'","('+s+' around 16)",show=1)' ],
+           [ 1, 'atoms within 20 A'  ,'cmd.select("'+s+'","('+s+' around 20)",show=1)' ],                      
            [ 0, ''               ,''                             ],           
            [ 1, 'residues within 4 A'  ,'cmd.select("'+s+'","(byres ('+s+' around 4))",show=1)' ],
-           [ 1, 'residues within 5 A'  ,'cmd.select("'+s+'","(byres ('+s+' around 5))",show=1)' ],
            [ 1, 'residues within 6 A'  ,'cmd.select("'+s+'","(byres ('+s+' around 6))",show=1)' ],
-           [ 1, 'residues within 8 A'  ,'cmd.select("'+s+'","(byres ('+s+' around 8))",show=1)' ],           
+           [ 1, 'residues within 8 A'  ,'cmd.select("'+s+'","(byres ('+s+' around 8))",show=1)' ],
+           [ 1, 'residues within 16 A'  ,'cmd.select("'+s+'","(byres ('+s+' around 12))",show=1)' ],
+           [ 1, 'residues within 20 A'  ,'cmd.select("'+s+'","(byres ('+s+' around 20))",show=1)' ],                                 
            ]
    
 def extend(s):
@@ -298,6 +378,7 @@ def extend(s):
 def sele_action(s):
    return [[ 2, 'Actions:'       ,''                        ],     
            [ 1, 'delete selection', 'cmd.delete("'+s+'")'          ],
+           [ 1, 'rename selection', 'cmd.wizard("renaming","'+s+'")'          ],
            [ 0, ''               ,''                             ],
            [ 1, 'zoom'           ,'cmd.zoom("'+s+'")'            ],
            [ 1, 'center'         ,'cmd.center("'+s+'")'            ],           
@@ -306,27 +387,23 @@ def sele_action(s):
            [ 0, ''               ,''                             ],
            [ 1, 'preset'         ,presets(s)         ],
            [ 0, ''               ,''                             ],
-           [ 1, 'remove atoms'   ,'cmd.remove("'+s+'")'          ],
+           [ 1, 'remove atoms'   ,'cmd.remove("'+s+'");cmd.delete("'+s+'")'          ],
            [ 0, ''               ,''                             ],
+           [ 1, 'around'         , around(s)         ],           
+           [ 1, 'expand'         , expand(s)         ],
+           [ 1, 'extend'         , extend(s)         ],
+           [ 1, 'invert'         , invert(s)         ],
+           [ 1, 'complete'       , complete(s)         ],
+           [ 0, ''          ,''                                              ],
+           [ 1, 'duplicate selection'      ,'cmd.select("'+s+'")'          ],
+           [ 1, 'create object'  ,'cmd.create(None,"'+s+'")'     ],           
            [ 1, 'find polar contacts'  ,
              'cmd.dist("'+s+'_polar_conts","'+s+'","'+s+'",quiet=1,mode=2,labels=0)'
              ],                      
-           [ 0, ''               ,''                             ],
-           [ 1, 'expand'         , expand(s)         ],
-           [ 1, 'around'         , around(s)         ],           
-           [ 1, 'extend'         , extend(s)         ],           
-           [ 0, ''          ,''                                              ],
-           [ 1, 'invert selection'     ,'cmd.select("'+s+'","(not '+s+')",show=1)'    ],
-           [ 1, 'duplicate selection'      ,'cmd.select("'+s+'")'          ],
-           [ 1, 'create object'  ,'cmd.create(None,"'+s+'")'     ],           
            [ 0, ''          ,''                                  ],
-           [ 1, 'mask'           ,'cmd.mask("'+s+'")'            ],
-           [ 1, 'unmask'         ,'cmd.unmask("'+s+'")'          ],
-           [ 0, ''          ,''                                              ],
-           [ 1, 'protect'        ,'cmd.protect("'+s+'")'         ],
-           [ 1, 'deprotect'      ,'cmd.deprotect("'+s+'")'       ],
-           [ 0, ''          ,''                                              ],
-           [ 1, 'count atoms'    ,'cmd.count_atoms("'+s+'",quiet=0)'     ],           
+           [ 1, 'selection'      , selection(s)         ],           
+           [ 1, 'movement'       , movement(s)         ],
+           [ 1, 'compute'        , compute(s)         ],           
            ]
 
 
@@ -338,29 +415,26 @@ def mol_action(s):
            [ 1, 'orient'       , 'cmd.orient("'+s+'")'    ],
            [ 0, ''          ,''                                              ],
            [ 1, 'preset'  , presets(s)       ],
+           [ 0, ''          ,''                                              ],           
+           [ 1, 'generate'  , mol_generate(s)       ],           
            [ 0, ''               ,''                             ],
            [ 1, 'assign sec. struc.'  ,'cmd.dss("'+s+'")'        ],
            [ 1, 'find polar contacts'  ,
              'cmd.dist("'+s+'_polar_conts","'+s+'","'+s+'",quiet=1,mode=2,labels=0)'
              ],                      
-           [ 0, ''          ,''                                              ],
-           [ 1, 'freeze state'  ,'cmd.set("state",cmd.get_state(),"'+s+'")'        ],
-           [ 1, 'thaw state'  ,'cmd.set("state",0,"'+s+'")'        ],           
            [ 0, ''             , ''                       ],
+           [ 1, 'rename object', 'cmd.wizard("renaming","'+s+'")'          ],
+           [ 1, 'duplicate object'    ,'cmd.create(None,"'+s+'")'     ],           
            [ 1, 'delete object'       , 'cmd.delete("'+s+'")'    ],
-           [ 1, 'duplicate'       ,'cmd.create(None,"'+s+'")'     ],           
            [ 0, ''          ,''                                              ],
-           [ 1, 'add hydrogens' ,'cmd.h_add("'+s+'")'     ],           
-           [ 1, 'remove hydrogens'  ,'cmd.remove("(elem h and ('+s+'))")'     ],
-           [ 1, 'remove waters'  ,'cmd.remove("(resn HOH+WAT and ('+s+'))")'     ],           
+           [ 1, 'add hydrogens'  ,'cmd.h_add("'+s+'")'     ],           
+           [ 1, 'remove hydrogens'  ,'cmd.remove("(hydro and ('+s+'))")'     ],           
+           [ 1, 'remove waters'  ,'cmd.remove("(resn HOH+WAT and ('+s+'))")'     ],
            [ 0, ''          ,''                                              ],
-           [ 1, 'protect'  ,'cmd.protect("'+s+'")'        ],
-           [ 1, 'deprotect'  ,'cmd.deprotect("'+s+'")'        ],
-           [ 0, ''          ,''                                              ],
-           [ 1, 'mask'  ,'cmd.mask("'+s+'")'        ],
-           [ 1, 'unmask'  ,'cmd.unmask("'+s+'")'        ],
-           [ 0, ''          ,''                                              ],
-           [ 1, 'count atoms'  ,'cmd.count_atoms("'+s+'",quiet=0)'        ],
+           [ 1, 'state'      , state(s)         ],                      
+           [ 1, 'selection'      , selection(s)         ],           
+           [ 1, 'movement'       , movement(s)         ],           
+           [ 1, 'compute'        , compute(s)         ],
            ]
 
 def simple_action(s):
@@ -400,10 +474,9 @@ def all_action(s):
            [ 1, 'remove hydrogens'  ,'cmd.remove("(elem h and ('+s+'))")'     ],
            [ 1, 'remove waters'  ,'cmd.remove("(resn HOH+WAT and ('+s+'))")'     ],                      
            [ 0, ''          ,''                                              ],
-           [ 1, 'protect'        ,'cmd.protect("'+s+'")'         ],
-           [ 1, 'deprotect'      ,'cmd.deprotect("'+s+'")'       ],
-           [ 0, ''             , ''                      ],
-           [ 1, 'count atoms'       , 'cmd.count_atoms("all",quiet=0)'     ]
+           [ 1, 'selection'      , selection(s)         ],                      
+           [ 1, 'movement'       , movement(s)         ],
+           [ 1, 'compute'        , compute(s)         ],                      
            ]
 
 
@@ -424,7 +497,8 @@ def mol_labels(s):
            [ 1, 'segment identifier'       , 'cmd.label("'+s+'","segi")'         ],           
            [ 0, ''               , ''                                  ],
            [ 1, 'b-factor'       , 'cmd.label("'+s+'","\'%1.2f\'%b")'  ],
-           [ 1, 'occupancy'       , 'cmd.label("'+s+'","\'%1.2f\'%q")'  ],           
+           [ 1, 'occupancy'       , 'cmd.label("'+s+'","\'%1.2f\'%q")'  ],
+           [ 1, 'vdw radius'       , 'cmd.label("'+s+'","\'%1.2f\'%vdw")'  ],
            [ 0, ''               , ''                                  ],
            [ 1, 'partial charge(.2f)' ,            
   'cmd.label("'+s+'","\'%.2f\'%partial_charge")'                      ],
@@ -432,6 +506,7 @@ def mol_labels(s):
   'cmd.label("'+s+'","\'%.4f\'%partial_charge")'                      ],
            [ 1, 'formal charge' , 
   'cmd.label("'+s+'","\'%d\'%formal_charge")'                      ],
+           [ 1, 'bohr radius'       , 'cmd.label("'+s+'","\'%1.2f\'%bohr")'  ],                                 
            [ 0, ''               , ''                                  ],
            [ 1, 'text type'      , 'cmd.label("'+s+'","text_type")'    ],
            [ 1, 'numeric type'   , 'cmd.label("'+s+'","numeric_type")' ],
@@ -513,7 +588,7 @@ def pick_option(title,s,object=0):
       [ 1, 'orient'           ,'cmd.orient("'+s+'")'            ],
       [ 1, 'indicate'        ,'cmd.indicate("'+s+'")'            ],
       [ 0, ''             , ''                      ],
-      [ 1, 'labels'      , mol_labels(s) ],      
+      [ 1, 'labels'      , mol_labels(s) ],
       ]
    
    if object:
@@ -525,11 +600,18 @@ def pick_option(title,s,object=0):
          ])
    else:
       result.extend([
+      [ 0, ''             , ''                      ],      
+      [ 1, 'create object','cmd.create(None,"'+s+'")'            ],      
       [ 0, ''             , ''                      ],
       [ 1, 'remove'      , 'cmd.remove("'+s+'")' ],     
                     ])
    return result
 
+def pick_option_rev(title,s,object=0):
+   result = pick_option(title,s,object)[1:]
+   result.reverse()
+   return result
+
 def pick_menu(s1,s2):
    if s1[-1]=='`':
       title = s1[0:-1]
@@ -548,5 +630,57 @@ def pick_menu(s1,s2):
            [ 1, 'fragment+joint(s)', pick_option("Fragment","((byfrag ("+s2+")) extend 1)") ],
            ]
       
+def seq_menu(s2,s3):
+   
+   return [[ 2, 'Sequence'    , '' ],
+           [ 1, 'selection', pick_option('('+s3+')',s3) ],
+           [ 0, ''             , ''                      ],
+           [ 1, 'residue' , pick_option("Residue","(byres ("+s2+"))") ],
+           [ 1, 'chain'   , pick_option("Chain","(bychain ("+s2+"))") ],
+           [ 1, 'segment' , pick_option("Segment","(byseg ("+s2+"))") ],
+           [ 1, 'object'  , pick_option("Object","(byobject ("+s2+"))",1) ],
+           [ 0, ''             , ''                      ],
+           [ 1, 'molecule', pick_option("Molecule","(bymol ("+s2+"))") ],
+           [ 0, ''             , ''                      ],
+           [ 1, 'C-alpha'    , pick_option("C-alpha",s2) ],
+           ]
+      
 
+def seq_option(title,s,object=0):
+   c=len(title)-1
+   while title[c]!='/':
+      c = c-1
+   title = title[0:c+1]
+   
+   result = [
+      [ 2, title, '' ],
+      [ 1, 'color'      , mol_color(s) ],
+      [ 1, 'show'      , mol_show(s) ],
+      [ 1, 'hide'      , mol_hide(s) ],
+      [ 1, 'preset'  , presets(s)       ],      
+      [ 0, ''             , ''                      ],
+      [ 1, 'zoom'           ,'cmd.zoom("'+s+'")'            ],
+      [ 1, 'center'           ,'cmd.center("'+s+'")'            ],
+      [ 1, 'origin'           ,'cmd.origin("'+s+'")'            ],
+      [ 1, 'orient'           ,'cmd.orient("'+s+'")'            ],
+      [ 1, 'indicate'        ,'cmd.indicate("'+s+'")'            ],
+      [ 0, ''             , ''                      ],
+      [ 1, 'labels'      , mol_labels(s) ],
+      ]
+   
+   if object:
+      result.extend([
+         [ 0, ''             , ''                      ],         
+         [ 1, 'disable'        ,'cmd.disable("'+s+'")'            ],
+         [ 0, ''             , ''                      ],
+         [ 1, 'delete'        ,'cmd.delete("'+s+'")'            ]         
+         ])
+   else:
+      result.extend([
+      [ 0, ''             , ''                      ],      
+      [ 1, 'create object','cmd.create(None,"'+s+'")'            ],            
+      [ 0, ''             , ''                      ],
+      [ 1, 'remove'      , 'cmd.remove("'+s+'")' ],     
+                    ])
+   return result
    
