@@ -372,7 +372,40 @@ if __name__=='pymol.setting':
       dot_normals           = 332
       session_migration     = 333
       mesh_normals          = 334
-
+      mesh_type             = 335
+      dot_lighting          = 336
+      mesh_lighting         = 337
+      surface_solvent       = 338
+      triangle_max_passes   = 339
+      ray_interior_reflect  = 340
+      internal_gui_mode     = 341
+      surface_carve_selection = 342
+      surface_carve_state   = 343
+      surface_carve_cutoff  = 344
+      surface_clear_selection = 345
+      surface_clear_state   = 346
+      surface_clear_cutoff  = 347
+      surface_trim_cutoff   = 348
+      surface_trim_factor   = 349
+      ray_max_passes        = 350
+      active_selections     = 351
+      ray_transparency_contrast = 352
+      seq_view              = 353
+      mouse_selection_mode  = 354
+      seq_view_label_spacing = 355
+      seq_view_label_start = 356
+      seq_view_format      = 357
+      seq_view_location    = 358
+      seq_view_overlay     = 359
+      auto_classify_atoms  = 360
+      cartoon_nucleic_acid_mode = 361
+      seq_view_color       = 362
+      seq_view_label_mode  = 363
+      surface_ramp_above_mode = 364
+      stereo               = 365
+      wizard_prompt_mode   = 366
+      coulomb_cutoff       = 367
+      
    setting_sc = Shortcut(SettingIndex.__dict__.keys())
 
    index_list = []
@@ -498,7 +531,12 @@ NOTES
                   else:
                      v = (int(value),)
                elif type==3: # float
-                  v = (float(value),)
+                  if boolean_sc.has_key(str(value)):
+                     v = (float(boolean_dict[
+                        boolean_sc.auto_err(
+                        str(value),"boolean")]),)
+                  else:
+                     v = (float(value),)
                elif type==4: # float3 - some legacy handling req.
                   if is_string(value):
                      if not ',' in value:
